using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Remoting;
using ThoughtWorks.CruiseControl.Remote;
using Snak.CCNetScreenSaver.Properties;
using System.Threading;
using Snak.CCNetScreenSaver.Core.State;
using System.IO;
using System.Xml;

namespace Snak.CCNetScreenSaver.Core.CCNet
{
    internal class CCNetFacade
    {
        #region singleton implementation
                
        private volatile static CCNetFacade _instance;

        internal static CCNetFacade Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (typeof(CCNetFacade))
                    {
                        if (_instance == null)
                        {
                            _instance = CreateInstance();
                        }
                    }
                }
                return _instance;
            }
        }

        internal static void ClearInstance()
        {
            if (_instance != null)
            {
                lock (typeof(CCNetFacade))
                {
                    if (_instance != null)
                    {
                        _instance = null;
                    }
                }
            }
        }

        internal static void SetInstance(CCNetFacade instance)
        {
            lock (typeof(CCNetFacade))
            {
                _instance = instance;
            }
        }

        private static CCNetFacade CreateInstance()
        {
            CCNetFacade instance = new CCNetFacade();
            instance.SetCCNetServer(Settings.Default.CruiseControlRemotingInterfaceUri);
            return instance;
        }

        public bool CruiseManagerOnLine
        { 
            get
            {
                return (_cruiseManager != null);
            }
        }

        #endregion 

        private Timer pollingTimer;

        private ICruiseManager _cruiseManager;

        internal CCNetFacade() { }

        internal bool SetCCNetServer(string uri)
        {
            bool success = false;

            try
            {
                _cruiseManager = (ICruiseManager)Activator.GetObject(typeof(ICruiseManager), uri);
                // hit the remote object to make sure its there...
                string serverVersion = _cruiseManager.GetServerVersion();

                success = true;
            }
            catch (RemotingException)
            {
                _cruiseManager = null;
            }
            catch (System.Net.Sockets.SocketException)
            {
                _cruiseManager = null;
            }

            return success;
        }

        internal ProjectStatus[] GetProjectStatus()
        {
            return _cruiseManager.GetProjectStatus();
        }

        internal List<string> GetCCNetProjectsList()
        {
            List<string> projects = new List<string>();

            foreach (string projectName in this.GetCCNetProjects())
            {
                projects.Add(projectName);
            }

            return projects;
        }

        internal IEnumerable<string> GetCCNetProjects()
        {
            foreach (ProjectStatus projectStatus in GetProjectStatus())
            {
                yield return projectStatus.Name;
            }
        }

        private void PollingTimer_Elapsed(object args)
        {
            if (this.CruiseManagerOnLine)
            {
                foreach (ProjectStatus projectStatus in GetProjectStatus())
                {
                    if (projectStatus.Name == Settings.Default.ProjectToMonitor)
                    {
                        if (ScreenSaverState.Instance.CurrentBuildIsOld(projectStatus.LastBuildDate))
                            ScreenSaverState.Instance.UpdateBuildStatus(projectStatus.BuildStatus, ParseCCNetModificationsFileToGetPersonsWhoLastCheckedIn(), projectStatus.LastBuildDate);
                    }
                }
            }
            else
            {
                StopPollingBuildServer();
            }
        }

        internal void StartPollingBuildServer()
        {
            if (this.CruiseManagerOnLine && Settings.Default.ProjectToMonitor != String.Empty)
            {
                StopPollingBuildServer();
                // The method specified for callback should be reentrant (if it can be safely called recursively or from multiple processes),
                // because it is called on ThreadPool threads
                //
                // "Use a TimerCallback delegate to specify the method that is called by a Timer. This method does not execute in the 
                // thread that created the timer; it executes in a separate thread pool thread that is provided by the system. The 
                // TimerCallback delegate invokes the method once after the start time elapses, and continues to invoke it once per 
                // timer interval until the Dispose method is called, or until the Timer.Change method is called with the interval 
                // value Infinite." (source: http://msdn2.microsoft.com/en-US/library/system.threading.timercallback.aspx)
                pollingTimer = new Timer(new TimerCallback(PollingTimer_Elapsed), null, 0, Settings.Default.PollingInterval * 1000);
            }
        }

        internal void StopPollingBuildServer()
        {
            if (pollingTimer != null)
            {
                if (pollingTimer != null)
                    pollingTimer.Dispose();

                pollingTimer = null;
            }
        }

        private List<string> ParseCCNetModificationsFileToGetPersonsWhoLastCheckedIn()
        {
            List<string> personsWhoLastCheckedIn = new List<string>();

            if (Settings.Default.ModificationsFile != null && Settings.Default.ModificationsFile != String.Empty)
            {
                FileInfo modificationsFile = new FileInfo(Settings.Default.ModificationsFile);

                if (modificationsFile.Exists)
                {

                    try
                    {
                        XmlDocument document = new XmlDocument();
                        document.Load(modificationsFile.FullName);
                        XmlNodeList matchingNodes = document.SelectNodes("/ArrayOfModification/Modification/UserName");

                        foreach (XmlElement node in matchingNodes)
                        {
                            // initially i was trimming the domain here but on second thoughts 
                            // we'll leave that decision to something else a little later up the pipe

                            if (!personsWhoLastCheckedIn.Contains(node.InnerText))
                                personsWhoLastCheckedIn.Add(node.InnerText);
                        }
                    }
                    catch (XmlException ex)
                    {
                        throw new ApplicationException("Error trying to parse the modificationsFile, please check the inner exception", ex);
                    }
                }
            }

            return personsWhoLastCheckedIn;
        }
    }
}
