using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using ThoughtWorks.CruiseControl.Remote;

using Snak.CCNetScreenSaver.Properties;
using Snak.CCNetScreenSaver.Core.CCNet;

namespace Snak.CCNetScreenSaver.Core.State
{
    internal class ScreenSaverState
    {
        #region singleton implementation

        private volatile static ScreenSaverState _instance;

        internal static ScreenSaverState Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (typeof(ScreenSaverState))
                    {
                        if (_instance == null)
                        {
                            _instance = CreateInstance();
                        }
                    }
                }
                return _instance;
            }
        }

        internal static void ClearInstance()
        {
            if (_instance != null)
            {
                lock (typeof(ScreenSaverState))
                {
                    if (_instance != null)
                    {
                        _instance = null;
                    }
                }
            }
        }

        internal static void SetInstance(ScreenSaverState instance)
        {
            lock (typeof(ScreenSaverState))
            {
                _instance = instance;
            }
        }

        private static ScreenSaverState CreateInstance()
        {
            ScreenSaverState instance = new ScreenSaverState();
            instance.Load();
            return instance;
        }

        #endregion

        #region events 
        
        internal event EventHandler OnIntegrationStatusChanged;

        #endregion 

        #region configuration related state
        
        private bool _pendingConfigurationChanges = false;

        private string _cruiseControlRemotingInterfaceUri;
        private string _projectToMonitor;
        private string _modificationsFile;
        private string _urlOfExternalPageToDisplay;
        private int _pollingInterval;
        private PageGenerationMode _pageGenerationMode;

        internal string CruiseControlRemotingInterfaceUri
        {
            get 
            {
                return _cruiseControlRemotingInterfaceUri;
            }
            set 
            {
                _pendingConfigurationChanges = true;
                _cruiseControlRemotingInterfaceUri = CheckIfStringNullAndReturnStringEmptyIfTrue(value);
            }
        }

        internal string ProjectToMonitor
        {
            get
            {
                return _projectToMonitor;
            }
            set
            {
                _pendingConfigurationChanges = true;
                _projectToMonitor = CheckIfStringNullAndReturnStringEmptyIfTrue(value);
            }
        }

        internal string ModificationsFile
        {
            get
            {
                return _modificationsFile;
            }
            set
            {
                _pendingConfigurationChanges = true;
                _modificationsFile = CheckIfStringNullAndReturnStringEmptyIfTrue(value);
            }
        }

        internal string UrlOfExternalPageToDisplay
        {
            get
            {
                return _urlOfExternalPageToDisplay;
            }
            set
            {
                _pendingConfigurationChanges = true;
                _urlOfExternalPageToDisplay = CheckIfStringNullAndReturnStringEmptyIfTrue(value);
            }
        }

        internal int PollingInterval
        {
            get
            {
                return _pollingInterval;
            }
            set
            {
                _pendingConfigurationChanges = true;
                _pollingInterval = (value == 0) ? 30 : value;
            }
        }

        internal PageGenerationMode PageGenerationMode
        {
            get
            {
                return _pageGenerationMode;
            }
            set
            {
                _pendingConfigurationChanges = true;
                _pageGenerationMode = value;
            }
        }

        #endregion

        #region runtime related state

        private IntegrationStatus _integrationStatus = IntegrationStatus.Unknown;

        internal IntegrationStatus IntegrationStatus
        {
            get { return _integrationStatus; }
        }

        private List<string> _personsWhoBrokeBuild = null;

        internal List<string> PersonsWhoBrokeBuild
        {
            get { return _personsWhoBrokeBuild; }
        }

        private List<string> _personsWhoLastCheckedIn = null;

        internal List<string> PersonsWhoLastCheckedIn
        {
            get { return _personsWhoLastCheckedIn; }
        }

        #endregion 

        #region constructors

        private ScreenSaverState() 
        {
            Load();
        }

        #endregion 

        #region Integration status related

        private DateTime _lastBuildDate = DateTime.MinValue;

        internal bool CurrentBuildIsOld(DateTime buildToCheckAgainstLastBuildDate)
        {
            return buildToCheckAgainstLastBuildDate > _lastBuildDate;
        }

        internal void UpdateBuildStatus(IntegrationStatus newIntegrationStatus, List<string> personsWhoLastCheckedInCode, DateTime lastBuildDate)
        {
            if ((_integrationStatus == IntegrationStatus.Success || _integrationStatus == IntegrationStatus.Unknown) && (newIntegrationStatus == IntegrationStatus.Failure || newIntegrationStatus == IntegrationStatus.Exception))
            {
                // build was ok but now is broken
                _personsWhoBrokeBuild = personsWhoLastCheckedInCode;
            }
            else if (newIntegrationStatus == IntegrationStatus.Failure || newIntegrationStatus == IntegrationStatus.Exception)
            {
                // build was broken and is still broken (person who first brok it is at the front of the list)
                foreach (string person in personsWhoLastCheckedInCode)
                {
                    // if the person is already in the list, we'll just let them appear more than once
                    // it's funny if its the person that originally broke it cus we get an idea 
                    // of how may times they've tried fix it :)
                    _personsWhoBrokeBuild.Add(person);
                }
            }
            else if (newIntegrationStatus == IntegrationStatus.Success || newIntegrationStatus == IntegrationStatus.Unknown)
            {
                // build is ok or unknown
                _personsWhoBrokeBuild = null;
            }

            _personsWhoLastCheckedIn = personsWhoLastCheckedInCode;

            _integrationStatus = newIntegrationStatus;

            _lastBuildDate = lastBuildDate;

            if (OnIntegrationStatusChanged != null)
            {
                OnIntegrationStatusChanged(this, EventArgs.Empty);
            }
        }

        internal List<string> GetNamesToPublish()
        {
            List<string> namesToPublish;

            if (_integrationStatus == IntegrationStatus.Failure || _integrationStatus == IntegrationStatus.Exception)
                namesToPublish = _personsWhoBrokeBuild;
            else if (_integrationStatus == IntegrationStatus.Unknown)
            {
                namesToPublish = new List<string>();
                namesToPublish.Add(":(");
            }
            else
                namesToPublish = _personsWhoLastCheckedIn;

            return namesToPublish;
        }

        #endregion 

        #region state management (load/reload/reset)

        //
        // TODO: this is way to messy.. all these reset, set, reload... to many opps, 
        // internal Save, internal DiscardPendingChanges, private Reload, internal Reset

        internal void Save()
        {
            Settings.Default.CruiseControlRemotingInterfaceUri  = _cruiseControlRemotingInterfaceUri;
            Settings.Default.ProjectToMonitor = _projectToMonitor;
            Settings.Default.ModificationsFile = _modificationsFile;
            Settings.Default.UrlOfExternalPageToDisplay = _urlOfExternalPageToDisplay;
            Settings.Default.PollingInterval = _pollingInterval;
            Settings.Default.PageGenerationMode = _pageGenerationMode;
            Settings.Default.Save();
            
            _pendingConfigurationChanges = false;

            Load();
        }

        internal void Load()
        {
            _lastBuildDate = DateTime.MinValue;
            _personsWhoBrokeBuild = new List<string>();
            _personsWhoLastCheckedIn = new List<string>();

            Settings.Default.Reload();
            UpdateProperties();

            CCNetFacade.Instance.SetCCNetServer(_cruiseControlRemotingInterfaceUri);
        }

        internal void Reset()
        {
            _lastBuildDate = DateTime.MinValue;
            _personsWhoBrokeBuild = new List<string>();
            _personsWhoLastCheckedIn = new List<string>();

            Settings.Default.Reset();
            UpdateProperties();
        }

        internal void DiscardPendingChangesIfAny()
        {
            if (_pendingConfigurationChanges)
            {
                Load();
            }
        }

        private void UpdateProperties()
        {
            _cruiseControlRemotingInterfaceUri = Settings.Default.CruiseControlRemotingInterfaceUri;
            _projectToMonitor = Settings.Default.ProjectToMonitor;
            _modificationsFile = Settings.Default.ModificationsFile;
            _urlOfExternalPageToDisplay = Settings.Default.UrlOfExternalPageToDisplay;
            _pollingInterval = Settings.Default.PollingInterval;
            _pageGenerationMode = Settings.Default.PageGenerationMode;

            _pendingConfigurationChanges = false;
        }

        #endregion 

        #region utils

        private string CheckIfStringNullAndReturnStringEmptyIfTrue(string value)
        {
            return (value == null) ? String.Empty : value;
        }

        #endregion
    }
}
