﻿using System;
using System.Text;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Text.RegularExpressions;
using NUnit.Framework;

namespace Snak.ConfigurationTransformation.Tests
{
    [TestFixture]
    public class ConfigurationTransformerTest
    {
        [Test]
        public void TransformTest()
        {
            try
            {
                List<TransformationInfo> transformationInfos = TransformTestConfigFiles();

                foreach (TransformationInfo transformationInfo in transformationInfos)
                {
                    CheckTransformationOutput(transformationInfo);
                    Console.WriteLine(transformationInfo.PreProcessedFile + " successfully processed for environment target " + transformationInfo.Environment);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        private List<TransformationInfo> TransformTestConfigFiles()
        {
            ConfigurationTransformer configurationTransformer = new ConfigurationTransformer();

            string[] environments = new string[] { "buildVerification", "systemTest", "production", "support", "training", "dev" };
            List<TransformationInfo> transformationInfos = new List<TransformationInfo>();

            DirectoryInfo configDirectory = new DirectoryInfo(".\\TestConfigFiles\\ParserTests_Unprocessed");

            Directory.CreateDirectory(".\\ConfigurationTransformer_ProcessedOutput");

            foreach (FileInfo file in configDirectory.GetFiles("*.testing.config"))
            {
                string inputFileContent = File.ReadAllText(file.FullName);

                for (int i = 0; i < environments.Length; i++)
                {
                    // we need to call GetFullPath() beacuse we don’t really have control over where the base class writes the project file 
                    // we are generating to, any relative paths we specify will only be relative to that temporary location
                    string outFile = Path.GetFullPath(".\\ConfigurationTransformer_ProcessedOutput\\" + Path.GetFileNameWithoutExtension(file.FullName) + "-" + environments[i] + Path.GetExtension(file.FullName));

                    configurationTransformer.Transform(environments[i], file.FullName, outFile);

                    // store the information for this transformation in a list so we can later check the transformation output
                    transformationInfos.Add(new TransformationInfo(environments[i], file.FullName, outFile, inputFileContent.Contains(environments[i]), inputFileContent.Contains("#ENV:$(default)")));
                }
            }

            return transformationInfos;
        }

        private void CheckTransformationOutput(TransformationInfo transformationInfo)
        {
            StreamReader preProcessedFileStream = null;
            StreamReader processedFileStream = null;

            try
            {
                bool readerNotFinished = true;
                preProcessedFileStream = new StreamReader(transformationInfo.PreProcessedFile);
                processedFileStream = new StreamReader(transformationInfo.ProcessedFile);

                XmlReaderSettings xmlReaderSettings = new XmlReaderSettings();
                xmlReaderSettings.IgnoreComments = true;
                xmlReaderSettings.IgnoreWhitespace = true;
                xmlReaderSettings.IgnoreProcessingInstructions = true;

                XmlReader unprocessedFileStreamReader = XmlTextReader.Create(preProcessedFileStream);
                XmlReader processedFileStreamReader = XmlTextReader.Create(processedFileStream);

                // both the processed and unprocessed file should be the exact same, except for any values of the attribute specialAttributeToHelpTest,
                // this attributes value is that of the environment for which it is a target.
                // e.g. see the sample below, see how the value of the specialAttributeToHelpTest maps to the environment which it is under.

                //<!--  
                //#ENV:$(buildVerification) 
                //    <authentication specialAttributeToHelpTest="buildVerification" /> 
                //#ENV:$(systemTest)
                //    <authentication specialAttributeToHelpTest="systemTest" />  
                //#ENV:$(production)
                //    <authentication specialAttributeToHelpTest="production" /> 
                //#ENV:$(training)
                //    <authentication specialAttributeToHelpTest="training" /> 
                //#ENV:$(practice),$(support),$(UAT)    
                //    <authentication specialAttributeToHelpTest="practice,support,UAT" /> 
                //#ENV:$(dev) 
                //-->
                //    <authentication specialAttributeToHelpTest="dev" />
                //<!--
                //#END
                //-->

                // All we have to do is check the processed and unprocessed xml files and 
                // 1 -  Ensure they are structurally the same
                // 2 -  If the above condition is meet then ensure the value of specialAttributeToHelpTest 
                //      matches the environment for which the file was processed for. All the other constructs 
                //      should have been stripped out.

                bool success = false;

                string genericErrorMessage =
                            "An error occurred while comparing the processed output ('" + transformationInfo.ProcessedFile +
                            "') against the original file ('" + transformationInfo.PreProcessedFile + "'). ";

                while (readerNotFinished)
                {
                    unprocessedFileStreamReader.Read();
                    processedFileStreamReader.Read();

                    if (unprocessedFileStreamReader.Name == String.Empty)
                        unprocessedFileStreamReader.MoveToContent();
                    if (processedFileStreamReader.Name == String.Empty)
                        processedFileStreamReader.MoveToContent();

                    // make sure each node is the same name
                    if (String.Compare(unprocessedFileStreamReader.Name, processedFileStreamReader.Name, true) != 0 && ! transformationInfo.InputFileConainsDefaultEnvironmentMarkup)
                    {
                        // nodes may differ if there was no markup for the target environment, in such instances the entire 
                        // markup blocks are removed. However if there is a 'default' section then we wont end up in here.
                        //
                        // If we know there was markup for the target environment when we must fail at this point because the processed and 
                        // unprocessed files differ
                        if (transformationInfo.InputFileConainsDesiredEnvironment)
                            Assert.Fail(genericErrorMessage +
                                " The element named '" +
                                unprocessedFileStreamReader.Name + "' in the original file should exist at the same location in the " +
                                " processed file however an element called '" + processedFileStreamReader.Name + "' was found"
                                );
                        else
                        {
                            // Else we move the unprocessedFileStreamReader reader until it's out of the markup block, after that point
                            // the 2 file should be back in sync. We know every element in the markup block has a specialAttributeToHelpTest
                            // attribute so here we just read until we dont see that anymore...
                            // progress the reader until we pass all the elements that contain the preProcessedFileElementSuffix suffix
                            while (unprocessedFileStreamReader.Read())
                            {
                                if (unprocessedFileStreamReader.Name == String.Empty)
                                    unprocessedFileStreamReader.MoveToContent();

                                if (unprocessedFileStreamReader.GetAttribute("specialAttributeToHelpTest") == null)
                                    break;
                            }
                        }
                    }
                    else
                    {
                        // do this element have a specialAttributeToHelpTest attribute?
                        if (unprocessedFileStreamReader.GetAttribute("specialAttributeToHelpTest") != null)
                        {
                            // since we found an processedFileStreamReader attribute in the unprocessed file we should expect 
                            // the same in the processed file, however the processed file should have a value that maps 
                            // to the environment we are targeting
                            if (processedFileStreamReader.GetAttribute("specialAttributeToHelpTest") == null)
                                Assert.Fail(genericErrorMessage + " The specialAttributeToHelpTest attribute was not found on the element '" + processedFileStreamReader.Name + "' in the processed file");

                            string expected_environment = String.Empty;

                            if (!transformationInfo.InputFileConainsDesiredEnvironment && transformationInfo.InputFileConainsDefaultEnvironmentMarkup)
                            {
                                // If we have no specific markup for the target environment but the markup has a default section, 
                                // e.g. when running the following markup against target a sysTest target we should expect all but the 
                                // default section to remain (if there was no default section then all markup would be striped out)
                                //  <!--  
                                //  #ENV:$(buildVerification) 
                                //      <authentication specialAttributeToHelpTest="buildVerification" /> 
                                //  #ENV:$(production)    
                                //      <authentication specialAttributeToHelpTest="production" /> 
                                //  #ENV:$(dev,support,training) 
                                //  -->
                                //      <authentication specialAttributeToHelpTest="dev,support,training" />
                                //  <!--
                                //  #ENV:$(default)
                                //      <authentication specialAttributeToHelpTest="default" mode="Windows" />    	   
                                //  #END
                                //  -->

                                expected_environment = "default";
                            }
                            else
                            {
                                expected_environment = transformationInfo.Environment;
                            }

                            // make sure its value contains the target environment string
                            if (!Regex.Match(processedFileStreamReader.GetAttribute("specialAttributeToHelpTest"), expected_environment, RegexOptions.IgnoreCase).Success)
                                Assert.Fail(genericErrorMessage + " The target environment '" + transformationInfo.Environment + "' is not contained in the processed files specialAttributeToHelpTest attribute. The current value of  the attribute is " + processedFileStreamReader.GetAttribute("specialAttributeToHelpTest"));

                            success = true;
                        }
                    }

                    if (unprocessedFileStreamReader.EOF || processedFileStreamReader.EOF)
                        readerNotFinished = false;
                }

                if (transformationInfo.InputFileConainsDesiredEnvironment)
                    Assert.IsTrue(success, genericErrorMessage + " Could not verify files match");
            }
            finally
            {
                if (preProcessedFileStream != null)
                {
                    preProcessedFileStream.Dispose();
                    preProcessedFileStream.Close();
                }

                if (processedFileStream != null)
                {
                    processedFileStream.Dispose();
                    processedFileStream.Close();
                }
            }
        }
    }
}
