using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

using Snak.Core;
using Snak.BuildDrop;
using Tests.NAnt.Contrib;
using NUnit.Framework;
using NAnt.VSNet.Types;
using System.Xml;
using NAnt.Core;


namespace Snak.FunctionalTests
{
    public abstract class AbstractBuildRunnerTester : BuildTestBase
    {
        private FileInfo _solutionFile = null;
        private ISolutionInfo _solution = null;
        private DotNetFrameworkVersion _dotNetFrameworkVersion = DotNetFrameworkVersion.v2_0;
        private DirectoryInfo _dropDir = null;
        private BuildDrop.BuildDrop _buildDrop = null;

        protected FileInfo SolutionFile
        {
            get { return _solutionFile; }
        }

        protected DotNetFrameworkVersion DotNetFrameworkVersion
        {
            get { return _dotNetFrameworkVersion; }
        }

        protected DirectoryInfo DropDir
        {
            get { return _dropDir; }
        }

        protected BuildDrop.BuildDrop BuildDrop
        {
            get { return _buildDrop; }
        }

        protected ISolutionInfo Solution
        {
            get { return _solution; }
        }

        public abstract Dictionary<string, WebMap> WebMaps { get; }

        public virtual string VirDirServerName
        {
            get
            {
                return (String.Empty);
            }
        }
        
        /// <summary>
        /// provides common functionality to test the DeploySolutionTask
        /// </summary>
        /// <param name="relativePathtoSolution"></param>
        /// <param name="relativePathtoDropDir"></param>
        /// <param name="dotNetFrameworkVersion"></param>
        public AbstractBuildRunnerTester(string relativePathtoSolution, string relativePathtoDropDir, DotNetFrameworkVersion dotNetFrameworkVersion)
        {
            if (dotNetFrameworkVersion == DotNetFrameworkVersion.v1_0)
                throw new NotSupportedException();

            this._solutionFile = new FileInfo(Path.GetFullPath(relativePathtoSolution));
            if (!_solutionFile.Exists)
                // Can't Assert.Fail() as inside test constructor - best to throw
                throw new FileNotFoundException(string.Format("Failed to locate {0}: did you GET the QuickStarts?", relativePathtoSolution), relativePathtoSolution);
            
            this._dropDir = new DirectoryInfo(relativePathtoDropDir);
            this._dotNetFrameworkVersion = dotNetFrameworkVersion;
            _solution = SolutionFactory.GetSolution(_solutionFile, Log);

            if (this.WebMaps != null)
            {
                foreach (KeyValuePair<string, WebMap> keyValuePair in this.WebMaps)
                {
                    _solution.WebMaps.Add(keyValuePair.Value);
                }
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="buildConfiguration"></param>
        /// <param name="platform"></param>
        /// <returns></returns>
        /// <remarks>
        /// SNAK currently doesnt deal with the concept relating to the separation of the build configuration and platform
        /// e.g. it takes the build configuration as a whole: "Release|Any CPU". We make a distinction in the tests 
        /// because we use the same code to test both v1.1 and 2.0 apps. v1.1. doesnt have a platform value.
        /// </remarks>
        protected string GetCorrectBuildConfigString(string buildConfiguration, string platform)
        {
            string returnValue = buildConfiguration;

            if (platform != String.Empty)
            {
                buildConfiguration += "|";
                buildConfiguration += platform;
            }

            return buildConfiguration;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="buildDrop"></param>
        protected bool CheckDeployedFilesExistInBothCommonAndCurrentDropDirectories(BuildDrop.BuildDrop buildDrop)
        {
            //
            // In here we want to check that labeled drop dir has the same dir structure (including files) as the common drop dir.
            // We want to do this for the drop dirs that are common throughout the solution and also for the projects that get their own drop dir.
            //

            bool success = true;

            // Solution drop dirs: check that the labeled drop dir has the same dir structure (including files) as the common drop dir
            success = CheckDirectoryHasSameFilesAndSubDirectories(buildDrop.CommonAssembliesDropLocation.DropDir, buildDrop.CommonAssembliesDropLocation.DropDirCurrent);

            if (success)
            {
                // project that get their own drop dirs: check that the labeled drop dir has the same dir structure (including files) as the common drop dir
                foreach (KeyValuePair<string, DropLocation> keyValuePair in buildDrop.DropLocationsByProjectName)
                {
                    // we check that the root directory has files
                    // we should really do it for all but some may be meant to be empty, for now just check the root 
                    if (keyValuePair.Value.DropDir.GetFiles().Length == 0)
                    {
                        success = false;
                        break;
                    }
                    else if (keyValuePair.Value.DropDirCurrent.GetFiles().Length == 0)
                    {
                        success = false;
                        break;
                    }

                    // this next method recursively checks the directories to ensure they are the same
                    CheckDirectoryHasSameFilesAndSubDirectories(keyValuePair.Value.DropDir, keyValuePair.Value.DropDirCurrent);
                }
            }

            return success;
        }

        /// <summary>
        /// recursively checks the directories to ensure they are the same (checks on file name and file length)
        /// </summary>
        /// <param name="expectedDirStructure"></param>
        /// <param name="actualDirStructure"></param>
        /// <returns></returns>
        protected bool CheckDirectoryHasSameFilesAndSubDirectories(DirectoryInfo expectedDirStructure, DirectoryInfo actualDirStructure)
        {
            bool success = true;

            Dictionary<string, FileInfo> filesInExpectedDirStructure = new Dictionary<string, FileInfo>();
            Dictionary<string, DirectoryInfo> subDirsInExpectedDirStructure = new Dictionary<string, DirectoryInfo>();

            // build a dictionary of files that exist in the expected dir structure, keyed it on file name
            foreach (FileInfo file in expectedDirStructure.GetFiles())
            {
                filesInExpectedDirStructure.Add(file.Name, file);
            }

            // build a dictionary of sub directories that exist in the expected dir structure, keyed it on dir name
            foreach (DirectoryInfo subDirectory in expectedDirStructure.GetDirectories())
            {
                subDirsInExpectedDirStructure.Add(subDirectory.Name, subDirectory);
            }

            // check that the files in the actual directory structure are in our dictionary of expected file 
            // this leaves a little gap in that there may be more files in actualDirStructure than expectedDirStructure
            // but at this point were not to worries about that
            foreach (FileInfo file in actualDirStructure.GetFiles())
            {
                if (filesInExpectedDirStructure.ContainsKey(file.Name))
                {
                    // check the lenght just to be safe
                    if (filesInExpectedDirStructure[file.Name].Length != file.Length)
                    {
                        success = false;
                        break;
                    }
                }
                else
                {
                    success = false;
                    break;
                }
            }

            if (success)
            {
                // now we check that the sub directorieis in the actual directory structure are in our dictionary of expected sub directories 
                foreach (DirectoryInfo subDirectory in actualDirStructure.GetDirectories())
                {
                    if (subDirsInExpectedDirStructure.ContainsKey(subDirectory.Name))
                    {
                        // recursively check that the sub directories are the same
                        if (!CheckDirectoryHasSameFilesAndSubDirectories(subDirsInExpectedDirStructure[subDirectory.Name], subDirectory))
                        {
                            success = false;
                            break;
                        }
                    }
                    else
                    {
                        success = false;
                        break;
                    }
                }
            }

            return success;
        }

        protected void WriteBuildFileToDisk(string testName, string contents)
        {
            //string language = "C#";

            // hack:
            if (this.Solution.SolutionFile.FullName.Contains("VB"))
            {
                //language = "VB";
            }

            string buildFilePath = String.Format(
                            @"c:\temp\SNAKTestBuildFiles\{0}.{1}.{2}.build",
                            this.GetType().Name,
                            testName,
                            Path.GetFileNameWithoutExtension(this.Solution.SolutionFile.Name)
                        );

            new DirectoryInfo(@"c:\temp\SNAKTestBuildFiles").Create();

            try
            {
                // could just write out contents to disk but we'll use an XmlTextWriter to get some decent formatting
                XmlWriterSettings xmlWriterSettings = new XmlWriterSettings();
                xmlWriterSettings.Indent = true;
                xmlWriterSettings.NewLineOnAttributes = true;

                using (XmlWriter writer = XmlTextWriter.Create(buildFilePath, xmlWriterSettings))
                {
                    XmlDocument document = new XmlDocument();
                    document.LoadXml(contents);
                    document.PreserveWhitespace = false;
                    document.WriteTo(writer);
                }
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Error trying to write the build file '" + buildFilePath + "'. The test application will only try write the dynamically created build files when the test application setting WriteDynamicallyCreatedBuildFilesToDisk is true.", ex);
            }
        }

        private void Log(LogLevel level, string message, params object[] args) { }
    }
}
