using System;
using System.Collections.Generic;
using System.Text;
using Snak.Core;
using NAnt.VSNet.Types;
using System.IO;
using Snak.BuildDrop;
using NUnit.Framework;

namespace Snak.FunctionalTests
{
    public abstract class AbstractPackageOutputsTaskTests : AbstractBuildRunnerTester
    {
        public abstract override Dictionary<string, WebMap> WebMaps { get; }

        public AbstractPackageOutputsTaskTests(string relativePathtoSolution, string relativePathtoDropDir, DotNetFrameworkVersion dotNetFrameworkVersion)
            : base(relativePathtoSolution, relativePathtoDropDir, dotNetFrameworkVersion)
        {

        }

        public void BuildDeployAndCheckDeployment(string testName, string buildConfig)
        {
            BuildDeployAndCheckDeployment(testName, buildConfig, String.Empty);
        }

        public void BuildDeployAndCheckDeployment(string testName, string buildConfiguration, string platform)
        {
            try
            {
                string projectXml = CreateProjetXmlSnippet(buildConfiguration, platform, testName);

                IDropNamingStrategy packageNamingStrategy = new LabelDirectoryDropNamingStrategy(this.Solution.SolutionFile.Name, this.DropDir, new BuildVersion("65"), GetCorrectBuildConfigString(buildConfiguration, platform));
                BuildDrop.BuildDrop buildDrop = new BuildDrop.BuildDrop(packageNamingStrategy, this.Solution, GetCorrectBuildConfigString(buildConfiguration, platform));
                // delete anything that exists from the last test run
                buildDrop.DeleteDropDirectores();

                string consoleOutput = RunBuild(projectXml);

                if (!CheckDeployedFilesExistInBothCommonAndCurrentDropDirectories(buildDrop))
                {
                    Assert.Fail("Error on deployment. Either no files were deployed to the target folders or else the labeled drop folder does not match the 'current' drop folder.");
                }

                // check that the directory structure for the Build_65 and the Build_Current drop folders are the same
                bool currentAndLabeledAreSame =  this.CheckDirectoryHasSameFilesAndSubDirectories(packageNamingStrategy.DropDirectoryCurrent, packageNamingStrategy.DropDirectory);
                Assert.IsTrue(currentAndLabeledAreSame, "The current and labeled dropdirectories are not the same"); Assert.IsTrue(currentAndLabeledAreSame, "The current and labelled dropdirectories are not the same");

                // make sure the additional packages also got dropped
                // we want to check the for the following packages 
                // these were generated in the CreateProjetXmlSnippet method below
                //      AdditionalPackage1 (a dir with files)
                //      AdditionalPackage2 (a dir with files and sub dirs)
                //      AdditionalFileToPackage1 (a files)
                //      AdditionalFileToPackage2 (a files)


                // Below we use the packageNamingStrategy to get the expected drop locations for the packages.
                // We asume packageNamingStrategy is tested elsewhere and is safe to use to look up the 
                // drop locations for the packages, internally the package task also uses this method, however were not 
                // testing that the naming is correct but rather that the package task correctly copied file across.

                // AdditionalPackage1
                DirectoryInfo initialAdditionalPackage1Location = new DirectoryInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\AdditionalPackage1");
                DirectoryInfo expectedAdditionalPackage1Location = packageNamingStrategy.GetPackageDropDirectory("AdditionalPackage1", true, false);
                bool additionalPackage1DropedCorrectly = this.CheckDirectoryHasSameFilesAndSubDirectories(initialAdditionalPackage1Location, expectedAdditionalPackage1Location);
                Assert.IsTrue(additionalPackage1DropedCorrectly, "The additional package did not get packaged as expected"); 

                // AdditionalPackage2
                DirectoryInfo initialAdditionalPackage2Location = new DirectoryInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\AdditionalPackage2");
                DirectoryInfo expectedAdditionalPackage2Location = packageNamingStrategy.GetPackageDropDirectory("AdditionalPackage2", true, false);
                bool additionalPackage2DropedCorrectly = this.CheckDirectoryHasSameFilesAndSubDirectories(initialAdditionalPackage2Location, expectedAdditionalPackage2Location);
                Assert.IsTrue(additionalPackage1DropedCorrectly, "The additional package did not get packaged as expected"); 

                //AdditionalFileToPackage1 
                // the package named AdditionalFileToPackage1 contained one file and that should have went into the main DropDirectory,
                // given this so the following line should retrieve this file.
                FileInfo[] foundFiles = packageNamingStrategy.DropDirectory.GetFiles("AdditionalFileToPackage1.bat");
                Assert.IsTrue(foundFiles != null && foundFiles.Length == 1, "Could not verify that the package that contained the single file AdditionalFileToPackage1.bat was packaged with the build drop correctly.");

                // AdditionalFileToPackage2
                // the package named AdditionalFileToPackage2 contained one file and that should have went into the main DropDirectory,
                // given this so the following line should retrieve this file.
                FileInfo[] foundFiles2 = packageNamingStrategy.DropDirectory.GetFiles("AdditionalFileToPackage2.bat");
                Assert.IsTrue(foundFiles2 != null && foundFiles2.Length == 1, "Could not verify that the package that contained the single file AdditionalFileToPackage2.bat was packaged with the build drop correctly.");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        protected string CreateProjetXmlSnippet(string buildConfiguration, string platform, string testName)
        {
            StringBuilder solutionXMLSnippet = new StringBuilder();
            StringBuilder webMappings = new StringBuilder();

            IProjectInfo[] projects = this.Solution.GetProjectsFor(GetCorrectBuildConfigString(buildConfiguration, platform));

            string buildTask = String.Empty;

            if (this.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1)
            {
                solutionXMLSnippet.Append(@"<solution configuration=""");
                solutionXMLSnippet.Append(buildConfiguration);
                solutionXMLSnippet.Append(@""" solutionfile=""");
                solutionXMLSnippet.Append(this.SolutionFile.FullName);
                solutionXMLSnippet.Append(@""">");

                webMappings.Append(@"<webmap>");

                foreach (IProjectInfo project in projects)
                {
                    if (project.IsWebProject)
                    {
                        webMappings.Append(@"<map url=""");
                        webMappings.Append(this.WebMaps[project.Name].Url);
                        webMappings.Append(@""" path=""");
                        webMappings.Append(this.WebMaps[project.Name].Path);
                        webMappings.Append(@""" />");
                    }
                }

                webMappings.Append(@"</webmap>");
                solutionXMLSnippet.Append(webMappings.ToString());
                solutionXMLSnippet.Append("</solution>");

                buildTask = solutionXMLSnippet.ToString();
            }
            else if (this.DotNetFrameworkVersion == DotNetFrameworkVersion.v2_0)
            {
                // /p:Configuration=Debug /p:Platform="Any CPU"
                buildTask = @"  <msbuild project=""" + this.SolutionFile.FullName + @""">
                                    <property name=""Configuration"" value=""" + buildConfiguration + @""" />
                                    <property name=""Platform"" value=""" + platform + @""" />
                                </msbuild>";
            }

            string xmlsnippet = String.Format(
                                        @"<project name=""myTestProject"">
                                            {0}
                                            <loadtasks assembly=""{1}"" />
                                            {2}
                                            <packageOutputs 
                                                solution=""{3}"" 
                                                config=""{4}"" 
                                                artifactsDir=""{5}""
                                                buildLabel=""65""
                                                property=""packageOutputsTask""
                                            >
                                                <additionalPackage>
                                                    <package name=""AdditionalPackage1"" toDir=""AdditionalPackage1"">
                                                      <fileset basedir=""{6}\AdditionalPackage1"">
                                                        <include name=""**\*"" />
                                                      </fileset>
                                                    </package>
                                                    <package name=""AdditionalPackage2"" toDir=""AdditionalPackage2"">
                                                      <fileset basedir=""{6}\AdditionalPackage2"">
                                                        <include name=""**\*"" />
                                                      </fileset>
                                                    </package>
                                                    <package name=""CommonPackage"" toDir=""CommonPackage"">
                                                      <fileset basedir=""{6}\..\Common\CommonPackage"">
                                                        <include name=""**\*"" />
                                                      </fileset>
                                                    </package>
                                                    <package name=""AdditionalFileToPackage1"" file=""{6}\AdditionalFileToPackage1.bat"" toFile=""AdditionalFileToPackage1.bat""/>
                                                    <package name=""AdditionalFileToPackage2"" file=""{6}\AdditionalFileToPackage2.bat"" toFile=""AdditionalFileToPackage2.bat""/>
                                                </additionalPackage>
                                                {7}
                                            </packageOutputs>
                                        </project>",
                                    // tell nant which .net framework to compile with
                                    ((this.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1) ? @"<property name=""nant.settings.currentframework"" value=""net-1.1"" />" : @"<property name=""nant.settings.currentframework"" value=""net-2.0"" />"),
                                    // GetFullPath will return the rooted path to where ever this the test is running from, as it turns out 
                                    // this is different from where the base class writes this xml and loads it into nant, hence we need the full path to Snak.dll
                                    Path.GetFullPath("Snak.dll"),
                                    buildTask,
                                    this.SolutionFile.FullName,
                                    this.GetCorrectBuildConfigString(buildConfiguration, platform),
                                    this.DropDir.FullName,
                                    Path.GetFullPath(this.SolutionFile.DirectoryName),
                                    webMappings.ToString()
                                );

            // do we want to write the temp project file for later use (e.g. as an example reference)
            if (Snak.FunctionalTests.Properties.Settings.Default.WriteDynamicallyCreatedBuildFilesToDisk)
            {
                WriteBuildFileToDisk(testName, xmlsnippet);
            }

            return xmlsnippet;
        }
    }
}
