using System;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;
using NUnit.Framework;

using Snak.Core;
using Snak.BuildDrop;

namespace Snak.FunctionalTests
{
    /// <summary>
    ///  
    /// </summary>
    /// <remarks>
    ///  This class has obvious dependencies upon other parts of the SNAK library (e.g. it uses BuildDropLocations, ISolutionInfo). As such it assumes that these
    ///  are working correctly. As we are only testing the DeploySolutionTask this should not be an issue however if BuildDropLocations or ISolutionInfo are broken that
    ///  will have obvious effects here. Its not that bad utilizing theses classes because we at least gain more code coverage with our tests.
    /// </remarks>
    public abstract class AbstractDeploySolutionTaskTest : BuildTestBase
    {
        private FileInfo _solutionFile = null;
        private ISolutionInfo _solution = null;
        private DotNetFrameworkVersion _dotNetFrameworkVersion = DotNetFrameworkVersion.v2_0;
        private DirectoryInfo _dropDir = null;
        private BuildDropLocations _buildDropLocations = null;

        protected FileInfo SolutionFile
        {
            get { return _solutionFile; }
        }

        protected DotNetFrameworkVersion DotNetFrameworkVersion
        {
            get { return _dotNetFrameworkVersion; }
        }

        protected DirectoryInfo DropDir
        {
            get { return _dropDir; }
        }

        protected BuildDropLocations BuildDropLocations
        {
            get { return _buildDropLocations; }
        }

        protected ISolutionInfo Solution
        {
            get { return _solution; }
        }

        [TestFixtureSetUp]
        public void Init() { }

        /// <summary>
        /// provides the webmappings that get set against the DeploySolutionTaskTestsBase.Solution, more related to the .Net 1.1. support as 
        /// NAnt uses WebMaps to figure out the location of web projects. See the NAnt solution task documentation for more information. 
        /// </summary>
        public abstract Dictionary<string, WebMap> WebMaps { get; }

        /// <summary>
        /// provides common functionality to test the DeploySolutionTask
        /// </summary>
        /// <param name="relativePathtoSolution"></param>
        /// <param name="relativePathtoDropDir"></param>
        /// <param name="dotNetFrameworkVersion"></param>
        public AbstractDeploySolutionTaskTest(string relativePathtoSolution, string relativePathtoDropDir, DotNetFrameworkVersion dotNetFrameworkVersion)
        {
            if (dotNetFrameworkVersion == DotNetFrameworkVersion.v1_0)
                throw new NotSupportedException();

            this._solutionFile = new FileInfo(Path.GetFullPath(relativePathtoSolution));
            this._dropDir = new DirectoryInfo(relativePathtoDropDir);
            this._dotNetFrameworkVersion = dotNetFrameworkVersion;

            WebMapCollection webMapCollection = new WebMapCollection();

            if (this.WebMaps != null)
                foreach (WebMap map in this.WebMaps.Values)
                    webMapCollection.Add(map);

            _solution = SolutionFactory.GetSolution(_solutionFile, webMapCollection);
        }

        /// <summary>
        /// build the solution passed to the constructor of this class, deploys the output, checks the deployment, if createVirtualDirsForWebsProjects is true
        /// creates virtual directories for web apps and checks that they work.
        /// </summary>
        /// <remarks></remarks>
        /// <param name="buildConfig"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        public void BuildDeployAndCheckDeployment(string buildConfig, bool createVirtualDirsForWebsProjects)
        {
            BuildDeployAndCheckDeployment(buildConfig, String.Empty, createVirtualDirsForWebsProjects);
        }

        /// <summary>
        /// build the solution passed to the constructor of this class, deploys the output, checks the deployment, if createVirtualDirsForWebsProjects is true
        /// creates virtual directories for web apps and checks that they work.
        /// </summary>
        /// <remarks></remarks>
        /// <param name="buildConfiguration"></param>
        /// <param name="platform"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        public void BuildDeployAndCheckDeployment(string buildConfig, string buildPlatform, bool createVirtualDirsForWebsProjects)
        {
            try
            {
                BuildConfiguration buildConfigurationToUse = new BuildConfiguration(buildConfig, buildPlatform);

                string projectXml = CreateProjetXmlSnippet(buildConfig, buildPlatform, createVirtualDirsForWebsProjects);

                BuildDropLocations buildDropLocations = new BuildDropLocations(this._dropDir, _solution, buildConfigurationToUse, new BuildLabel("65"));
                // delete anything that exists from the last test run
                buildDropLocations.DeleteDropDirectores();

                string coneoleOutput = RunBuild(projectXml);

                if (!CheckDeployedFilesExistInBothCommonAndCurrentDropDirectories(buildDropLocations))
                {
                    Assert.Fail("Error on deployment. Either no files were deployed to the target folders or else the labeled drop folder does not match the 'current' drop folder.");
                }

                // if we deployed the web projects we will make a request to the web app in the project and see that we 
                // get results, its a bit hacky but should be sufficient. We will parse the page and check that a given string exists 
                // just to confirm that we didnt receive a server error 
                if (createVirtualDirsForWebsProjects)
                {
                    System.Net.WebClient browser = new System.Net.WebClient();
                    browser.UseDefaultCredentials = true;

                    string page = browser.DownloadString("http://localhost/SnakFunctionalTests_WebApplication1_ANiceSuffix/Default.aspx");

                    if (page == null || page == String.Empty || !page.Contains("It works!"))
                        Assert.Fail("Error accessing the web project at http://localhost/SnakFunctionalTests_WebApplication1_ANiceSuffix/Default.aspx. At the time of the test this is expected to map to the WebApplication1 project that belongs to the '" + _solution.SolutionFile.FullName + "' solution");

                    // delete any virtual directories what were created for the tests
                    // TODO: 
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        /// <summary>
        /// creates an XML string useful to pass to nant to test the DeploySolutionTask, uses both parameters and internal 
        /// class data to determine the structure of the xml.
        /// </summary>
        /// <remarks>ekk. this method is getting a bit big! however its fairly flat in that it doesnt call out to other functionality, so shouldnt be too hard to understand</remarks>
        /// <param name="buildConfiguration"></param>
        /// <param name="platform"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        /// <returns></returns>
        private string CreateProjetXmlSnippet(string buildConfig, string buildPlatform, bool createVirtualDirsForWebsProjects)
        {
            BuildConfiguration buildConfigurationToUse = new BuildConfiguration(buildConfig, buildPlatform);
            StringBuilder solutionXMLSnippet = new StringBuilder();
            StringBuilder webMappings = new StringBuilder();
            StringBuilder virtualDirectorySettings = new StringBuilder();
            _solution.ConfigurationManager.SetActiveBuildConfigurationAndLoad(buildConfigurationToUse);

            Dictionary<String, IProjectInfo> projects = _solution.ProjectsByName;

            string buildTask = String.Empty;

            if (_dotNetFrameworkVersion == DotNetFrameworkVersion.v1_1)
            {
                solutionXMLSnippet.Append(@"<solution configuration=""");
                solutionXMLSnippet.Append(buildConfig);
                solutionXMLSnippet.Append(@""" solutionfile=""");
                solutionXMLSnippet.Append(_solutionFile.FullName);
                solutionXMLSnippet.Append(@""">");

                webMappings.Append(@"<webmap>");

                foreach (IProjectInfo project in projects.Values)
                {
                    if (project.ProjectType == ProjectType.Web)
                    {
                        webMappings.Append(@"<map url=""");
                        webMappings.Append(this.WebMaps[project.ProjectName].Url);
                        webMappings.Append(@""" path=""");
                        webMappings.Append(this.WebMaps[project.ProjectName].Path);
                        webMappings.Append(@""" />");
                    }
                }

                webMappings.Append(@"</webmap>");
                solutionXMLSnippet.Append(webMappings.ToString());
                solutionXMLSnippet.Append("</solution>");

                buildTask = solutionXMLSnippet.ToString();
            }
            else if (_dotNetFrameworkVersion == DotNetFrameworkVersion.v2_0)
            {
                // /p:Configuration=Debug /p:Platform="Any CPU"
                buildTask = @"  <msbuild project=""" + _solutionFile.FullName + @""">
                                    <property name=""Configuration"" value=""" + buildConfig + @""" />
                                    <property name=""Platform"" value=""" + buildPlatform + @""" />
                                </msbuild>";
            }

            if (createVirtualDirsForWebsProjects)
            {
                virtualDirectorySettings.Append(@"<virDirSettings>");

                // figure out the virtual directory settings
                foreach (IProjectInfo project in projects.Values)
                {
                    if (project.ProjectType == ProjectType.WebSite || project.ProjectType == ProjectType.Web)
                    {
                        virtualDirectorySettings.Append(@"<virDirSetting projectName=""");
                        virtualDirectorySettings.Append(project.ProjectName);
                        virtualDirectorySettings.Append(@""" clrVersion=""");
                        virtualDirectorySettings.Append((project.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1) ? "v1.1.4322" : "v2.0.50727"); // bit of a hardcoded dependency here... its not that bad
                        virtualDirectorySettings.Append(@""" authanonymous=""");
                        virtualDirectorySettings.Append("True");
                        virtualDirectorySettings.Append(@""" authbasic=""");
                        virtualDirectorySettings.Append("False");
                        virtualDirectorySettings.Append(@""" authntlm=""");
                        virtualDirectorySettings.Append("True");
                        virtualDirectorySettings.Append(@""" enabledirbrowsing=""");
                        virtualDirectorySettings.Append("False");
                        virtualDirectorySettings.Append(@""" defaultdoc=""");
                        virtualDirectorySettings.Append("Default.htm,Default.asp,index.htm,iisstart.asp,Default.aspx");
                        virtualDirectorySettings.Append(@""" />");
                    }
                }

                virtualDirectorySettings.Append(@"</virDirSettings>");
            }

            string xmlsnippet = String.Format(
                                    @"<project name=""myTestProject"">
                                        {0}
                                        <loadtasks assembly=""{1}"" />
                                        {2}
                                        <deploySolution 
                                          solution=""{3}"" 
                                          config=""{4}"" 
                                          artifactsDir=""{5}""
                                          buildLabel=""65""
                                          webProjects_SetUpVirtualDirectories=""{6}""
                                          webProjects_VirtualDirectoriesPrefix=""SnakFunctionalTests_""
                                          webProjects_VirtualDirectoriesSuffix=""_ANiceSuffix""
                                          property=""deploySolutionTask""
                                          >
                                          {7}
                                          {8}
                                        </deploySolution>
                                      </project>",
                                    // tell nant which .net framework to compile with
                                    ((this.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1) ? @"<property name=""nant.settings.currentframework"" value=""net-1.1"" />" : @"<property name=""nant.settings.currentframework"" value=""net-2.0"" />"),
                                    // GetFullPath will return the rooted path to where ever this the test is running from, as it turns out 
                                    // this is different from where the base class writes this xml and loads it into nant, hence we need the full path to Snak.dll
                                    Path.GetFullPath("Snak.dll"),
                                    buildTask,
                                    _solutionFile.FullName,
                                    buildConfigurationToUse.ToString(),
                                    _dropDir.FullName,
                                    createVirtualDirsForWebsProjects.ToString(),
                                    // specify the web mapping similar to those passed to the 'solution' task that comes out of the box with nant
                                    webMappings.ToString(),
                                    // if we are setting up virtual directories for web projects we need to specify the setting for them
                                    // check out the doco in DeploySolutionTask for more info and an xml example
                                    ((createVirtualDirsForWebsProjects) ? virtualDirectorySettings.ToString() : String.Empty)
                                );

            // do we want to write the temp project file to to c:\temp for later use (e.g. as an example reference)
            if (Snak.FunctionalTests.Properties.Settings.Default.WriteDynamicallyCreatedBuildFilesToDisk)
            {
                string language = "C#";

                // hack:
                if (_solution.SolutionFile.FullName.Contains("VB"))
                { 
                    language = "VB";
                }

                string buildFilePath = String.Format(
                                            @"c:\temp\{0}.{1}.{2}.{3}{4}.build",
                                            Path.GetFileNameWithoutExtension(_solution.SolutionFile.Name),
                                            _dotNetFrameworkVersion.ToString(),
                                            language,
                                            buildConfig.Replace("|", "-").Replace(" ", ""),
                                            ((createVirtualDirsForWebsProjects) ? ".createVirtualDirs" : "")
                                        );

                try
                {
                    // could just write out xmlsnippet to disk but we'll use an XmlTextWriter to get some decent formatting
                    XmlWriterSettings xmlWriterSettings = new XmlWriterSettings();
                    xmlWriterSettings.Indent = true;
                    xmlWriterSettings.NewLineOnAttributes = true;

                    using (XmlWriter writer = XmlTextWriter.Create(buildFilePath, xmlWriterSettings))
                    {
                        XmlDocument document = new XmlDocument();
                        document.LoadXml(xmlsnippet);
                        document.PreserveWhitespace = false;
                        document.WriteTo(writer);
                    }
                }
                catch (Exception ex)
                {
                    throw new ApplicationException("Error trying to write the build file '" + buildFilePath + "'. The test application will only try write the dynamically created build files when the test application setting WriteDynamicallyCreatedBuildFilesToDisk is true.", ex);
                }                    
            }

            return xmlsnippet;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="buildDropLocations"></param>
        private bool CheckDeployedFilesExistInBothCommonAndCurrentDropDirectories(BuildDropLocations buildDropLocations)
        {
            //
            // In here we want to check that labeled drop dir has the same dir structure (including files) as the common drop dir.
            // We want to do this for the drop dirs that are common throughout the solution and also for the projects that get their own drop dir.
            //

            bool success = true;

            // Solution drop dirs: check that the labeled drop dir has the same dir structure (including files) as the common drop dir
            success = CheckDirectoryHasSameFilesAndSubDirectories(buildDropLocations.CommonAssembliesDropLocation.DropDirLabeled, buildDropLocations.CommonAssembliesDropLocation.DropDirCurrent);

            if (success)
            {
                // project that get their own drop dirs: check that the labeled drop dir has the same dir structure (including files) as the common drop dir
                foreach (KeyValuePair<string, DropLocation> keyValuePair in buildDropLocations.DropLocationsByProjectSafeName)
                {
                    // we check that the root directory has files
                    // we should really do it for all but some may be meant to be empty, for now just check the root 
                    if (keyValuePair.Value.DropDirLabeled.GetFiles().Length == 0)
                    {
                        success = false;
                        break;
                    }
                    else if (keyValuePair.Value.DropDirCurrent.GetFiles().Length == 0)
                    {
                        success = false;
                        break;
                    }

                    // this next method recursively checks the directories to ensure they are the same
                    CheckDirectoryHasSameFilesAndSubDirectories(keyValuePair.Value.DropDirLabeled, keyValuePair.Value.DropDirCurrent);
                }
            }

            return success;
        }

        /// <summary>
        /// recursively checks the directories to ensure they are the same (checks on file name and file length)
        /// </summary>
        /// <param name="expectedDirStructure"></param>
        /// <param name="actualDirStructure"></param>
        /// <returns></returns>
        private bool CheckDirectoryHasSameFilesAndSubDirectories(DirectoryInfo expectedDirStructure, DirectoryInfo actualDirStructure)
        {
            bool success = true;

            Dictionary<string, FileInfo> filesInExpectedDirStructure = new Dictionary<string, FileInfo>();
            Dictionary<string, DirectoryInfo> subDirsInExpectedDirStructure = new Dictionary<string, DirectoryInfo>();

            // build a dictionary of files that exist in the expected dir structure, keyed it on file name
            foreach (FileInfo file in expectedDirStructure.GetFiles())
            {
                filesInExpectedDirStructure.Add(file.Name, file);
            }

            // build a dictionary of sub directories that exist in the expected dir structure, keyed it on dir name
            foreach (DirectoryInfo subDirectory in expectedDirStructure.GetDirectories())
            {
                subDirsInExpectedDirStructure.Add(subDirectory.Name, subDirectory);
            }

            // check that the files in the actual directory structure are in our dictionary of expected file 
            // this leaves a little gap in that there may be more files in actualDirStructure than expectedDirStructure
            // but at this point were not to worries about that
            foreach (FileInfo file in actualDirStructure.GetFiles())
            {
                if (filesInExpectedDirStructure.ContainsKey(file.Name))
                {
                    // check the lenght just to be safe
                    if (filesInExpectedDirStructure[file.Name].Length != file.Length)
                    {
                        success = false;
                        break;
                    }
                }
                else
                {
                    success = false;
                    break;
                }
            }

            if (success)
            {
                // now we check that the sub directorieis in the actual directory structure are in our dictionary of expected sub directories 
                foreach (DirectoryInfo subDirectory in actualDirStructure.GetDirectories())
                {
                    if (subDirsInExpectedDirStructure.ContainsKey(subDirectory.Name))
                    {
                        // recursively check that the sub directories are the same
                        if (!CheckDirectoryHasSameFilesAndSubDirectories(subDirsInExpectedDirStructure[subDirectory.Name], subDirectory))
                        {
                            success = false;
                            break;
                        }
                    }
                    else
                    {
                        success = false;
                        break;
                    }
                }
            }

            return success;
        }
    }
}

