using System;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;

using Tests.NAnt.Contrib;
using NUnit.Framework;

namespace Snak.FunctionalTests
{
    [TestFixture]
    public class TransformConfigurationFileTaskTests : BuildTestBase
    {
        private string preProcessedFileElementSuffix = "_dev";

        [TestFixtureSetUp]
        public void Init() { }

        [NUnit.Framework.Test(Description = "")]
        public void TestTransformation()
        {
            try
            {
                bool inputFileConainsDesiredEnvironment;
                string[] environments = new string[] { "buildVerification", "systemTest", "production", "support", "training", "dev" };
                List<TransformationInfo> transformedFileInfo = new List<TransformationInfo>();

                DirectoryInfo test = new DirectoryInfo(".");
                DirectoryInfo exampleConfigDirectory = new DirectoryInfo("..\\..\\..\\TestArtifacts\\TestConfigFiles\\ParserTests_Unprocessed");

                if (!Directory.Exists("..\\..\\..\\TestArtifacts\\TestConfigFiles\\ParserTests_ProcessedOutput"))
                    Directory.CreateDirectory("..\\..\\..\\TestArtifacts\\TestConfigFiles\\ParserTests_ProcessedOutput");

                // xmlsnippet contains a nant project file with multiple transformConfigurationFile tasks. For each 
                // test file there will be a transformConfigurationFile task for each target environment. 
                // for example if we have a config file called Example.config and 2 target environments, support and production then we should have
                // 2 transformConfigurationFile tasks that produce 2 transformed files called Example.support.config and Example.prod.config
                StringBuilder xmlsnippet = new StringBuilder(@"<project name=""testTransformConfigurationFiles""><loadtasks assembly=""");
                xmlsnippet.Append(Path.GetFullPath("Snak.dll"));
                xmlsnippet.Append(@""" />");

                foreach (FileInfo file in exampleConfigDirectory.GetFiles())
                {
                    string inputFileContent = File.ReadAllText(file.FullName);

                    for (int i = 0; i < environments.Length; i++)
                    {
                        // we need to call GetFullPath() beacuse we dont really have control over where the base class writes the project file 
                        // we are generating to, any relative paths we specify will only be relative to that temporary location
                        string outFile = Path.GetFullPath("..\\..\\..\\TestArtifacts\\TestConfigFiles\\ParserTests_ProcessedOutput\\" + Path.GetFileNameWithoutExtension(file.FullName) + "-" + environments[i] + Path.GetExtension(file.FullName));
                        
                        xmlsnippet.Append(@"<transformConfigurationFile targetConfigurationFile=""");
                        xmlsnippet.Append(file.FullName);
                        xmlsnippet.Append(@""" "); 
                        xmlsnippet.Append(@"outputConfigurationFile=""");
                        xmlsnippet.Append(outFile);
                        xmlsnippet.Append(@""" "); 
                        xmlsnippet.Append(@"targetEnvironment=""");
                        xmlsnippet.Append(environments[i]);
                        xmlsnippet.Append(@""" />"); 

                        inputFileConainsDesiredEnvironment = (inputFileContent.IndexOf("$(" + environments[i] + ")", 0, inputFileContent.Length) > -1);

                        // store the information for this transformation in a list so we can later check the transformation output
                        transformedFileInfo.Add(new TransformationInfo(environments[i],file.FullName, outFile, inputFileConainsDesiredEnvironment));
                    }
                }
                
                xmlsnippet.Append("</project>");

                // call base class functionality to fire up nant and run the project file we just created
                RunBuild(xmlsnippet.ToString());                        

                foreach (TransformationInfo transformationInfo in transformedFileInfo)
                {
	                 CheckTransformationOutput(transformationInfo);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        private void CheckTransformationOutput(TransformationInfo transformationInfo)
        {
            StreamReader preProcessedFileStream = null;
            StreamReader processedFileStream = null;

            try
            {
                bool readerNotFinished = true;
                preProcessedFileStream = new StreamReader(transformationInfo.PreProcessedFile);
                processedFileStream = new StreamReader(transformationInfo.ProcessedFile);

                XmlReaderSettings xmlReaderSettings = new XmlReaderSettings();
                xmlReaderSettings.IgnoreComments = true;
                xmlReaderSettings.IgnoreWhitespace = true;
                xmlReaderSettings.IgnoreProcessingInstructions = true;

                XmlReader unprocessedFileStreamReader = XmlTextReader.Create(preProcessedFileStream);
                XmlReader processedFileStreamReader = XmlTextReader.Create(processedFileStream);

                while (readerNotFinished)
                {
                    unprocessedFileStreamReader.Read();
                    processedFileStreamReader.Read();

                    if (unprocessedFileStreamReader.Name == String.Empty)
                        unprocessedFileStreamReader.MoveToContent();
                    if (processedFileStreamReader.Name == String.Empty)
                        processedFileStreamReader.MoveToContent();

                    if (unprocessedFileStreamReader.Name.Contains(preProcessedFileElementSuffix))
                    {
                        if (transformationInfo.InputFileConainsDesiredEnvironment)
                        {
                            if (String.Compare(unprocessedFileStreamReader.Name, processedFileStreamReader.Name, true) != 0)
                            {
                                string expectedElementName = unprocessedFileStreamReader.Name.Replace(preProcessedFileElementSuffix, "_" + transformationInfo.Environment);

                                if (String.Compare(processedFileStreamReader.Name, expectedElementName, true) != 0)
                                {
                                    throw new ApplicationException("An error occurred while comparing the processed output ('" + transformationInfo.ProcessedFile + "') against the original file ('" + transformationInfo.PreProcessedFile + "'). The element named '" + unprocessedFileStreamReader.Name + "' in the original file should map to the element named '" + expectedElementName + "' in the processed file, however the processed file element was named: '" + processedFileStreamReader.Name + "'");
                                }
                            }
                        }
                        else
                        {
                            // progress the reader until we pass all the elements that contain the preProcessedFileElementSuffix suffix
                            while (unprocessedFileStreamReader.Read())
                            {
                                if (unprocessedFileStreamReader.Name == String.Empty)
                                    unprocessedFileStreamReader.MoveToContent();

                                if (!unprocessedFileStreamReader.Name.Contains(preProcessedFileElementSuffix))
                                {
                                    break;
                                }
                            }
                        }
                    }
                    else if (String.Compare(unprocessedFileStreamReader.Name, processedFileStreamReader.Name, true) != 0)
                    {
                        Assert.Fail();
                    }

                    if (unprocessedFileStreamReader.EOF || processedFileStreamReader.EOF)
                        readerNotFinished = false;
                }
            }
            finally
            {
                if (preProcessedFileStream != null)
                {
                    preProcessedFileStream.Dispose();
                    preProcessedFileStream.Close();
                }

                if (processedFileStream != null)
                {
                    processedFileStream.Dispose();
                    processedFileStream.Close();
                }
            }
        }

        private class TransformationInfo
        {
            private string _environment;

	        public string Environment
	        {
		        get { return _environment;}
		        set { _environment = value;}
	        }

            private string _preProcessedFile;

	        public string PreProcessedFile
	        {
		        get { return _preProcessedFile;}
		        set { _preProcessedFile = value;}
	        }

            private string _processedFile;

	        public string ProcessedFile
	        {
		        get { return _processedFile;}
		        set { _processedFile = value;}
	        }

            private bool  _inputFileConainsDesiredEnvironment;

	        public bool  InputFileConainsDesiredEnvironment
	        {
		        get { return _inputFileConainsDesiredEnvironment;}
		        set { _inputFileConainsDesiredEnvironment = value;}
	        }

            public TransformationInfo(string environment, string preProcessedFile, string processedFile, bool inputFileConainsDesiredEnvironment)
            {
                this._environment = environment;
                this._preProcessedFile = preProcessedFile;
                this._processedFile = processedFile;
                this._inputFileConainsDesiredEnvironment = inputFileConainsDesiredEnvironment;
            }
        }
    }
}
