using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

using Snak.Core;


namespace Snak.BuildDrop
{
    /// <summary>
    /// Provides a single location for the management of directories used when SNAK is dropping the build.
    /// 
    /// Drop locations for the projects depends upon the project type (e.g. web project, class library, this information is stored in IProjectInfo.OutputType)
    /// each solution gets a main assemblies folder, web, exe's and winexe's get their own folder, class libraries just get added to the main assemblies folder. These folders 
    /// get a Suffixed with the build label. In addition to these folders we also have a _Current folder. This is basically a copy of whats in the 
    /// folder with the highest build label. Check out the test for example folder names.
    ///
    /// You can either use this class when building and then droping the build or if you want to pick up a build and do further work (e.g. deployment). 
    /// </summary>
    public class BuildDropLocations
    {
        private string _solutionName = String.Empty;
        private DirectoryInfo _artifactsDirectory = null;
        private DropLocation _commonAssembliesDropLocation = null;
        private Dictionary<string, DropLocation> _dropLocationsByProjectSafeName = new Dictionary<string, DropLocation>();
        private ISolutionInfo _solution = null;
        private BuildLabel _buildLabel = null;

        /// <summary>
        /// The solution passed in via the constructor
        /// </summary>
        public ISolutionInfo Solution
        {
            get { return _solution; }
        }

        /// <summary>
        /// Common drop location for projects that dont get dropped into their own folder.
        /// </summary>
        public DropLocation CommonAssembliesDropLocation
        {
            get { return _commonAssembliesDropLocation; }
        }

        /// <summary>
        /// Drop locations by project name
        /// </summary>
        /// <remarks>
        /// if the project shares a drop location (e.g. the project type is not a web app, windows app, service app but rather a class library)
        /// then its reference in this property will be then same as the property CommonAssembliesDropLocation
        /// </remarks>
        public Dictionary<string, DropLocation> DropLocationsByProjectSafeName
        {
            get { return (this._dropLocationsByProjectSafeName); }   
        }

        /// <summary>
        /// reference to the artifactsDirDirectory passed in via the constructors
        /// </summary>
        public DirectoryInfo ArtifactsDirDirectory
        {
            get { return _artifactsDirectory; }
        }

        /// <summary>
        /// provides a single point of reference for drop locations for solutions and their projects
        /// </summary>
        /// <param name="artifactsDirDirectory">The containing directory which the build will be dropped into (sub directories will be created with the appropriate name)</param>
        /// <param name="solution">The solution which contains the projects to drop</param>
        /// <param name="solutionConfiguration">The build configuration (e.g. .Net 1.1: "Debug", .Net 2.0: "Debug|Any Cpu" ) </param>
        /// <param name="buildLabel">The build label information</param>
        public BuildDropLocations(DirectoryInfo artifactsDirDirectory, ISolutionInfo solution, BuildConfiguration buildConfiguration, BuildLabel buildLabel)
        {
            if (artifactsDirDirectory == null)
                throw new ArgumentNullException("artifactsDirDirectory");
            if (solution == null)
                throw new ArgumentNullException("solution");
            if (buildConfiguration == null)
                throw new ArgumentException("the argument buildConfiguration cannot be null or empty", "buildConfiguration");
            if (buildLabel == null)
                throw new ArgumentException("the argument buildLabel cannot be null", "buildLabel");

            _artifactsDirectory = artifactsDirDirectory;
            _solution = solution;
            _buildLabel = buildLabel;

            DropLocation projectDropLocations = null;

            string commonSolutionDropDirNameWithoutBuildNumberPart = _artifactsDirectory + "\\" + _solution.SolutionFile.Name.Replace(".sln", "") + "_Assemblies_" + buildConfiguration.ToSafeDirNameString() + "_";

            _commonAssembliesDropLocation = new DropLocation(new DirectoryInfo(commonSolutionDropDirNameWithoutBuildNumberPart + buildLabel.BuildLabelString), new DirectoryInfo(commonSolutionDropDirNameWithoutBuildNumberPart + "Current"), TryFigureOutPreviousDropDir(commonSolutionDropDirNameWithoutBuildNumberPart), false);

            _solution.ConfigurationManager.SetActiveBuildConfigurationAndLoad(buildConfiguration);

            foreach (IProjectInfo project in _solution)
            {
                bool deployToOwnDirectory = (project.ProjectType == ProjectType.WebSite || project.ProjectType == ProjectType.WebApplication || project.ProjectOutputType == ProjectOutputType.Exe || project.ProjectOutputType == ProjectOutputType.WinExe);

                if (deployToOwnDirectory)
                {
                    string projectDropDirNameWithoutBuildNumberPart = _artifactsDirectory + "\\" + project.ProjectSafeName + "_" + buildConfiguration.ToSafeDirNameString() + "_";

                    projectDropLocations = new DropLocation(new DirectoryInfo(projectDropDirNameWithoutBuildNumberPart + buildLabel.BuildLabelString), new DirectoryInfo(projectDropDirNameWithoutBuildNumberPart + "Current"), TryFigureOutPreviousDropDir(projectDropDirNameWithoutBuildNumberPart), deployToOwnDirectory);    
                }
                else 
                {
                    projectDropLocations = new DropLocation(_commonAssembliesDropLocation.DropDirLabeled, _commonAssembliesDropLocation.DropDirCurrent, TryFigureOutPreviousDropDir(commonSolutionDropDirNameWithoutBuildNumberPart), deployToOwnDirectory);
                }

                this._dropLocationsByProjectSafeName.Add(project.ProjectSafeName, projectDropLocations);
            }
        }

        /// <summary>
        /// Creates all the drop directories for solution passed to the constructor
        /// </summary>
        public void CreateDropDirectores()
        {
            _commonAssembliesDropLocation.CreateDropDirectories();

            foreach (KeyValuePair<string, DropLocation> keyValuePair in _dropLocationsByProjectSafeName)
            {
                keyValuePair.Value.CreateDropDirectories();
            }
        }

        /// <summary>
        /// Deletes all the drop directories for solution passed to the constructor
        /// </summary>
        public void DeleteDropDirectores()
        {
            // TODO: do we want to delete here or perhaps just delete the files

            _commonAssembliesDropLocation.DeleteDropDirectories();

            foreach (KeyValuePair<string, DropLocation> keyValuePair in _dropLocationsByProjectSafeName)
            {
                keyValuePair.Value.DeleteDropDirectories();
            }
        }

        private DirectoryInfo TryFigureOutPreviousDropDir(string dropDirNameWithoutBuildNumberPart)
        { 
            DirectoryInfo previousDropDir = null;

            // try figure out the name that would have been given to the last drop location, 
            if (_buildLabel.BuildLabelType == BuildLabelType.Numeric)
            { 
                int previousBuildLabel = _buildLabel.BuildLabelNumber - 1;
                previousDropDir = new DirectoryInfo(dropDirNameWithoutBuildNumberPart + previousBuildLabel.ToString());

                if (!previousDropDir.Exists)
                    previousDropDir = null;
            }
            else if (_buildLabel.BuildLabelType == BuildLabelType.VersionNumber)
            {
                throw new NotImplementedException("Cannot figure out PreviousDropDir when the build label is a version (e.g. 1.0.0.0), currently we only support numerical build labels");
                // GetFirstDirectoryNameBasedOnSortTask getFirstDirectoryNameBasedOnSortTask = new GetFirstDirectoryNameBasedOnSortTask();
            }

            return previousDropDir;
        }

#if(UNITTEST)

        [NUnit.Framework.TestFixture]
        public class BuildDropInfoTester
        {
            [NUnit.Framework.TestFixtureSetUp]
            public void Init() { }

            [NUnit.Framework.SetUp]
            public void Setup() { }

            [Test]
            public void CheckBuildDropDirectoryNames()
            {
                BuildConfiguration buildConfiguration = new BuildConfiguration("Debug", "Any CPU");

                ISolutionInfo solution = new Snak.Core.Whidbey.WhidbeySolutionInfo(new FileInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\SolutionWithMultipleDifferingProjects.sln"), buildConfiguration);
                DirectoryInfo artifactDir = new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#");
                BuildDropLocations buildDropLocations = new BuildDropLocations(artifactDir, solution, buildConfiguration, new BuildLabel("65"));

                // IProjectInfo[] projects = solution.GetProjectsFor("Debug|Any CPU");
                Dictionary<string, IProjectInfo> projectsInSolution = new Dictionary<string, IProjectInfo>();

                foreach (IProjectInfo project in solution)
                {
                    projectsInSolution.Add(project.ProjectSafeName, project);
                }

                CheckDirectoryName(
                    buildDropLocations.CommonAssembliesDropLocation.DropDirLabeled,
                    new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects_Assemblies_Debug-AnyCPU_65"),
                    "SolutionWithMultipleDifferingProjects.sln (Solution)"
                );

                CheckDirectoryName(
                    buildDropLocations.CommonAssembliesDropLocation.DropDirCurrent,
                    new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects_Assemblies_Debug-AnyCPU_Current"),
                    "SolutionWithMultipleDifferingProjects.sln (Solution)"
                );
                
                foreach (KeyValuePair<string, DropLocation> keyValuePair in buildDropLocations.DropLocationsByProjectSafeName)
                {
                    IProjectInfo project = projectsInSolution[keyValuePair.Key];

                    bool deployToOwnDirectory = (project.ProjectType == ProjectType.WebSite || project.ProjectType == ProjectType.WebApplication || project.ProjectOutputType == ProjectOutputType.Exe || project.ProjectOutputType == ProjectOutputType.WinExe);

                    if (deployToOwnDirectory)
                    {
                        CheckDirectoryName(
                            keyValuePair.Value.DropDirLabeled, 
                            new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\" + keyValuePair.Key + "_Debug-AnyCPU_65"),
                            project.ProjectName
                        );

                        CheckDirectoryName(
                            keyValuePair.Value.DropDirCurrent, 
                            new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\" + keyValuePair.Key + "_Debug-AnyCPU_Current"),
                            project.ProjectName
                        );
                    }
                    else
                    {
                        CheckDirectoryName(
                            keyValuePair.Value.DropDirLabeled,
                            new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects_Assemblies_Debug-AnyCPU_65"),
                            project.ProjectName
                        );
                        
                        CheckDirectoryName(
                            keyValuePair.Value.DropDirCurrent,
                            new DirectoryInfo(@"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects_Assemblies_Debug-AnyCPU_Current"),
                            project.ProjectName
                        );
                    }
                }
            }

            private void CheckDirectoryName(DirectoryInfo actual, DirectoryInfo expected, string description)
            {
                Console.WriteLine("-------------------");
                Console.WriteLine("Checking drop directory for project/solution '" + description + "'.");
                Console.WriteLine("The path: ");
                Console.WriteLine("'" + actual.FullName + "'");
                Console.WriteLine("shouild match the expected path of:");
                Console.WriteLine("'" + expected.FullName + "'");

                if (String.Compare(actual.FullName, expected.FullName, false) != 0)
                    NUnit.Framework.Assert.Fail("The build drop directory path '{0}' does not match the expected path of '{1}'.", actual.FullName, expected.FullName);
            }
        }

#endif

    }
}
