using System;
using System.Threading;
using System.IO;
using System.Collections.Generic;
using System.Text;
using NAnt.Core;
using NAnt.Core.Tasks;
using NAnt.Core.Types;

using Snak;
using Snak.Utilities;

namespace Snak.BuildDrop
{
    /// <summary>
    /// Defines a drop location for a project or solution.
    /// 
    /// There are 2 drop directories for any given item (either a project or solution), firstly a drop directory thats suffix is the build label.
    /// Secondly there is a drop directory that has suffix of _Current, this will always contain the latest build. This provides a well known 
    /// point of reference build outputs (useful if other application need to pull in the latest dependencies).
    /// </summary>
    public class DropLocation
    {
        private string _projectName = String.Empty;
        private DirectoryInfo _dropDirlabeled = null;
        private DirectoryInfo _dropDirCurrent = null;
        private DirectoryInfo _dropDirPrevious = null;
        private bool _dropIsForSingleApplicaiton = true;

        /// <summary>
        /// Drop directory that gets a suffix which is the build label
        /// </summary>
        public DirectoryInfo DropDirLabeled
        {
            get { return _dropDirlabeled; }
        }

        /// <summary>
        /// The drop directory that contains the build output from the last build
        /// </summary>
        public DirectoryInfo DropDirCurrent
        {
            get { return _dropDirCurrent; }
        }

        /// <summary>
        /// The drop directory that contains the build output from the build before last
        /// </summary>
        public DirectoryInfo DropDirPrevious
        {
            get { return _dropDirPrevious; }
        }

        /// <summary>
        /// defines if this DropLocation is for a single application as apposed to a shared drop location for multiple projects
        /// </summary>
        public bool DropIsForSingleApplicaiton
        {
            get { return _dropIsForSingleApplicaiton; }
        }

        /// <summary>
        /// provides reference to the drop locations for a build item (e.g. a project or a solution output)
        /// </summary>
        /// <param name="dropDir"></param>
        /// <param name="dropDirCurrent"></param>
        /// <param name="dropIsForSingleApplicaiton"></param>
        public DropLocation(DirectoryInfo dropDir, DirectoryInfo dropDirCurrent, DirectoryInfo dropDirPrevious, bool dropIsForSingleApplicaiton)
        {
            if (dropDir == null)
                throw new ArgumentNullException("projectDeploymentDir");

            if (dropDirCurrent == null)
                throw new ArgumentNullException("projectDeploymentDirCurrent");

            this._dropDirlabeled = dropDir;
            this._dropDirCurrent = dropDirCurrent;
            this._dropDirPrevious = dropDirPrevious;
            this._dropIsForSingleApplicaiton = dropIsForSingleApplicaiton;
        }

        /// <summary>
        /// copies the files in DropDirlabeled to DropDirCurrent
        /// </summary>
        public void PublishDropToCurrentDir()
        {
            try
            {
                DirectoryInfo currentBuildDir = null;
                string newFile = String.Empty;

                FileSet fileSet = new FileSet();
                fileSet.BaseDirectory = _dropDirlabeled;
                fileSet.Includes.Add(@"**\*");

                foreach (string fileName in fileSet.FileNames)
                {
                    newFile = fileName.Replace(_dropDirlabeled.FullName, _dropDirCurrent.FullName);
                    currentBuildDir = new DirectoryInfo(Path.GetDirectoryName(newFile));

                    if (!currentBuildDir.Exists)
                        currentBuildDir.Create();

                    FileInfo file = new FileInfo(fileName);
                    file.CopyTo(newFile, true);
                }
            }
            catch
            {
                // TODO: logging.
                // Log(Level.Error, "Error publishing files in the directory '{0}' to '{1}'", dirToCopyFrom.FullName, dirToPublishTo.FullName);
                throw;
            }
        }

        internal void CreateDropDirectories()
        {
            CreateDropDirectory(_dropDirlabeled);
            CreateDropDirectory(_dropDirCurrent);
        }

        internal void DeleteDropDirectories()
        {
            DeleteDropDirectory(_dropDirlabeled);
            DeleteDropDirectory(_dropDirCurrent);
        }

        private void CreateDropDirectory(DirectoryInfo directoryToCreate)
        {
            if (directoryToCreate.Exists)
            {
                // TODO: implement ibuildlogger somewhere!, perhaps in a base class of some description, or use log4net or internal logging and leave Log() to the tasks?
                // Log(Level.Verbose, "Directory '" + directoryToCreate.FullName + "' exists. Attempting to delete and recreate...");

                if (DeleteDropDirectory(directoryToCreate))
                {
                    directoryToCreate.Create();
                }
            }
            else
            {
                // Log(Level.Verbose, "Directory '" + directoryToCreate.FullName + "' does not exist. Creating now...");
                directoryToCreate.Create();
            }
        }

        private bool DeleteDropDirectory(DirectoryInfo directoryToDelete)
        {
            bool success = false;

            // TODO: implement ibuildlogger somewhere!, perhaps in a base class of some description, or use log4net or internal logging and leave Log() to the tasks?
            // Log(Level.Verbose, "Directory '" + directoryToCreate.FullName + "' exists. Attempting to delete and recreate...");
            // Log(Level.Verbose, "About to delete directory '{0}'.", directoryToDelete.FullName);

            try
            {
                if (directoryToDelete.Exists)
                {
                    TaskUtils.RecursivelyApplyFileAttribute(directoryToDelete, FileAttributes.Normal);
                    directoryToDelete.Delete(true);
                }

                success = true;
            }
            catch (Exception)
            {
                // TODO: log and continue
            }

            return success;
        }
    }
}
