using System;
using System.IO;
using System.Xml;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

using NUnit.Framework;
using Snak;

namespace Snak.ConfigurationTransformation
{
    /// <summary>
    /// ConfigurationTransformer provides a mechanism to transform an xml configuration file based on certain conditional statements which appear within the file.
    /// 
    /// This is useful in situations where you have to maintain multiple environments each having some different configuration settings. Its allows for all of the 
    /// setting to be stored in one file. For sensitive settings either encrypt the section or store it in the registry. For more information on this see 
    /// http://aspnet.4guysfromrolla.com/articles/021506-1.aspx
    /// </summary>
    /// <example>
    /// The snippet below shows some xml that may appear in an application configuration file. The value within the $( ) portionis is the environment for which 
    /// the setting is applicable. 
    /// <![CDATA[
    ///     <!--  
    ///     #ENV:$(buildVerification) 
    ///       <element_buildVerification mode="Off" />
    ///     #ENV:$(systemTest)
    ///       <element_SystemTest mode="Off" /> 
    ///     #ENV:$(production)
    ///         <element_production mode="Off" />
    ///     #ENV:$(support)
    ///         <element_support mode="Off" />
    ///     #ENV:$(training)
    ///         <element_training mode="Off" />
    ///     #ENV:$(dev)
    ///     -->
    ///         <!-- this is a comment -->
    ///         <element_dev mode="Off" />
    ///     <!-- 
    ///     #END
    ///     -->
    /// ]]>
    /// For example if you ran the above xml through ConfigurationTransformer.Transform(string, string) passing the value "buildVerification"
    /// for the environment variable, the output would be:
    /// <![CDATA[<element_buildVerification mode="Off" />]]>
    /// If you passed "training" the output would be:
    /// <![CDATA[<element_training mode="Off" />]]>
    /// 
    /// The last environment setting dev appears out of the XML comments and as such is the value used when the configuration file has not been run through the 
    /// ConfigurationTransformer. This last section is meant to hold the development settings. As part of your build process you transform the dev config file to 
    /// create a new file for the target environments.
    /// </example>
    internal class ConfigurationTransformer
    {
        internal ConfigurationTransformer() { }

        /// <summary>
        /// Transforms a configuration file and saves off the new file to the value specified in 'outFile'.
        /// </summary>
        /// <param name="environment">the target environment</param>
        /// <param name="inFile">the file to transform, e.g. c:\dev\MyApp.config </param>
        /// <param name="outFile">the transformed file will be written to this path, e.g. c:\prod\MyApp.config </param>
        internal void Transform(string environment, string inFile, string outFile)
        {
            string configAsString = File.ReadAllText(inFile);
            string newConfig = configAsString;

            IfElseEndIFConstruct ifElseConstruct = null;
            ArrayList ifElseConstructs = new  ArrayList();

            // the ifElseEndIfBlockPattern pattern below will grab each indivudual '#ENV:' '#END' processor blocks, we can then concentrate on 
            // parsing the text within these blocks (if any)
            string ifElseEndIfBlockPattern = @"(?<ifElseEndIfBlock><!--(\r\n|\s)*?#ENV:[\s]*?\$\([a-z]*?\)(.|\r\n|\s)*?#END(\r\n|\s)*?-->)";
            // the ifElseStatements pattern below will capture each if/elseif/else block, branchVariable will contain the text 
            // that identifies the branch and branchInnerText contains the actual inner text of the branch
            string ifElseStatements = @"(#ENV:)([\s]*?\$\((?<branchVariable>[a-z]*?)\)){1}((\r\n|\s)*?-->)?(?<branchInnerText>(.|\r\n|\s)*?)(?=(#ENV:|<!--(\r\n|\s)*?#END|#END(\r\n|\s)*?-->))";

            for (Match ifElseEndIfBlockMatch = Regex.Match(configAsString, ifElseEndIfBlockPattern, RegexOptions.IgnoreCase); ifElseEndIfBlockMatch.Success; ifElseEndIfBlockMatch = ifElseEndIfBlockMatch.NextMatch())
            {
                ifElseConstruct = new IfElseEndIFConstruct(ifElseEndIfBlockMatch.Value);
                ifElseConstructs.Add(ifElseConstruct);

                string ifElseEndIfBlock = ifElseEndIfBlockMatch.Groups["ifElseEndIfBlock"].Value.ToString();

                for (Match innerMatch = Regex.Match(ifElseEndIfBlock, ifElseStatements, RegexOptions.IgnoreCase); innerMatch.Success; innerMatch = innerMatch.NextMatch())
                {
                    string matchValue = innerMatch.Value;
                    string branchVariable = innerMatch.Groups["branchVariable"].Value;
                    string branchInnerText = innerMatch.Groups["branchInnerText"].Value;

                    ifElseConstruct.AddIfBranchPart(branchVariable, branchInnerText);
                }

                if (ifElseConstruct.IfElseBranchPartsByVariable.ContainsKey(environment))
                {
                    string replacementValue = ifElseConstruct.IfElseBranchPartsByVariable[environment];

                    // replace any comments with XML comments
                    replacementValue = replacementValue.Replace("/*", "<!--").Replace("*/", "-->");

                    // replace the entire if else branch with the content of the chosen branch
                    newConfig = newConfig.Replace(ifElseConstruct.IfElseConstructInnerText, replacementValue);
                }
                else 
                {   
                    // we remove any sections that were not used
                    newConfig = newConfig.Replace(ifElseConstruct.IfElseConstructInnerText, String.Empty);
                }

                if (System.Diagnostics.Debugger.IsAttached)
                {
                    ifElseConstruct.Print();
                }
            }

            if (outFile == String.Empty)
                outFile = Path.GetFileNameWithoutExtension(inFile) + "-" + environment + Path.GetExtension(inFile);

            File.WriteAllText(outFile, newConfig); 
        }
    }
}
