using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Xml;
using System.Text.RegularExpressions;

namespace Snak.Core
{
    /// <summary>
    /// represents a base class for all visual studio projects
    /// </summary>
    /// <remarks>
    /// </remarks>
    public abstract class AbstractProject : IProjectInfo
    {
        #region AbstractProject properties

        private XmlDocument _projectDoc = null;

        protected XmlDocument ProjectDoc
        {
            get { return _projectDoc; }
            private set { _projectDoc = value; }
        }

        private XmlNamespaceManager _projectDocXMLNamespaceManager = null;

        protected XmlNamespaceManager ProjectDocXMLNamespaceManager
        {
            get { return _projectDocXMLNamespaceManager; }
            set { _projectDocXMLNamespaceManager = value; }
        }

        private AbstractXmlProjectFileParser _abstractXmlProjectFileParser;

        protected AbstractXmlProjectFileParser XmlProjectDocParser
        {
            get { return _abstractXmlProjectFileParser; }
            private set { _abstractXmlProjectFileParser = value; }
        }

        private string _solutionsProjectSection = String.Empty;

        protected string SolutionsProjectSection
        {
            get { return _solutionsProjectSection; }
            private set { _solutionsProjectSection = value; }
        }

        private bool _projectHasPhysicalFile = false;

        /// <summary>
        /// for example a VS 2005 web site does not have a project file, its project info is stored in the .sln
        /// </summary>
        protected bool ProjectHasPhysicalFile
        {
            get { return _projectHasPhysicalFile; }
            private set { _projectHasPhysicalFile = value; }
        }

        #endregion

        #region constructors

        /// <summary>
        /// 
        /// </summary>
        /// <param name="projectFile"></param>
        /// <param name="buildConfigurationToLoad"></param>
        protected AbstractProject(FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : this(Guid.Empty, String.Empty, projectFile, buildConfigurationToLoad)
        { 
        
        }

        /// <summary>
        /// Creates a new AbstractProject and calls functionality in abstract methods to populate internal values
        /// </summary>
        /// <param name="id"></param>
        /// <param name="projectName"></param>
        /// <param name="projectFile"></param>
        /// <param name="activeBuildConfiguration"></param>
        protected AbstractProject(Guid id, string projectName, FileInfo projectFile, BuildConfiguration activeBuildConfiguration)
            : this(id, projectName, String.Empty, projectFile, null, activeBuildConfiguration)
        { 
        
        }

        /// <summary>
        /// Creates a new AbstractProject, this overload is useful for projects that dont have a project file (i.e. 2005 'Web Site' projects)
        /// </summary>
        /// <param name="id"></param>
        /// <param name="projectName"></param>
        /// <param name="solutionsProjectSection"></param>
        /// <param name="projectDir"></param>
        /// <param name="activeBuildConfiguration"></param>
        protected AbstractProject(Guid id, string projectName, string solutionsProjectSection, DirectoryInfo projectDir, BuildConfiguration activeBuildConfiguration)
            : this(id, projectName, solutionsProjectSection, null, projectDir, activeBuildConfiguration)
        {

        }

        private AbstractProject(Guid id, string projectName, string solutionsProjectSection, FileInfo projectFile, DirectoryInfo projectDir, BuildConfiguration activeBuildConfiguration)
        {
            if (activeBuildConfiguration == null)
                throw new ArgumentNullException("activeBuildConfiguration");

            this.ID = id;
            this.ProjectName = projectName;
            this.SolutionsProjectSection = solutionsProjectSection;
            this.ActiveBuildConfiguration = activeBuildConfiguration;
            this.ProjectHasPhysicalFile = (projectFile != null);               

            if (this.ProjectHasPhysicalFile)
            {
                if (!projectFile.Exists)
                    throw new FileNotFoundException("Failed to find project file", projectFile.FullName);

                if (!projectFile.Extension.EndsWith("proj"))
                    throw new ArgumentException(string.Format("{0} is not a valid project file. Invalid extension.", projectFile.FullName));

                this.ProjectFile = projectFile;
                this.ProjectDir = new DirectoryInfo(this.ProjectFile.DirectoryName);
            }
            else
            {
                if (projectDir == null)
                    throw new SnakException("The projectDir property cannot be null for projects without a physical project file, (i.e. some web site projects).");

                this.ProjectDir = projectDir;
            }


            InternalLoad();  
        }

        #endregion

        #region abstract methods

        protected abstract XmlNamespaceManager GetXmlXMLNamespaceManagerForProjectFile();

        /// <summary>
        /// Sets properties (mainly the IProjectInfo related properties) based on information read in from the project file.
        /// </summary>
        protected abstract void LoadNonBuildConfigSpeificProperties();

        /// <summary>
        /// Sets properties (mainly the IProjectInfo related properties) based on information contained in the protected property 
        /// </summary>
        protected virtual void LoadNonBuildConfigSpeificProperties_ForFilelessProjects() {}

        /// <summary>
        /// Returns a Dictionary with IProjectBuildConfig information relating the specific project configuration
        /// </summary>
        /// <returns></returns>
        protected abstract void LoadProjectSettingsForBuildConfig(BuildConfiguration buildConfiguration);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="projectDocument"></param>
        /// <param name="xmlNamespaceManager"></param>
        /// <returns></returns>
        protected abstract AbstractXmlProjectFileParser GetAbstractXmlProjectFileParser(XmlDocument projectDocument, XmlNamespaceManager xmlNamespaceManager);

        #endregion

        #region private methods

        /// <summary>
        /// 
        /// </summary>
        private void InternalLoad()
        {
            if (this.ProjectHasPhysicalFile)
            {
                // load the project file up so we can start setting properties 
                this.ProjectDoc = LoadXmlDocument(this.ProjectFile.FullName);
                this.ProjectDocXMLNamespaceManager = GetXmlXMLNamespaceManagerForProjectFile();
                this.XmlProjectDocParser = this.GetAbstractXmlProjectFileParser(this.ProjectDoc, this.ProjectDocXMLNamespaceManager);

                this.LoadNonBuildConfigSpeificProperties();
            }
            else
                this.LoadNonBuildConfigSpeificProperties_ForFilelessProjects();

            // confirm some key properties were set 
            if (this.ID == null || this.ID == Guid.Empty)
                throw new SnakException("The ID should have been set by one of the abstract Load methods.");

            if (this.ProjectName == null || (this.ProjectName != null && this.ProjectName == string.Empty))
                throw new SnakException("Project name not set.");

            this.LoadProjectSettingsForBuildConfig(this.ActiveBuildConfiguration);
        }

        private XmlDocument LoadXmlDocument(string documentPath)
        {
            XmlDocument projectDoc = new XmlDocument();
            using (FileStream projectFileStream = File.OpenRead(documentPath))
                projectDoc.Load(projectFileStream);
            return projectDoc;
        }

        #endregion

        #region IProjectInfo Members

        private bool _build;

        public bool Build
        {
            get { return _build; }
            set { _build = value; }
        }
	
        private BuildConfiguration _activeBuildConfiguration = null;

        public BuildConfiguration ActiveBuildConfiguration
        {
            get { return (this._activeBuildConfiguration); }
            set { this._activeBuildConfiguration = value; }
        }
        
        #region IProjectProperties Members

        private string _projectName = String.Empty;

        public string ProjectName
        {
            get { return (this._projectName); }
            protected set { this._projectName = value; }
        }

        public virtual string ProjectSafeName
        {
            get 
            {
                return Regex.Replace(this.ProjectName, @"[\\:\.]", "");// removes . and \ and : from the project name
                // [\\:]*(\.\.)? // this pattern leaves single dots
            }
        }

        private Guid _id = Guid.Empty;

        public Guid ID
        {
            get { return (this._id); }
            protected set { this._id = value; }
        }

        private FileInfo _projectFile;

        public FileInfo ProjectFile
        {
            get { return (this._projectFile); }
            protected set { this._projectFile = value; }
        }

        public string ProjectFileExtension
        {
            get
            {
                return (this.ProjectFile.Extension);
            }
        }

        public string CodeFileExtension
        {
            get
            {
                switch (this.ProjectLanguage)
                {
                    case ProjectLanguage.CSHARP:
                        return ".cs";
                    case ProjectLanguage.VisualBasic:
                        return ".vb";
                    default:
                        throw new InvalidEnumArgumentException("Unknown project language " + ProjectLanguage);
                }
            }
        }

        private DirectoryInfo _projectDir;

        public DirectoryInfo ProjectDir
        {
            get { return (this._projectDir); }
            private set { this._projectDir = value;  }
        }

        private ProjectLanguage _projectLanguage;

        public ProjectLanguage ProjectLanguage
        {
            get { return (this._projectLanguage); }
            protected set { this._projectLanguage = value; }
        }

        private string _assemblyName;

        public string AssemblyName
        {
            get { return (this._assemblyName); }
            protected set { this._assemblyName = value; }
        }

        /// <summary>
        /// The name of the project's output assembly with the file extension
        /// </summary>
        public string AssemblyFileName
        {
            get
            {
                switch (this.ProjectOutputType)
                {
                    case ProjectOutputType.Exe:
                    case ProjectOutputType.WinExe:
                    default:
                        return this.AssemblyName + ".exe";

                    case ProjectOutputType.Library:
                        return this.AssemblyName + ".dll";
                }
            }
        }

        private ProjectOutputType _projectOutputType;

        public ProjectOutputType ProjectOutputType
        {
            get { return (this._projectOutputType); }
            protected set { this._projectOutputType = value; }
        }

        private ProjectType _projectType;

        public ProjectType ProjectType
        {
            get { return (this._projectType); }
            protected set { this._projectType = value; }
        }

        public string ConfigFile
        {
            get
            {
                if (ProjectType == ProjectType.WebApplication || ProjectType == ProjectType.WebSite)
                    return "Web.config";
                else
                    return "App.config";
            } 
        }

        public abstract List<String> GetProjectFiles();

        public abstract List<String> GetProjectFiles(ProjectFileType contentType);

        private DotNetFrameworkVersion _dotNetFrameworkVersion;

        public DotNetFrameworkVersion DotNetFrameworkVersion
        {
            get { return (this._dotNetFrameworkVersion); }
            protected set { this._dotNetFrameworkVersion = value; }
        }

        private IProjectBuildProperties _projectBuildProperties;

        public IProjectBuildProperties IProjectBuildProperties
        {
            get { return _projectBuildProperties ; }
            protected set { this._projectBuildProperties = value; }
        }

        #endregion
        
        #endregion
    }
}
