using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace Snak.Core
{
    /// <summary>
    /// represents a base class for all project properties that are related to a build configuration as apposed to generic properties related to all build configurations.
    /// </summary>
    public abstract class AbstractProjectBuildProperties : IProjectBuildProperties
    {
        public AbstractProjectBuildProperties(AbstractProject parentProject)
        {
            this._parentProject = parentProject;
        }

        #region public properties

        private BuildConfiguration _buildConfiguration;

        protected BuildConfiguration BuildConfiguration
        {
            get { return (this._buildConfiguration); }
            set { this._buildConfiguration = value; }
        }

        private AbstractProject _parentProject;

        protected AbstractProject ParentProject
        {
            get { return (this._parentProject); }
            private set { this._parentProject = value; }
        }

        private AbstractXmlProjectFileParser _abstractXmlProjectFileParser;

        protected AbstractXmlProjectFileParser AbstractXmlProjectFileParser
        {
            get { return (this._abstractXmlProjectFileParser); }
            set { this._abstractXmlProjectFileParser = value; }
        }

        #endregion 

        #region IProjectBuildProperties Members

        #region common project values (common to most VS projects)

        private bool _allowUnsafeBlocks;

        public bool AllowUnsafeBlocks
        {
            get { return (this._allowUnsafeBlocks); }
            protected set { this._allowUnsafeBlocks = value; }
        }

        private uint _baseAddress;

        public uint BaseAddress
        {
            get { return (this._baseAddress); }
            protected set { this._baseAddress = value; }
        }

        private bool _checkForOverflowUnderflow;

        public bool CheckForOverflowUnderflow
        {
            get { return (this._checkForOverflowUnderflow); }
            protected set { this._checkForOverflowUnderflow = value; }
        }

        private FileInfo _configurationOverrideFile;

        public FileInfo ConfigurationOverrideFile
        {
            get { return (this._configurationOverrideFile); }
            protected set { this._configurationOverrideFile = value; }
        }

        private List<string> _defineConstants;

        public List<string> DefineConstants
        {
            get { return (this._defineConstants); }
            protected set { this._defineConstants = value; }
        }

        private FileInfo _documentationFile;

        public FileInfo DocumentationFile
        {
            get { return (this._documentationFile); }
            protected set { this._documentationFile = value; }
        }

        private bool _debugSymbols;

        public bool DebugSymbols
        {
            get { return (this._debugSymbols); }
            protected set { this._debugSymbols = value; }
        }

        private string _fileAlignment;

        public string FileAlignment
        {
            get { return (this._fileAlignment); }
            protected set { this._fileAlignment = value; }
        }

        private bool _incrementalBuild;

        public bool IncrementalBuild
        {
            get { return (this._incrementalBuild); }
            protected set { this._incrementalBuild = value; }
        }

        private bool _noStdLib;

        public bool NoStdLib
        {
            get { return (this._noStdLib); }
            protected set { this._noStdLib = value; }
        }

        private string _noWarn;

        public string NoWarn
        {
            get { return (this._noWarn); }
            protected set { this._noWarn = value; }
        }

        private bool _optimize;

        public bool Optimize
        {
            get { return (this._optimize); }
            protected set { this._optimize = value; }
        }

        private bool _registerForComInterop;

        public bool RegisterForComInterop
        {
            get { return (this._registerForComInterop); }
            protected set { this._registerForComInterop = value; }
        }

        private bool _removeIntegerChecks;

        public bool RemoveIntegerChecks
        {
            get { return (this._removeIntegerChecks); }
            protected set { this._removeIntegerChecks = value; }
        }

        private bool _treatWarningsAsErrors;

        public bool TreatWarningsAsErrors
        {
            get { return (this._treatWarningsAsErrors); }
            protected set { this._treatWarningsAsErrors = value; }
        }

        private int _warningLevel;

        public int WarningLevel
        {
            get { return (this._warningLevel); }
            protected set { this._warningLevel = value; }
        }

        private string _debugType;

        public string DebugType
        {
            get { return (this._debugType); }
            protected set { this._debugType = value; }
        }

        private string _errorReport;

        public string ErrorReport
        {
            get { return (this._errorReport); }
            protected set { this._errorReport = value; }
        }

        private string _outputPath;

        public string OutputPath
        {
            get { return (this._outputPath); }
            protected set { this._outputPath = value; }
        }

        private string _outputPathFull;

        public virtual string OutputPathFull
        {
            get { return (this._outputPathFull); }
            protected set { this._outputPathFull = value; }
        }

        /// <summary>
        /// The complete path to the project's output assembly
        /// </summary>
        public virtual string OutputFilePath
        {
            get { return Path.Combine(this.OutputPathFull, this.ParentProject.AssemblyFileName); }
        }

        /// <summary>
        /// Either a specially named 'nunit.config' in the project root,
        /// or the same as the project's configuration file, or
        /// </summary>
        public FileInfo TestingConfigFile
        {
            get
            {
                string testingConfig = Path.Combine(this.ParentProject.ProjectDir.FullName, "nunit.config");
                string binConfig = this.OutputFilePath + ".config";
                string binConfigParent = Path.Combine(this.OutputPathFull, "..\\") + this.ParentProject.AssemblyFileName + ".config";

                // fall through a couple of attempts

                if (File.Exists(binConfig))
                    return new FileInfo(binConfig);	// bin/assemblyname.dll.config
                else if (File.Exists(binConfigParent))
                    return new FileInfo(binConfigParent);	// assemblyname.dll.config (or bin/assemblyname.dll.config if above was in bin/debug)
                else if (File.Exists(testingConfig))
                    return new FileInfo(testingConfig);	// nunit.config
                else	// app.config / web.config
                    return new FileInfo(Path.Combine(this.ParentProject.ProjectDir.FullName, this.ParentProject.ConfigFile));
            }
        }

        #endregion

        #region web site project based values

        private string _virtualPath;

        public string VirtualPath
        {
            get { return (this._virtualPath); }
            protected set { this._virtualPath = value; }
        }

        private string _physicalPath;

        public string PhysicalPath
        {
            get { return (this._physicalPath); }
            protected set { this._physicalPath = value; }
        }

        private string _targetPath;

        public string TargetPath
        {
            get { return (this._targetPath); }
            protected set { this._targetPath = value; }
        }

        private bool _updateable;

        public bool Updateable
        {
            get { return (this._updateable); }
            protected set { this._updateable = value; }
        }

        private bool _forceOverwrite;

        public bool ForceOverwrite
        {
            get { return (this._forceOverwrite); }
            protected set { this._forceOverwrite = value; }
        }

        private FileInfo _keyFile;

        public FileInfo KeyFile
        {
            get { return (this._keyFile); }
            protected set { this._keyFile = value; }
        }

        private bool _delaySign;

        public bool DelaySign
        {
            get { return (this._delaySign); }
            protected set { this._delaySign = value; }
        }

        private string _keyContainer;

        public string KeyContainer
        {
            get { return (this._keyContainer); }
            protected set { this._keyContainer = value; }
        }

        private bool _allowPartiallyTrustedCallers;

        public bool AllowPartiallyTrustedCallers
        {
            get { return (this._allowPartiallyTrustedCallers); }
            protected set { this._allowPartiallyTrustedCallers = value; }
        }

        private bool _fixedNames;

        public bool FixedNames
        {
            get { return (this._fixedNames); }
            protected set { this._fixedNames = value; }
        }

        private int _vwdPort;

        public int VWDPort
        {
            get { return (this._vwdPort); }
            protected set { this._vwdPort = value; }
        }

        private ProjectLanguage _defaultWebSiteLanguage;

        public ProjectLanguage DefaultWebSiteLanguage
        {
            get { return (this._defaultWebSiteLanguage); }
            protected set { this._defaultWebSiteLanguage = value; }
        }

        #endregion

        #endregion
    }
}
