using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using Snak.Core;
using NUnit.Framework;
using NAnt.VSNet.Types;

namespace Snak.Core
{
    public abstract class AbstractSolution : ISolutionInfo, IEnumerable<IProjectInfo>
    {
        #region public interface

        private readonly ConfigurationManagerState _configurationManager;

        public ConfigurationManagerState ConfigurationManager
        {
            get { return _configurationManager; }
        }

        #region ISolutionInfo Members

        private FileInfo _solutionFile;

        public FileInfo SolutionFile
        {
            get { return _solutionFile; }
            protected set { _solutionFile = value; }
        }

        private Dictionary<String, IProjectInfo> _projectsByName;

        public Dictionary<String, IProjectInfo> ProjectsByName
        {
            get { return _projectsByName; }
            private set { _projectsByName = value; }
        }

        private Dictionary<Guid, IProjectInfo> _projectsByGuid;

        public Dictionary<Guid, IProjectInfo> ProjectsByGuid
        {
            get { return _projectsByGuid; }
            private set { _projectsByGuid = value; }
        }

        private WebMapCollection _webMaps = null;

        public WebMapCollection WebMaps
        {
            get { return _webMaps; }
            protected set { _webMaps = value; }
        }

        #endregion

        #region IEnumerable<IProjectInfo> Members

        /// <summary>
        /// Returns projects for the active solution configuration
        /// </summary>
        /// <returns></returns>
        IEnumerator<IProjectInfo> IEnumerable<IProjectInfo>.GetEnumerator()
        {
            if (this.ConfigurationManager.ActiveBuildConfiguration == null)
                throw new ApplicationException("You must set the AbstractSolution.ConfigurationManager.ActiveBuildConfiguration value before enumerating over the projects in the solution.");

            foreach (IProjectInfo project in this.ProjectsByName.Values)
                yield return project;
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return ((IEnumerable<IProjectInfo>)this).GetEnumerator();
        }

        #endregion

        #endregion

        #region protected properties

        private Dictionary<Guid, SolutionsProjectInfo> _solutionsProjectInfos;

        protected Dictionary<Guid, SolutionsProjectInfo> SolutionsProjectInfos
        {
            get { return _solutionsProjectInfos; }
            set { _solutionsProjectInfos = value; }
        }

        private string _solutionFileContents;

        protected string SolutionFileContents
        {
            get { return _solutionFileContents; }
            private set { _solutionFileContents = value; }
        }

        #endregion

        #region constructors

        protected AbstractSolution(FileInfo solutionFile) : this(solutionFile, null, new WebMapCollection()) { }

        protected AbstractSolution(FileInfo solutionFile, WebMapCollection webMaps) : this(solutionFile, null, webMaps) { }

        protected AbstractSolution(FileInfo solutionFile, BuildConfiguration initialBuildConfigurationToLoad) : this(solutionFile, initialBuildConfigurationToLoad, new WebMapCollection()) { }

        protected AbstractSolution(FileInfo solutionFile, BuildConfiguration initialBuildConfigurationToLoad, WebMapCollection webMaps)
        {
            if (solutionFile == null)
                throw new ArgumentNullException("solutionFile");

            if (!solutionFile.Exists)
                throw new ArgumentException("The solutionFile does not exist");

            this.SolutionFile = solutionFile;
            this.SolutionFileContents = File.ReadAllText(this.SolutionFile.FullName, Encoding.Default);
            this._configurationManager = new ConfigurationManagerState(initialBuildConfigurationToLoad, this.ReloadOnConfigurationChangeDelegate);
            this._webMaps = webMaps;

            Load();
        }

        #endregion

        #region internal interface

        internal void Load()
        {
            if (this.SolutionFile == null || !this.SolutionFile.Exists)
                throw new ApplicationException("You must first set the SolutionFile.");

            InternalLoad();
        }

        #endregion

        #region abstract and virtual methods

        /// <summary>
        /// Keyed on project name, this Dictionary contains the SolutionsProjectInfo's as created form info parsed in the solution file
        /// </summary>
        /// <returns></returns>
        /// <param name="solutionFileContents"></param>
        /// <param name="webMaps">a collection of web maps used to map a url to a project found in the solutionFileContents to a project file. Can be ignored and may be null for 2005 projects.</param>
        protected abstract Dictionary<Guid, SolutionsProjectInfo> GetSolutionsProjectInfos(string solutionFileContents, WebMapCollection webMaps);

        /// <summary>
        /// Keyed on the BuildConfiguration (e.g. this object contains the "Debug | Any Cpu" settings), this Dictionary contains another dictionary
        /// that is keyed on project name and its value is a reference a SolutionsProjectInfo as constructed on information read in from the solution file
        /// </summary>
        /// <param name="solutionFile"></param>
        /// <returns></returns>
        protected abstract Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> GetSolutionConfigMappingToProjectConfig(string solutionFileContents);

        /// <summary>
        /// Creates an IProjectInfo project
        /// </summary>
        /// <param name="solutionsProjectInfo"></param>
        /// <returns></returns>
        protected abstract IProjectInfo CreateProject(Guid id, string projectName, FileInfo projectFile, DirectoryInfo projectDir, string projectSectionFromSolutionFile, BuildConfiguration buildConfigurationToLoad);

        #endregion

        #region private

        private void InternalLoad()
        {
            // call out to the concrete implementation to get the project information from the solution file
            this.SolutionsProjectInfos = GetSolutionsProjectInfos(this.SolutionFileContents, this.WebMaps);

            // here we call out to the concrete implementation to get the information about what projects configuration we will use for 
            // solutions configuration. Normally if the solution was configured to build as Debug, each project in the solution would also 
            // be build in debug, however its entirely possible that the solution be set to Debug and one or more of the solutions projects be 
            // set to another configuration. This is why we store a Dictionary keyed on the solutions configuration and its values are all the 
            // project configurations for the given solutions configuration 

            this.ConfigurationManager.SolutionsProjectConfigsKeyedOnSolutionsBuildConfig = GetSolutionConfigMappingToProjectConfig(this.SolutionFileContents);

            // here we load the projects
            InternalLoadProjects();
        }

        /// <summary>
        /// loads all projects for the active build configuration
        /// </summary>
        private void InternalLoadProjects()
        {
            if (this.ConfigurationManager.ActiveBuildConfiguration != null)
            {
                IProjectInfo newProject = null;
                SolutionsProjectConfig solutionsProjectConfig = null;
                this.ProjectsByName = new Dictionary<string, IProjectInfo>();
                this.ProjectsByGuid = new Dictionary<Guid, IProjectInfo>();

                // here we loop over the projects found in the solutions and create IProjectInfos from each one for the current _activeBuildConfiguration
                // we dont store every configuration as it becomes a bit of a maintenance  nightmare given that we dont actually use this class in a manor that 
                // needs all the configs at once. Instead we only focus on providing the project infos for one configuration at a time.

                Dictionary<Guid, SolutionsProjectConfig> configurationsToUse = this.ConfigurationManager.GetSolutionsProjectConfigsBasedOnActiveConfiguration();

                foreach (KeyValuePair<Guid, SolutionsProjectInfo> solutionsProjectInfoKeyValuePair in this.SolutionsProjectInfos)
                {
                    if (!this.ProjectsByName.ContainsKey(solutionsProjectInfoKeyValuePair.Value.ProjectName))
                    {
                        if (solutionsProjectInfoKeyValuePair.Value.ProjectFile == null && solutionsProjectInfoKeyValuePair.Value.ProjectSection == null)
                            throw new SnakException("When trying to create a project invalid information was produced to create it. Both the ProjectFile and the ProjectSection are null, we need one or the other to create a project.");

                        BuildConfiguration buildConfigurationForThisProject = configurationsToUse[solutionsProjectInfoKeyValuePair.Value.ProjectGuid].BuildConfiguration;

                        // call out to the concrete implementation of CreateProject to get the IProjectInfo 
                        newProject = this.CreateProject(solutionsProjectInfoKeyValuePair.Value.ProjectGuid, solutionsProjectInfoKeyValuePair.Value.ProjectName, solutionsProjectInfoKeyValuePair.Value.ProjectFile, solutionsProjectInfoKeyValuePair.Value.ProjectDir, solutionsProjectInfoKeyValuePair.Value.ProjectSection, buildConfigurationForThisProject);

                        solutionsProjectConfig = configurationsToUse[solutionsProjectInfoKeyValuePair.Value.ProjectGuid];
                        newProject.Build = solutionsProjectConfig.Build;

                        this.ProjectsByName.Add(newProject.ProjectName, newProject);
                        this.ProjectsByGuid.Add(newProject.ID, newProject);
                    }
                    else
                        throw new SnakException("Error creating projects for the solution, somehow a project with the same name has already been added to a list.");

                }
            }
        }

        #endregion

        private void ReloadOnConfigurationChangeDelegate()
        {
            this.InternalLoadProjects();
        }

        ///// <summary>
        ///// Tries to get the project file based on the provided string, if the provides string is deemed to represent a 2003 web project then the webmap collection 
        ///// will be used to determine the project file location, otherwise a FileInfo created with full path will be returned based on the solutions location and the given string.
        ///// </summary>
        ///// <param name="expectedProjectFileString"></param>
        ///// <returns></returns>
        ///// <remarks></remarks>
        //protected FileInfo TryGetProjectFileInfo(string expectedProjectFileString)
        //{
        //    // TODO: this method is kinda implementation specific as its got a 2003 check... perhaps the webmap stuff only needs to be in the everett code...

        //    FileInfo fileInfo = null;

        //    if (expectedProjectFileString.StartsWith("http://", StringComparison.OrdinalIgnoreCase))
        //    {
        //        // if were dealing with a web project (i.e. a 2003 web project)
        //        string mappedPath = _webMaps.FindBestMatch(expectedProjectFileString);

        //        if (mappedPath != null)
        //        {
        //            fileInfo = new FileInfo(mappedPath);
        //            AssertFileExists(fileInfo, "Unable to locate web project {0} after mapping to {1}", expectedProjectFileString, mappedPath);
        //        }
        //        else
        //        {
        //            // Attempt to find the project directly under the solution
        //            string projectFileName = Path.GetFileName(expectedProjectFileString);
        //            string projectDirectory = Path.GetFileName(Path.GetDirectoryName(expectedProjectFileString));
        //            string pathAttempted = Path.Combine(Path.Combine(this.SolutionFile.FullName, projectDirectory), projectFileName);
        //            FileInfo inferredFile = new FileInfo(pathAttempted);
        //            AssertFileExists(inferredFile, "Failed to infer path to web project {0} at {1}", expectedProjectFileString, inferredFile);
        //        }
        //    }
        //    else
        //    {
        //        if (Path.HasExtension(expectedProjectFileString))
        //        {
        //            string fullPath = Path.GetFullPath(Path.Combine(this.SolutionFile.DirectoryName, expectedProjectFileString));
        //            fileInfo = new FileInfo(fullPath);
        //        }
        //        else
        //        {
        //            // else do nothing as we could be dealing with a fileless project type such as the 2005 web site projects
        //            // throw new SnakException("Could not locate a project file given the expected address of '" + fileInfo.FullName + "'");

        //            // TODO: log this using some sort of static reference to the nant logger whenever i wire that up
        //        }
        //    }

        //    return fileInfo;
        //}


    }
}
