using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

namespace Snak.Core
{
    public class BuildConfiguration
    {
        // these 2 lines (from a .sln) tell us that for the solutions Debug|Any CPU configuration
        // it knows about a project C2F079AC-40E1-4923-A2CC-DEC94D681465, the knows about line is the one with ActiveCfg.
        // it also test us it will build the project for the solution config, we know this by Build.0
        //
        // {C2F079AC-40E1-4923-A2CC-DEC94D681465}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
		// {C2F079AC-40E1-4923-A2CC-DEC94D681465}.Debug|Any CPU.Build.0 = Debug|Any CPU
        // 
        // if we hadn't checked the 'build' checkbox in the configuration manager the second line above would
        // not apear in the solution file.

        // nearly went down this path ConfigurationManager Interface http://msdn2.microsoft.com/en-us/library/envdte.configurationmanager.aspx
        private readonly string _config = String.Empty;
        
        public string Config
        {
            get
            {
                return (this._config);
            }
        }

        private readonly string _platform = String.Empty;

        public string Platform
        {
            get
            {
                return (this._platform);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="config">e.g. 'Debug' or perhaps 'Debug|.Net' in the case of the later the first component (before the pipe symbol) will be taken as the config value and the later as the platform value.</param>
        public BuildConfiguration(string config) : this(config, String.Empty)
        {
         
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="config">e.g. 'Debug'</param>
        /// <param name="platform">e.g. 'Any Cpu'</param>
        /// <remarks>if the value for platform is null or empty and the config param contains a pipe symbol (e.g. 'Debug|.Net') then the first component (before the pipe symbol) will be taken as the config value and the later as the platform value.</remarks>
        public BuildConfiguration(string config, string platform)
        {
            if (config == null)
                throw new ArgumentNullException("config");
            
            if (config == String.Empty)
                throw new ArgumentException("config cannot be empty", "config");

            if (config.Contains("|"))
            {
                if (platform != null && platform != String.Empty)
                    throw new SnakException("The config value passed to the constructor apears to contain both a config and a platform value and you have also specified a platform in another param");

                string[] parts = config.Split('|');

                _config = parts[0];
                _platform = parts[1];
            }
            else
            {
                _config = config;
                _platform = platform;
            }
        }

        public override string ToString()
        {
            return this.Config + ((this.Platform != String.Empty) ?  "|" + this.Platform : String.Empty);
        }

        public string ToSafeDirNameString()
        {
            return (this.Config + "-" + this.Platform).Replace(" ", "");
        }

        /// <summary>
        /// Performs a case insensitive comparison of Config  and Platform to determine if the  provided BuildConfiguration are equal
        /// </summary>
        /// <param name="left"></param>
        /// <param name="right"></param>
        /// <returns></returns>
        public static bool operator ==(BuildConfiguration left, BuildConfiguration right)
        {
            // we store the left and right into 'objects' so we can do a null reff check on them
            // this is to avoid recursively calling this overload to check the null reff and
            // rather we call 'objects' equal operator
            object l = left;
            object r = right;

            if (l == null && r == null)
                return true;

            if (l == null || r == null)
                return false;

            if ((String.Compare(left.Config, right.Config, true) == 0) && (String.Compare(left.Platform, right.Platform, true) == 0))
                return true;
            else
                return false;        
        }

        public static bool operator !=(BuildConfiguration left, BuildConfiguration right)
        {
            return !(left == right);
        }

        public override bool Equals(object obj)
        {
            if (!(obj is BuildConfiguration))
            {
                return false;
            }

            return this == (BuildConfiguration)obj;
        }

        public override int GetHashCode()
        {
            return this.ToString().GetHashCode();
        }

        [TestFixture()]
        public class BuildConfigurationTester
        {
            [Test()]
            public void TestOperatorOverloading()
            {
                BuildConfiguration debugAnyCpuConfig_1 = new BuildConfiguration("Debug", "Any CPU");
                BuildConfiguration debugAnyCpuConfig_2 = new BuildConfiguration("Debug", "Any CPU");
                BuildConfiguration releaseConfig_1 = new BuildConfiguration("Release", "Any CPU");
                BuildConfiguration releaseConfig_2 = new BuildConfiguration("Release", "Any CPU");

                Assert.IsTrue(debugAnyCpuConfig_1 == debugAnyCpuConfig_2);
                Assert.IsFalse(releaseConfig_1 == debugAnyCpuConfig_1);
                Assert.IsTrue(releaseConfig_1 != debugAnyCpuConfig_1);
                Assert.IsFalse(debugAnyCpuConfig_1 != debugAnyCpuConfig_2);
            }

            [Test()]
            public void TestToString()
            {
                BuildConfiguration debugAnyCpuConfig_1 = new BuildConfiguration("Debug", "Any CPU");

                Assert.IsTrue(String.Compare("Debug|Any CPU", debugAnyCpuConfig_1.ToString(), false) == 0);
            }
        }
    }
}
