using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Xml;
using Snak.Utilities;

namespace Snak.Core.Everett
{
	/// <summary>
	/// Represents a project (and a particular configuration) within a Visual Studio 2003 solution
	/// </summary>
	public class EverettProjectInfo : AbstractProject
	{

        internal EverettProjectInfo(FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(projectFile, buildConfigurationToLoad)
        {

        }

        internal EverettProjectInfo(Guid id, string projectName, FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, projectFile, buildConfigurationToLoad)
        {

        }

        internal EverettProjectInfo(Guid id, string projectName, string solutionsProjectSection, DirectoryInfo projectDir, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, solutionsProjectSection, projectDir, buildConfigurationToLoad)
        {

        }

        protected override XmlNamespaceManager GetXmlXMLNamespaceManagerForProjectFile()
        {
            // TODO: perhaps mark this as virtual as apposed to abstract in the base class
            return null;
        }

        protected override void LoadNonBuildConfigSpeificProperties()
        {
            if (this.ID == Guid.Empty)
                this.ID = this.XmlProjectDocParser.GetPropertySetting<Guid>("ProjectGuid");

            this.ProjectType = this.XmlProjectDocParser.GetPropertySetting<ProjectType>("ProjectType");
            this.ProjectLanguage = (this.ProjectFile.Extension.ToLower() == ".vbproj") ? ProjectLanguage.VisualBasic : ProjectLanguage.CSHARP;
            this.AssemblyName = this.XmlProjectDocParser.GetPropertySetting<String>("AssemblyName");
            this.DotNetFrameworkVersion = DotNetFrameworkVersion.v1_1;
            this.ProjectName = Path.GetFileNameWithoutExtension(this.ProjectFile.FullName);

            if (this.ProjectType == ProjectType.VisualStudioDeployment)
                this.ProjectOutputType = ProjectOutputType.Deployment;
            else
                this.ProjectOutputType = this.XmlProjectDocParser.GetPropertySetting<ProjectOutputType>("OutputType");
        }

        protected override void LoadProjectSettingsForBuildConfig(BuildConfiguration buildConfiguration)
        {
            AbstractProjectBuildProperties abstractProjectBuildConfig = null;

            abstractProjectBuildConfig = new EverettProjectBuildConfig(buildConfiguration, this.XmlProjectDocParser, this);

            this.IProjectBuildProperties = abstractProjectBuildConfig;
        }

        protected override AbstractXmlProjectFileParser GetAbstractXmlProjectFileParser(XmlDocument projectDocument, XmlNamespaceManager xmlNamespaceManager)
        {
            return new EverettXmlProjectFileParser(projectDocument, xmlNamespaceManager);
        }

		/// <summary>
		/// Gets an array of all the files contained within the project
		/// All paths are project-relative
		/// </summary>
        public override List<string> GetProjectFiles()
        {
            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded...

			XmlDocument projectXml = LoadXmlDocument(this.ProjectFile.FullName);
			XmlNodeList fileNodes = projectXml.SelectNodes("//Files/Include/File");
			return GetProjectFiles(fileNodes);
		}

		/// <summary>
		/// Gets an array of all the files contained within the project
		/// that match the content type provided (eg 'Content' for web pages).
		/// All paths are project-relative
		/// </summary>
        public override List<string> GetProjectFiles(ProjectFileType contentType)
        {
            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded...

			XmlDocument projectXml = LoadXmlDocument(this.ProjectFile.FullName);
			XmlNodeList fileNodes = projectXml.SelectNodes(string.Format("//Files/Include/File[@BuildAction='{0}']", contentType));
			return GetProjectFiles(fileNodes);
		}

        private List<string> GetProjectFiles(XmlNodeList fileNodes)
        {
            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded...

            List<string> files = new List<string>(fileNodes.Count);

			foreach (XmlElement fileNode in fileNodes)
			{
				files.Add(fileNode.GetAttribute("RelPath"));
			}
            return files;
		}

		private static XmlDocument LoadXmlDocument(string documentPath)
		{
			XmlDocument projectDoc = new XmlDocument();
			using(FileStream projectFileStream = File.OpenRead(documentPath))
				projectDoc.Load(projectFileStream);
			return projectDoc;
		}

#if( UNITTEST )
        [NUnit.Framework.TestFixture]
        public class EverettProjectInfoTester
        {
            // this assumes that the tests are always going to be run from the \bin\debug directory
            private const string PROJECT_NAME = @"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\ClassLibrary\ClassLibrary1\ClassLibrary1.csproj";
            private string testProjectRoot = String.Empty;

            [NUnit.Framework.TestFixtureSetUp]
            public void Init()
            {
                testProjectRoot = Path.GetFullPath(Path.GetDirectoryName(PROJECT_NAME)).ToLower();
            }

            [NUnit.Framework.Test(Description = "")]
            public void TestLoadProject()
            {
                EverettProjectInfo project = new EverettProjectInfo(new FileInfo(Path.GetFullPath(PROJECT_NAME)), new BuildConfiguration("Debug"));

                AssertEqualIgnoreCase("ClassLibrary1", project.ProjectName);
                AssertEqualIgnoreCase("Debug", project.ActiveBuildConfiguration.ToString());
                AssertEqualIgnoreCase("ClassLibrary1.dll", project.AssemblyFileName);
                AssertEqualIgnoreCase("ClassLibrary1", project.AssemblyName);
                AssertEqualIgnoreCase(".csproj", project.ProjectFileExtension);
                AssertEqualIgnoreCase(".cs", project.CodeFileExtension);
                AssertEqualIgnoreCase(testProjectRoot, project.ProjectDir.FullName);
                AssertEqualIgnoreCase(@"bin\debug\", project.IProjectBuildProperties.OutputPath);
                AssertEqualIgnoreCase(Path.Combine(testProjectRoot, @"bin\debug\"), project.IProjectBuildProperties.OutputPathFull);
                AssertEqualIgnoreCase(Path.Combine(testProjectRoot, @"bin\debug\ClassLibrary1.dll"), project.IProjectBuildProperties.OutputFilePath);
                NUnit.Framework.Assert.AreEqual(ProjectOutputType.Library, project.ProjectOutputType);
            }

            private void AssertEqualIgnoreCase(string expected, string actual)
            {
                NUnit.Framework.Assert.AreEqual(expected.ToLower(), actual.ToLower());
            }

            [NUnit.Framework.Test]
            public void TestGetProjectFiles()
            {
                Dictionary<ProjectFileType, String> expectedFiles = new Dictionary<ProjectFileType,string>();
                expectedFiles.Add(ProjectFileType.Compile, @"AssemblyInfo.cs");
                expectedFiles.Add(ProjectFileType.Content, "ContentItem.txt");
                expectedFiles.Add(ProjectFileType.EmbeddedResource, "EmbeddedItem.bmp");
                expectedFiles.Add(ProjectFileType.None, "NoneItem.txt");

                // Get the resource stream out and stick it on disk for the test
                string projectName = "TestCSharpWinformsProject2003.csproj";
                string tempFile = Path.Combine(Path.GetTempPath(), projectName);
                XmlDocument projectDocument = new XmlDocument();
                projectDocument.Load(ResourceUtils.GetResourceStream(GetType(), projectName));
                projectDocument.Save(tempFile);

                EverettProjectInfo otherProject = new EverettProjectInfo(new FileInfo(tempFile), new BuildConfiguration("Debug"));

                foreach (KeyValuePair<ProjectFileType, String> expectedFile in expectedFiles)
                {
                    ProjectFileType type = (ProjectFileType)expectedFile.Key;
                    List<String> compileItems = otherProject.GetProjectFiles(type);

                    if (type == ProjectFileType.None)
                        NUnit.Framework.Assert.AreEqual(2, compileItems.Count);
                    else
                        NUnit.Framework.Assert.AreEqual(1, compileItems.Count);

                    NUnit.Framework.Assert.AreEqual(expectedFile.Value, compileItems[0]);
                }
            }

            [NUnit.Framework.Test]
            [NUnit.Framework.ExpectedException(typeof(ArgumentException))]
            public void TestGetBadProject()
            {
                new EverettProjectInfo(new FileInfo(this.GetType().Assembly.Location), new BuildConfiguration("Debug"));
            }
        }

#endif
	}
}