using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
using NAnt.VSNet.Types;
using NUnit.Framework;

namespace Snak.Core.Everett
{
    /// <summary>
    /// Represents a Visual Studio 2003 solution file, and provides access to it's projects
    /// </summary>
    /// <remarks>
    /// <example>
    ///   <para>Some example</para>
    ///   <code>
    ///     <![CDATA[
    ///     some code here
    ///     ]]>
    ///   </code>
    /// </example>
    /// </remarks>
    public class EverettSolutionInfo : AbstractSolution
    {
        #region Declarations

        private const string PROJECT_PATTERN = @"Project\(""([^""]+)""\)\s*=\s*""([^""]+)"",\s*""([^""]+)"",\s""([^""]+)";
        private const string PROJECT_BUILD_PATTERN = @"(\{[^\}]+\})\.(?<config>[^\.]*)\.Build\.0\s*=\s*([^\|]+)\|\.NET"; //";
        private const string BUILD_TYPES_PATTERN = @"^\s*([\w]+)\s*=\s*\1\s*$";

        #endregion

        internal EverettSolutionInfo(FileInfo solutionFile, WebMapCollection webMaps) : base(solutionFile, webMaps) { }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="solutionFile"></param>
        /// <param name="activeBuildConfiguration"></param>
        internal EverettSolutionInfo(FileInfo solutionFile, BuildConfiguration activeBuildConfiguration, WebMapCollection webMaps) : base(solutionFile, activeBuildConfiguration, webMaps) { }

        protected override Dictionary<Guid, SolutionsProjectInfo> GetSolutionsProjectInfos(string solutionFileContents, WebMapCollection webMaps)
        {
            Regex projectMatcher = new Regex(PROJECT_PATTERN);
            MatchCollection matches = projectMatcher.Matches(solutionFileContents);
            Dictionary<Guid, SolutionsProjectInfo> solutionsProjectInfos = new Dictionary<Guid, SolutionsProjectInfo>(matches.Count);

            for (int projectIndex = 0; projectIndex < matches.Count; projectIndex++)
            {
                Match match = matches[projectIndex];

                string solutionGuid = match.Groups[1].Value;
                string name = match.Groups[2].Value;
                string location = match.Groups[3].Value;
                string projectGuid = match.Groups[4].Value;

                FileInfo projectFile = this.GetProjectFileInfo(location, webMaps);

                solutionsProjectInfos.Add(new Guid(projectGuid), new SolutionsProjectInfo(new Guid(projectGuid), new Guid(solutionGuid), name, projectFile, new DirectoryInfo(projectFile.DirectoryName), String.Empty, true));
            }

            return solutionsProjectInfos;
        }

        protected override Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> GetSolutionConfigMappingToProjectConfig(string solutionFileContents)
        {
            Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> projectConfigBySolutionConfig = new Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>>();
            Dictionary<Guid, SolutionsProjectConfig> solutionsProjectConfigs;
            SolutionsProjectConfig solutionsProjectConfig;
            BuildConfiguration buildConfiguration;

            string pattern = @"^*?(?<projectGuid>\{[^\}]*?\})\.(?<solutionConfig>[^\.]*)?\.(?<configMetaData>[^\s]*?)\s=\s(?<projectConfig>[^\|]*)?\|(?<projectPlatform>[^\r\n]*)";

            MatchCollection matchCollection = Regex.Matches(solutionFileContents, pattern);

            for (int matchIndex = 0; matchIndex < matchCollection.Count; matchIndex++)
            {
                Match projectConfigMatch = matchCollection[matchIndex];

                // get reference to the solution configuration we are currently dealing with
                string solutionConfig = projectConfigMatch.Groups["solutionConfig"].Value;

                buildConfiguration = new BuildConfiguration(solutionConfig);

                // check if we have already added an item for this solution config, if not we set it if so we get a reference to it 
                if (!projectConfigBySolutionConfig.ContainsKey(buildConfiguration))
                {
                    solutionsProjectConfigs = new Dictionary<Guid, SolutionsProjectConfig>();
                    projectConfigBySolutionConfig.Add(buildConfiguration, solutionsProjectConfigs);
                }
                else
                    solutionsProjectConfigs = projectConfigBySolutionConfig[buildConfiguration];

                Guid projectGuid = new Guid(projectConfigMatch.Groups["projectGuid"].Value);
                string projectConfig = projectConfigMatch.Groups["projectConfig"].Value;
                string projectPlatform = projectConfigMatch.Groups["projectPlatform"].Value;
                // In the following example the configMetaData will grab ActiveCfg, and Build for the first and second rows retrospectively
                //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.ActiveCfg = Release|Any CPU
                //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.Build.0 = Release|Any CPU    
                string configMetaData = projectConfigMatch.Groups["configMetaData"].Value;
                // the row with ActiveCfg always appears, if visual studio is meant to build the given project for the given config then 
                // where will also be a row with the .Build.0 part in it 
                bool build = configMetaData.Contains("Build");

                // as shown in the example above there may be more than one row for each project so we check if we've added it already
                if (!solutionsProjectConfigs.ContainsKey(projectGuid))
                {
                    solutionsProjectConfig = new SolutionsProjectConfig(this.SolutionsProjectInfos[projectGuid], new BuildConfiguration(projectConfig, projectPlatform), build);
                    solutionsProjectConfigs.Add(projectGuid, solutionsProjectConfig);
                }
                else
                    // if we've already added it we update it's build setting if necessary
                    if (build)
                        solutionsProjectConfigs[projectGuid].Build = true;
            }

            return projectConfigBySolutionConfig;
        }

        protected override IProjectInfo CreateProject(Guid id, string projectName, FileInfo projectFile, DirectoryInfo projectDir, string projectSectionFromSolutionFile, BuildConfiguration buildConfigurationToLoad)
        {
            IProjectInfo iProjectInfo;

            iProjectInfo = new EverettProjectInfo(id, projectName, projectFile, buildConfigurationToLoad);

            return iProjectInfo;
        }

        /// <summary>
        /// Gets a FileInfo of the project from the given string, if the given string is a url the given webmap collection will be used, if no map is found 
        /// an attempt to infer the path based on the solutions location and the name of the project file will be made.
        /// </summary>
        /// <param name="expectedProjectFileString"></param>
        /// <exception cref="FileNotFoundException">a FileNotFoundException will be thrown if the project file cannot be found</exception>
        /// <returns></returns>
        private FileInfo GetProjectFileInfo(string expectedProjectFileString, WebMapCollection webMapCollection)
        {
            FileInfo fileInfo = null;

            if (expectedProjectFileString.StartsWith("http://", StringComparison.OrdinalIgnoreCase))
            {
                string mappedPath = null;

                // if were dealing with a web project (i.e. a 2003 web project)
                if (webMapCollection != null)
                    mappedPath = webMapCollection.FindBestMatch(expectedProjectFileString);

                if (mappedPath != null)
                {
                    fileInfo = new FileInfo(mappedPath);
                    if (!fileInfo.Exists)
                        throw new FileNotFoundException(string.Format("Unable to locate web project {0} after mapping to {1}", expectedProjectFileString, mappedPath), fileInfo.FullName);
                }
                else
                {
                    // Attempt to find the project directly under the solution
                    string projectFileName = Path.GetFileName(expectedProjectFileString);
                    string projectDirectory = Path.GetFileName(Path.GetDirectoryName(expectedProjectFileString));
                    string soluionDirectory = Path.GetDirectoryName(this.SolutionFile.FullName);
                    string pathAttempted = Path.Combine(soluionDirectory, Path.Combine(projectDirectory, projectFileName));
                    //string p = Path.Combine(Path.Combine(Path.GetFileNameWithoutExtension(this.SolutionFile.FullName), projectDirectory), projectFileName);
                    //string pathAttempted = Path.Combine(Path.Combine(this.SolutionFile.FullName, projectDirectory), projectFileName);
                    FileInfo inferredFile = new FileInfo(pathAttempted);

                    if (!inferredFile.Exists)
                        throw new FileNotFoundException(string.Format("Failed to infer path to web project {0} at {1}", expectedProjectFileString, inferredFile), inferredFile.FullName);

                    fileInfo = inferredFile;
                }
            }
            else
            {
                if (Path.HasExtension(expectedProjectFileString))
                {
                    string fullPath = Path.GetFullPath(Path.Combine(this.SolutionFile.DirectoryName, expectedProjectFileString));
                    fileInfo = new FileInfo(fullPath);
                }
                else
                {
                    throw new SnakException("Could not locate a project file given the expected address of '" + fileInfo.FullName + "'");
                }
            }

            return fileInfo;
        }

#if(UNITTEST)

        [TestFixture]
        public class SolutionInfoTester
        {
            // this assumes that the tests are always going to be run from the \bin\debug directory
            private const string TEST_SOLUTION_FILE = @"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\SolutionWithMultipleDifferingProjects.sln";
            private FileInfo _testSolutionFile = new FileInfo(TEST_SOLUTION_FILE);
            private string _testSolutionFileContents = String.Empty;
            private WebMapCollection _webMapsForTestSolutionFile = null;
            private Guid _webApplication1_ID = new Guid("{1545A669-46DA-4F99-BA91-E2CA66C11E2B}");
            private Guid _webApplication1_BusinessLayer_ID = new Guid("{A761F6EF-7FF1-4343-A10E-BA75259BC811}");
            private Guid _webControlLibrary1_ID = new Guid("{36EC2461-8F33-4C63-A229-10729D9F1C89}");
            private Guid _classLibrary1_ID = new Guid("{FE1CD14C-63B9-45F4-8FEE-D592D79D74EE}");
            private Guid _consoleApplication1_ID = new Guid("{3BF349B0-BA0E-4A17-81B2-EE8C6558B4FC}");
            private Guid _windowsService1_ID = new Guid("{E8AED495-0ED6-418A-AC2F-0E5D66EFA098}");
            private Guid _webService1_ID = new Guid("{BC061A13-C81D-412E-8126-88DA2F5BDF35}");

            // below we create a wrapper for the class we want to test, as the test class is within the class on test we have access to the internals we want to test.
            // we could just have our test fixture inherit from the class we want to test but then  if something goes wrong in the constructor we cant really get 
            // meaningful feedback to the console about what went wrong. At least with the dummy wrapper we can wrap its instantiation in a try catch and dump any 
            // errors to the console before throwing the initial exception
            public class DummyEverettSolutionInfo : EverettSolutionInfo
            {
                public DummyEverettSolutionInfo(FileInfo solutionFile, WebMapCollection webMapCollection) : base(solutionFile, webMapCollection) { }
            }

            [TestFixtureSetUp]
            public void TestFixtureSetUp()
            {
                try
                {
                    _testSolutionFile = new FileInfo(Path.GetFullPath(TEST_SOLUTION_FILE));
                    _testSolutionFileContents = File.ReadAllText(_testSolutionFile.FullName);

                    _webMapsForTestSolutionFile = new WebMapCollection();
                    WebMap map = new WebMap();
                    map.Url = "http://localhost/WebApplication1/WebApplication1.csproj";
                    map.Path = new FileInfo(Path.Combine(Path.GetDirectoryName(_testSolutionFile.FullName), @"AspNetWebApplication\WebApplication1\WebApplication1.csproj"));
                    _webMapsForTestSolutionFile.Add(map);
                    map = new WebMap();
                    map.Url = "http://localhost/WebService1/WebService1.csproj";
                    map.Path = new FileInfo(Path.Combine(Path.GetDirectoryName(_testSolutionFile.FullName), @"AspNetWebApplication\WebService1\WebService1.csproj"));
                    _webMapsForTestSolutionFile.Add(map);
                }
                catch (Exception ex)
                {
                    throw new AssertionException("Error during a TestFixtureSetUp. See the inner exception.", ex);
                }
            }

            private string testSolutionRoot = String.Empty;

            [Test()]
            public void GetSolutionsProjectInfosTest()
            {
                try
                {
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(this._testSolutionFile, this._webMapsForTestSolutionFile);

                    string errorMessage = "Project from solution was not found in collection.";

                    Dictionary<Guid, SolutionsProjectInfo> solutionsProjectInfos = dummyEverettSolutionInfo.GetSolutionsProjectInfos(this._testSolutionFileContents, this._webMapsForTestSolutionFile);

                    Assert.AreEqual(7, solutionsProjectInfos.Count);

                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webApplication1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webApplication1_BusinessLayer_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webControlLibrary1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_classLibrary1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_consoleApplication1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_windowsService1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webService1_ID), errorMessage);

                    foreach (KeyValuePair<Guid, SolutionsProjectInfo> keyValuePair in solutionsProjectInfos)
                    {
                        SolutionsProjectInfo solutionsProjectInfo = keyValuePair.Value;

                        if (solutionsProjectInfo.ProjectName == null || solutionsProjectInfo.ProjectName == String.Empty)
                            Assert.Fail("ProjectName not set for project guid '" + keyValuePair.Key + "'");

                        if (solutionsProjectInfo.ProjectFile == null && solutionsProjectInfo.ProjectDir == null)
                            Assert.Fail("ProjectFile and or ProjectDir not set for project '" + solutionsProjectInfo.ProjectName + "'. For fileless project ProjectDir needs to be set for all others both need to be set.");

                        if (solutionsProjectInfo.ProjectDir == null)
                            Assert.Fail("ProjectDir not set for project '" + solutionsProjectInfo.ProjectName + "'.");

                        if (solutionsProjectInfo.ProjectGuid == Guid.Empty)
                            Assert.Fail("ProjectGuid not set for project '" + solutionsProjectInfo.ProjectName + "'");

                        if (solutionsProjectInfo.ProjectTypeGuid == Guid.Empty)
                            Assert.Fail("ProjectTypeGuid not set for project '" + solutionsProjectInfo.ProjectName + "'");
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            [Test()]
            public void GetSolutionConfigMappingToProjectConfigTest()
            {
                try
                {
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(this._testSolutionFile, this._webMapsForTestSolutionFile);

                    Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> projectConfigBySolutionsConfigs = dummyEverettSolutionInfo.GetSolutionConfigMappingToProjectConfig(this._testSolutionFileContents);

                    //
                    // Of all the projects in the sample solution we will only check some of them below,
                    // the line from the solution file has been copied in for easy reference
                    //

                    BuildConfiguration debugConfig = new BuildConfiguration("Debug");
                    BuildConfiguration debugDotNETConfig = new BuildConfiguration("Debug", ".NET");

                    BuildConfiguration releaseConfig = new BuildConfiguration("Release");
                    BuildConfiguration releaseDotNETConfig = new BuildConfiguration("Release", ".NET");

                    //
                    // 1st project to check: WebApplication1
                    //
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugConfig][_webApplication1_ID].BuildConfiguration == debugDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugConfig][_webApplication1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_webApplication1_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");

                    //
                    // 2nd project to check: ClassLibrary1
                    //
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugConfig][_classLibrary1_ID].BuildConfiguration == debugDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_classLibrary1_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built in its debug config so check that to..
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugConfig][_classLibrary1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");
                    // this project is NOT set to be built for its release configurations so check that..
                    Assert.IsFalse(projectConfigBySolutionsConfigs[releaseConfig][_classLibrary1_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig is true.");

                    //
                    // 3rd  project to check: WebService1
                    //
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugConfig][_webService1_ID].BuildConfiguration == debugDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_webService1_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built for both configurations so check that to..
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugConfig][_webService1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");
                    Assert.IsTrue(projectConfigBySolutionsConfigs[releaseConfig][_webService1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");

                    //
                    // 4th project to check: ConsoleApplication1
                    //

                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugConfig][_consoleApplication1_ID].BuildConfiguration == debugDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_consoleApplication1_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built for both configurations so check that to..
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugConfig][_consoleApplication1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");
                    Assert.IsTrue(projectConfigBySolutionsConfigs[releaseConfig][_consoleApplication1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig is false.");
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            [Test(Description = "")]
            public void TestInferedWebMapping()
            {
                try
                {
                    FileInfo testSolutionFile = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\AspNetWebApplication\AspNetWebApplication.sln"));
                    string testSolutionFileContents = File.ReadAllText(testSolutionFile.FullName);
                    // note that we don't actually add anything to the WebMapCollection below as the paths are left to be inferred
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(testSolutionFile, new WebMapCollection());

                    Dictionary<Guid, SolutionsProjectInfo> solutionsProjectInfos = dummyEverettSolutionInfo.GetSolutionsProjectInfos(testSolutionFileContents, new WebMapCollection());

                    Assert.IsNotNull(solutionsProjectInfos, "Extracted SolutionsProjectInfo's returned null");
                    Assert.AreEqual(4, solutionsProjectInfos.Count);
                    Assert.AreEqual("WebApplication1", solutionsProjectInfos[_webApplication1_ID].ProjectName);
                    Assert.AreEqual("WebService1", solutionsProjectInfos[_webService1_ID].ProjectName);
                    Assert.AreEqual("WebControlLibrary1", solutionsProjectInfos[_webControlLibrary1_ID].ProjectName);
                    // both the webApplication1 and the webService1 project file location should have been infered
                    Assert.IsNotNull(solutionsProjectInfos[_webApplication1_ID].ProjectFile);
                    Assert.IsNotNull(solutionsProjectInfos[_webService1_ID].ProjectFile);
                    Assert.IsNotNull(solutionsProjectInfos[_webControlLibrary1_ID].ProjectFile);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            [Test]
            public void EnumeratorTest()
            {
                try
                {
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(this._testSolutionFile, this._webMapsForTestSolutionFile);

                    //
                    // test "Debug|Any CPU"
                    //
                    dummyEverettSolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug"));

                    int projectToBuildCount = 0;
                    int totalProjects = 0;
                    foreach (IProjectInfo project in dummyEverettSolutionInfo)
                    {
                        if (project.Build)
                            projectToBuildCount++;

                        totalProjects++;
                    }

                    Assert.AreEqual(7, projectToBuildCount);
                    Assert.AreEqual(7, totalProjects);

                    //
                    // test "Release"
                    //
                    dummyEverettSolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Release"));

                    projectToBuildCount = 0;
                    totalProjects = 0;

                    foreach (IProjectInfo project in dummyEverettSolutionInfo)
                    {
                        if (project.Build)
                            projectToBuildCount++;

                        totalProjects++;
                    }

                    Assert.AreEqual(6, projectToBuildCount);
                    Assert.AreEqual(7, totalProjects);
                }
                catch (Exception ex)
                {
                    Console.Write(ex);
                    throw;
                }
            }

            [Test]
            public void TestAbstractSolutionProjectCollectionsTest()
            {
                try
                {
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(this._testSolutionFile, this._webMapsForTestSolutionFile);

                    //
                    // test "Debug"
                    //
                    dummyEverettSolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug"));

                    Assert.AreEqual(7, dummyEverettSolutionInfo.ProjectsByName.Count);

                    //
                    // test "Release"
                    //
                    dummyEverettSolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Release"));

                    Assert.AreEqual(7, dummyEverettSolutionInfo.ProjectsByName.Count);
                }
                catch (Exception ex)
                {
                    Console.Write(ex);
                    throw;
                }
            }

            [Test(), Ignore()]
            public void CreateProjectsTest_Isolated()
            {
                //IProjectInfo project;
                //Dictionary<String, IProjectInfo> projectsByName = new Dictionary<string,IProjectInfo>();
                //DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo();

                //// snippet from the solution file were using for the test
                ////Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "ConsoleApplication1", "ConsoleApplication\ConsoleApplication1\ConsoleApplication1.csproj", "{D5720673-3207-4F4C-810A-C8E6F0838CF4}"
                ////EndProject
                ////Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "WebService1", "AspNetWebApplication\WebService1\WebService1.csproj", "{0E075513-8FC8-4B05-9374-E74A374F05E1}"
                ////EndProject
                ////Project("{E24C65DC-7377-472B-9ABA-BC803B73C61A}") = "C:\...\FileSystemBasedWebSite\", "AspNetWebApplication\FileSystemBasedWebSite\", "{6A718792-82DF-4AD5-9FB8-B1EC47366CB1}"
                ////    ProjectSection(WebsiteProperties) = preProject
                ////        Debug.AspNetCompiler.VirtualPath = "/FileSystemBasedWebSite"
                ////        Debug.AspNetCompiler.PhysicalPath = "AspNetWebApplication\FileSystemBasedWebSite\"
                ////        Debug.AspNetCompiler.TargetPath = "PrecompiledWeb\FileSystemBasedWebSite\"
                ////        Debug.AspNetCompiler.Updateable = "true"
                ////        Debug.AspNetCompiler.ForceOverwrite = "true"
                ////        Debug.AspNetCompiler.FixedNames = "false"
                ////        Debug.AspNetCompiler.Debug = "True"
                ////        Release.AspNetCompiler.VirtualPath = "/FileSystemBasedWebSite"
                ////        Release.AspNetCompiler.PhysicalPath = "AspNetWebApplication\FileSystemBasedWebSite\"
                ////        Release.AspNetCompiler.TargetPath = "PrecompiledWeb\FileSystemBasedWebSite\"
                ////        Release.AspNetCompiler.Updateable = "true"
                ////        Release.AspNetCompiler.ForceOverwrite = "true"
                ////        Release.AspNetCompiler.FixedNames = "false"
                ////        Release.AspNetCompiler.Debug = "False"
                ////        VWDPort = "1097"
                ////        DefaultWebSiteLanguage = "Visual C#"
                ////    EndProjectSection
                ////EndProject


                //project = dummyEverettSolutionInfo.CreateProject(
                //    new Guid("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}"), "ConsoleApplication1", new FileInfo(;
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyEverettSolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyEverettSolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyEverettSolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //CheckProjectsCreatedOk(projectsByName);
            }

            [Test()]
            public void CreateProjectsTest_IntergrationWithOtherClassInternals()
            {
                try
                {
                    DummyEverettSolutionInfo dummyEverettSolutionInfo = new DummyEverettSolutionInfo(this._testSolutionFile, this._webMapsForTestSolutionFile);

                    // setting the active configuration will trigger a load of the projects for the given configuration.
                    dummyEverettSolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug"));

                    CheckProjectsCreatedOk(dummyEverettSolutionInfo.ProjectsByName);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            private void CheckProjectsCreatedOk(Dictionary<String, IProjectInfo> projectsByName)
            {
                foreach (IProjectInfo project in projectsByName.Values)
                {
                    //
                    // check some of the IProjectProperties props
                    // We check IsNotNull for most and additionally 
                    // IsNotEmpty for strings.
                    //
                    Assert.IsNotNull(project.ProjectName);
                    Assert.IsNotEmpty(project.ProjectName);

                    Assert.IsNotNull(project.ID);

                    if (project.ProjectType != ProjectType.WebSite)
                    {
                        Assert.IsNotNull(project.ProjectFile);

                        Assert.IsNotNull(project.ProjectFileExtension);
                        Assert.IsNotEmpty(project.ProjectFileExtension);

                        Assert.IsNotNull(project.AssemblyName);
                        Assert.IsNotEmpty(project.AssemblyName);

                        Assert.IsNotNull(project.AssemblyFileName);
                        Assert.IsNotEmpty(project.AssemblyFileName);
                    }

                    Assert.IsNotNull(project.CodeFileExtension);
                    Assert.IsNotEmpty(project.CodeFileExtension);

                    Assert.IsNotNull(project.ProjectDir);

                    Assert.IsNotNull(project.ProjectLanguage);

                    Assert.IsNotNull(project.ProjectOutputType);

                    Assert.IsNotNull(project.ProjectType);

                    Assert.IsNotNull(project.ConfigFile);
                    Assert.IsNotEmpty(project.ConfigFile);

                    Assert.IsNotNull(project.DotNetFrameworkVersion);

                    Assert.IsNotNull(project.IProjectBuildProperties);

                    //
                    // check some of the IProjectBuildProperties props
                    // We check IsNotNull for most and additionally 
                    // IsNotEmpty for strings.
                    //

                    if (project.ProjectType != ProjectType.WebSite)
                    {
                        Assert.IsNotNull(project.IProjectBuildProperties.OutputPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.OutputPathFull);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputPathFull);

                        Assert.IsNotNull(project.IProjectBuildProperties.OutputFilePath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputFilePath);
                    }
                    else
                    {
                        Assert.IsNotNull(project.IProjectBuildProperties.VirtualPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.VirtualPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.PhysicalPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.PhysicalPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.TargetPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.TargetPath);
                    }
                }
            }
        }
#endif
    }
}
