using System;
using System.IO;
using System.Xml;
using Snak.Utilities;

//using Snak.Utilities;

namespace Snak.Core
{
    /// <summary>
    /// Retrieves the currect <see cref="IProjectInfo"/> implementation
    /// for a VS project file, based on its version
    /// </summary>
    public class ProjectFactory
    {
        private ProjectFactory() { }

        /// <summary>
        /// Retrieves the correct <see cref="IProjectInfo"/> implementation
        /// for a VS project file, based on its version
        /// </summary>
        /// <param name="projectFilePath">A local path to a project file</param>
        /// <param name="config">The config for the retrieved project</param>
        /// <returns></returns>
        public static IProjectInfo GetProject(string projectFilePath, BuildConfiguration config)
        {
            return GetProject(new FileInfo(projectFilePath), config);
        }

        /// <summary>
        /// Retrieves the correct <see cref="IProjectInfo"/> implementation
        /// for a VS project file, based on its version
        /// </summary>
        /// <param name="projectFilePath">A local path to a project file</param>
        /// <param name="config">The config for the retrieved project</param>
        /// <returns></returns>
        public static IProjectInfo GetProject(FileInfo projectFilePath, BuildConfiguration config)
        {
            if (!projectFilePath.Exists) throw new FileNotFoundException(projectFilePath.FullName);

            XmlDocument document = new XmlDocument();
            using (FileStream stream = projectFilePath.OpenRead())
            {
                XmlTextReader reader = new XmlTextReader(projectFilePath.FullName, stream);
                document.Load(reader);
            }

            DotNetFrameworkVersion dotNetFrameworkVersion = DetermineFrameworkVersionForProject(document);
            switch (dotNetFrameworkVersion)
            {
                case DotNetFrameworkVersion.v2_0:
                    return new Whidbey.WhidbeyProjectInfo(projectFilePath, config);

                case DotNetFrameworkVersion.v1_1:
                    // return new Everett.ProjectInfo(projectFilePath, config);
                    throw new NotImplementedException();
                //				case ProjectType.Ranier:
                //					return Ranier.ProjectInfo(projectFilePath);
                default:
                    throw new ArgumentException(dotNetFrameworkVersion.ToString());
            }
        }

        /// <summary>
        /// Attempts to determine the correct project type for a VS project file
        /// by inspecting the XML for its version indicator
        /// </summary>
        private static DotNetFrameworkVersion DetermineFrameworkVersionForProject(XmlDocument projectfile)
        {
            // attempt to find vs2005 (whitbey) project
            XmlNamespaceManager ns = new XmlNamespaceManager(projectfile.NameTable);
            ns.AddNamespace("msb", "http://schemas.microsoft.com/developer/msbuild/2003");
            XmlNode node = projectfile.SelectSingleNode("//msb:Project/msb:PropertyGroup/msb:ProductVersion/text()", ns) as XmlText;
            if (node != null && node.Value.StartsWith("8."))
                return DotNetFrameworkVersion.v2_0;

            // attempt to find vs2003 (everett) project
            node = projectfile.SelectSingleNode("/VisualStudioProject/*/@ProductVersion") as XmlAttribute;
            if (node != null && node.Value.StartsWith("7."))
                return DotNetFrameworkVersion.v1_1;

            // TODO: attempt to find vs2002 (ranier) project??

            throw new ArgumentException("Unable to determine project file version");
        }

        [NUnit.Framework.TestFixture]
        public class ProjectFactoryTester
        {
            [NUnit.Framework.Test]
            public void TestParseEverettProject()
            {
                XmlDocument projectXml = new XmlDocument();
                projectXml.Load(ResourceUtils.GetResourceStream(typeof(Everett.EverettSolutionInfo), "TestCSharpWinformsProject2003.csproj"));

                NUnit.Framework.Assert.AreEqual(DotNetFrameworkVersion.v1_1, DetermineFrameworkVersionForProject(projectXml), "Failed to determine project type for known 2003 project");
            }

            [NUnit.Framework.Test]
            public void TestParseWhitbeyProject()
            {
                XmlDocument projectXml = new XmlDocument();
                projectXml.Load(ResourceUtils.GetResourceStream(typeof(Whidbey.WhidbeySolutionInfo), "TestCSharpWinformsProject2005.csproj"));

                NUnit.Framework.Assert.AreEqual(DotNetFrameworkVersion.v2_0, DetermineFrameworkVersionForProject(projectXml), "Failed to determine project type for known 2005 project");
            }
        }
    }

    //public enum ProjectType
    //{
    //    Ranier,
    //    Everett,
    //    Whitbey
    //}
}
