using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace Snak.Core
{
    public class ProjectPropertiesParserHelper : IStringToTypeConverter
    {
        public delegate T TryConvertOrReturnNullOrEmpty<T>(string valueToConvert);

        Dictionary<Type, MethodInfo> _conversionMethodsByType = new Dictionary<Type, MethodInfo>();

        #region static parsing methods that match the delegate TryConvertOrReturnNullOrEmpty<T>

        public Guid ConvertToGuid(string valueToConvert)
        {
            Guid guid = (valueToConvert != String.Empty) ? new Guid(valueToConvert) : Guid.Empty;
            
            return guid;        
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="valueToConvert"></param>
        /// <returns></returns>
        /// <remarks>would have been nice to put a constraint on the type argument T but "a type (argument) must not be one of the following types: System.Array, System.Delegate, System.Enum, or System.ValueType."</remarks>
        public T ConvertToEnum<T>(string valueToConvert)
        {
            T anEnum;
            try
            {
                anEnum = (T)Enum.Parse(typeof(T), valueToConvert);
            }
            catch (ArgumentException)
            {
                throw new InvalidOperationException(string.Format("OutputType of '{0}' not recognized as a valid value for enum '{1}'", valueToConvert, typeof(T).ToString()));
            }

            return anEnum;
        }

        public bool ConvertToBool(string valueToConvert)
        {
            bool result;
            bool.TryParse(valueToConvert, out result);
            return result;
        }

        public uint ConvertToUInt(string valueToConvert)
        {
            uint result;
            uint.TryParse(valueToConvert, out result);
            return result;
        }

        public int ConvertToInt(string valueToConvert)
        {
            int result;
            int.TryParse(valueToConvert, out result);
            return result;
        }

        public List<String> ConvertToCommaSeperatedListToListOfStrings(string valueToConvert)
        {
            List<String> list = new List<string>();
            string[] array = valueToConvert.Split(',');
            list.AddRange(array);
            return list;
        }

        public FileInfo ConvertToFileInfo(string valueToConvert)
        {
            return (valueToConvert == null || valueToConvert == String.Empty) ? null : new FileInfo(valueToConvert);
        }

        #endregion 

        public ProjectPropertiesParserHelper()
        {
            foreach (MethodInfo method in typeof(IStringToTypeConverter).GetMethods())
            {
                if (method.Name == "ConvertToEnum")
                    _conversionMethodsByType.Add(typeof(Enum), method); // hack
                else
                {
                    //if (method.IsGenericMethod && method.ReturnType.IsAssignableFrom(Enum))
                    //    _conversionMethodsByType.Add(typeof(Enum), method);
                    if (!method.IsGenericMethod)
                        _conversionMethodsByType.Add(method.ReturnType, method);
                }
            }
        }

        public T TryConvertValue<T>(string value)
        {
            return (T)TryConvertValue(typeof(T), value);
        }

        public object TryConvertValue(Type type, string value)
        {
            if (type.Equals(typeof(String)))
                return value;
            
            Type typeToConvertTo;

            if (type.IsEnum)
                typeToConvertTo = typeof(Enum);
            else
                typeToConvertTo = type;

            if (!_conversionMethodsByType.ContainsKey(typeToConvertTo))
                throw new ApplicationException("The " + typeof(IStringToTypeConverter).ToString() + " does not support conversion of type " + typeToConvertTo.ToString());

            MethodInfo methodToCall = _conversionMethodsByType[typeToConvertTo];

            if (methodToCall.IsGenericMethod)
            {
                // bit of a hack, we know that the convert methods that are generic are meant to have the type parameter 
                // of type passed to this method, given this we'll just create a generic methodinfo here to invoke
                // based on the type passed to this method
                methodToCall = methodToCall.MakeGenericMethod(new Type[] {type});
            }
            
            return methodToCall.Invoke(this, new object[] { value });
        }
    }
}
