using System;
using System.IO;
using NAnt.VSNet.Types;

namespace Snak.Core
{
	/// <summary>
	/// Provides a facade over the various Everett / Whitbey solution file formats
	/// </summary>
	/// <remarks>
	/// <example>
	///   <para>Some example</para>
	///   <code>
	///     <![CDATA[
	///     some code here
	///     ]]>
	///   </code>
	/// </example>
	/// </remarks>
	public class SolutionFactory
	{
		#region Constructor(s)
		private SolutionFactory()
		{
		}
		#endregion

		/// <summary>
		/// Returns the correct <see cref="ISolutionInfo"/> implementation
		/// for the solution file supplied
		/// </summary>
		public static ISolutionInfo GetSolution(string solutionFilePath)
		{
			return GetSolution(new FileInfo(solutionFilePath), null);
		}

        /// <summary>
        /// Returns the correct <see cref="ISolutionInfo"/> implementation
        /// for the solution file supplied
        /// </summary>
        /// <param name="solutionFilePath"></param>
        /// <param name="webMapCollection"></param>
        /// <returns></returns>
        public static ISolutionInfo GetSolution(string solutionFilePath, WebMapCollection webMapCollection)
        {
            return GetSolution(new FileInfo(solutionFilePath), webMapCollection);
        }

        /// <summary>
        /// Returns the correct <see cref="ISolutionInfo"/> implementation
        /// for the solution file supplied
        /// </summary>
        /// <param name="solutionFile"></param>
        /// <param name="webMapCollection"></param>
        /// <returns></returns>
        public static ISolutionInfo GetSolution(FileInfo solutionFile)
        {
            return GetSolution(solutionFile, null);
        }

        /// <summary>
        /// Returns the correct <see cref="ISolutionInfo"/> implementation
        /// for the solution file supplied
        /// </summary>
        /// <param name="solutionFile"></param>
        /// <param name="webMapCollection"></param>
        /// <returns></returns>
		public static ISolutionInfo GetSolution(FileInfo solutionFile, WebMapCollection webMapCollection)
		{
			try
			{
				bool isVS2005 = false;
				using (StreamReader sr = new StreamReader(solutionFile.FullName))
				{
					string firstLines = sr.ReadLine() + sr.ReadLine() + sr.ReadLine();
					if (firstLines.IndexOf(@"Microsoft Visual Studio Solution File, Format Version 9.00") > -1)
					{
						isVS2005 = true;
					}
				}

                if (isVS2005)
                    return new Whidbey.WhidbeySolutionInfo(solutionFile);
                else
                {
                    return new Everett.EverettSolutionInfo(solutionFile, webMapCollection); 
                }


			}catch(Exception err)
			{
				throw new InvalidOperationException(string.Format("Failed to load solution from {0}", solutionFile, err.Message), err);	
			}
		}

		[Obsolete("Use ProjectFactory instead")]
        public static IProjectInfo GetProject(FileInfo projectFile, BuildConfiguration buildConfiguration) 
		{
			bool isVS2005 = false;
			try
			{
				using (StreamReader sr = new StreamReader(projectFile.FullName))
				{
					string firstFewLines = String.Empty;
					int i = 0;

					while (i < 8 || sr == null)
					{
						firstFewLines += sr.ReadLine();	
						i ++;
					}

					if (firstFewLines.IndexOf(@"xmlns=""http://schemas.microsoft.com/developer/msbuild/2003""") > -1)
					{
						isVS2005 = true;
					}
				}

                if (isVS2005)
                    return new Whidbey.WhidbeyProjectInfo(projectFile, buildConfiguration);
                else
                    return new Everett.EverettProjectInfo(projectFile, buildConfiguration);
			}
			catch(Exception ex)
			{
                throw new InvalidOperationException(string.Format("Failed to load project from {0} [{1}]", projectFile, buildConfiguration.ToString(), ex.Message), ex);	
			}
		}

#if(UNITTEST)
		[NUnit.Framework.TestFixture]
		[NUnit.Framework.Ignore("Tests for SolutionFactory not written yet")]
		public class SolutionFactoryTester{

			#region Setup test and mock objects
			SolutionFactory aSolutionFactory = new SolutionFactory();

			[NUnit.Framework.TestFixtureSetUp]
			public void Init(){}

			[NUnit.Framework.SetUp]
			public void Setup(){}
			#endregion

			[NUnit.Framework.Test(Description="")]
			public void TestSomeMethod(){

			}


			[NUnit.Framework.Test(Description="")]
			public void TestGetSolution()
			{

			}


			[NUnit.Framework.Test(Description="")]
			public void TestGetProject()
			{

			}

		}
#endif

	}
}
