using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Snak.Core;
using System.Xml;

namespace Snak.Core.Whidbey
{
    public class WhidbeyProjectInfo : AbstractProject
    {

        internal WhidbeyProjectInfo(FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(projectFile, buildConfigurationToLoad)
        {

        }

        internal WhidbeyProjectInfo(Guid id, string projectName, FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, projectFile, buildConfigurationToLoad)
        {

        }

        protected override XmlNamespaceManager GetXmlXMLNamespaceManagerForProjectFile()
        {
            XmlNamespaceManager manager = new XmlNamespaceManager(this.ProjectDoc.NameTable);
            manager.AddNamespace("ms", "http://schemas.microsoft.com/developer/msbuild/2003");
            return manager;
        }

        protected override void LoadNonBuildConfigSpeificProperties()
        {
            //HACK - proper place might be /Project/Import/@Project=*Microsoft.<language>.targets", but language is still subtly different to the enum
            this.ProjectLanguage = (this.ProjectFile.Extension.ToLower() == ".vbproj") ? ProjectLanguage.VisualBasic : ProjectLanguage.CSHARP;   //(ProjectLanguage) Enum.Parse(typeof (ProjectLanguage), projectTypeNode.Name);
            this.AssemblyName = this.XmlProjectDocParser.GetPropertySetting<String>("AssemblyName");
            this.ProjectType = GetProjectType();
            this.DotNetFrameworkVersion = DotNetFrameworkVersion.v2_0;

            if (this.ProjectName == null || this.ProjectName == String.Empty)
            {
                if (this.ProjectFile != null)
                    this.ProjectName = Path.GetFileNameWithoutExtension(this.ProjectFile.FullName);
                else
                    throw new SnakException("Could not set the project name.");
            }

            if (this.ID == Guid.Empty)
                this.ID = this.XmlProjectDocParser.GetPropertySetting<Guid>("ProjectGuid");

            if (this.ProjectType == ProjectType.VisualStudioDeployment)
                this.ProjectOutputType = ProjectOutputType.Deployment;
            else
                this.ProjectOutputType = this.XmlProjectDocParser.GetPropertySetting<ProjectOutputType>("OutputType");
        }

        protected override void LoadNonBuildConfigSpeificProperties_ForFilelessProjects()
        {
            throw new InvalidOperationException();
        }

        protected override void LoadProjectSettingsForBuildConfig(BuildConfiguration buildConfiguration)
        {
            AbstractProjectBuildProperties abstractProjectBuildConfig = null;

            abstractProjectBuildConfig = new WhidbeyProjectBuildConfig(buildConfiguration, this.XmlProjectDocParser, this);

            this.IProjectBuildProperties = abstractProjectBuildConfig;
        }

        protected override AbstractXmlProjectFileParser GetAbstractXmlProjectFileParser(XmlDocument projectDocument, XmlNamespaceManager xmlNamespaceManager)
        {
            return new WhidbeyXmlProjectFileParser(projectDocument, xmlNamespaceManager);
        }

        public override List<string> GetProjectFiles()
        {
            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded??

            XmlDocument projectXml = LoadXmlDocument(this.ProjectFile.FullName);
            XmlNodeList fileNodes = projectXml.SelectNodes("//Files/Include/File");
            return GetProjectFiles(fileNodes);
        }

        /// <summary>
        /// returns the projects files according to the content type, if the project is a fileless project e.g. a 2005 web site then all file are returned. 
        /// </summary>
        /// <param name="contentType"></param>
        /// <returns></returns>
        public override List<string> GetProjectFiles(ProjectFileType contentType)
        {
            List<string> files = null;

            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded...

            string contentFilesXPath = string.Format("/ms:Project/ms:ItemGroup/ms:{0}", contentType);

            XmlNodeList matchingNodes = this.ProjectDoc.SelectNodes(contentFilesXPath, this.ProjectDocXMLNamespaceManager);

            files = new List<string>();

            for (int i = 0; i < matchingNodes.Count; i++)
            {
                string includeAttribute = ((XmlElement)matchingNodes[i]).GetAttribute("Include");

                if (includeAttribute.Length > 0)
                    files.Add(includeAttribute);
            }

            return files;
        }

        private List<string> GetProjectFiles(XmlNodeList fileNodes)
        {
            // TODO refactor: should this not be in XmlProjectDocParser as thats allready loaded...

            List<string> files = new List<string>();
            foreach (XmlElement fileNode in fileNodes)
                files.Add(fileNode.GetAttribute("RelPath"));
            return files;
        }

        private static XmlDocument LoadXmlDocument(string documentPath)
        {
            XmlDocument projectDoc = new XmlDocument();
            using (FileStream projectFileStream = File.OpenRead(documentPath))
                projectDoc.Load(projectFileStream);
            return projectDoc;
        }

        private ProjectType GetProjectType()
        {
            ProjectType projectType = ProjectType.Unknown;
            string projectTypeString;

            // there seems to be some inconsistencies regarding how to determine the 'project type' given the project file
            // some project files have a ProjectType element, some dont have a ProjectType but have a ProjectTypeGuids element.
            // Some such as the .wdproj dont have either of the above mentioned.
            //
            // Given this we end up with a few areas were we can figure out the project type. At this point we really only care about
            // Web projects, Deployment projects and others (which we will label Local). (In 2003 we had Web or Local).
            //
            // At this point we can get the info from one of 3 places.
            //
            // 1: for project file that have a ProjectType element we can simply get that value, if that fails
            //
            // 2: for project file that have a ProjectTypeGuids element we can look in the registry, 
            // we could sniff around in the registry using code here but we have a class ProjectTypeFactory that has some hardcoded Guids, yuk.
            // that'll do for now.
            //
            // 3: for project file that dont have ProjectType or ProjectTypeGuids element we can look at the file extension
            //
            // 4: if that fails we will just give it a ProjectType ProjectType.Unknown

            projectTypeString = this.XmlProjectDocParser.GetPropertySetting<String>("ProjectType");

            if (projectTypeString == String.Empty)
            {
                string guid = String.Empty;
                string guids = this.XmlProjectDocParser.GetPropertySetting<String>("ProjectTypeGuids");

                if (guids != null && guids != String.Empty)
                {
                    if (guids.IndexOf(";", 0, guids.Length) > -1)
                        guid = guids.Split(';')[0];
                    else
                        guid = guids;

                    projectType = ProjectTypeFactory.GetProjectType(new Guid(guid));
                }

                if (projectType == ProjectType.Unknown && this.ProjectFile.Extension == ".wdproj")
                    projectType = ProjectType.VisualStudioDeployment;

                if (projectType == ProjectType.Unknown)
                {
                    // TODO: figure out how the nant logging working in here
                    // or perhaps configure log4 net and globally try point it to nant output
                    Console.WriteLine("Could not find a project type for '" + this.ProjectFile.FullName + "' using default value of 'ProjectType.Unknown'");
                }
            }
            else
            {
                projectType = ProjectTypeFactory.GetProjectType(projectTypeString);
            }

            return projectType;
        }

#if( UNITTEST )
        [NUnit.Framework.TestFixture]
        public class ProjectInfoTester
        {
            private FileInfo testProjectFile = new FileInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\ClassLibrary\ClassLibrary1\ClassLibrary1.csproj");
            private DirectoryInfo projectRoot = new DirectoryInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\ClassLibrary\ClassLibrary1");
            private BuildConfiguration _buildConfiguration = new BuildConfiguration("Debug", "Any CPU");

            [NUnit.Framework.TestFixtureSetUp]
            public void Init()
            {
                _buildConfiguration = new BuildConfiguration("Debug", "Any CPU");
            }

            [NUnit.Framework.Test(Description = "")]
            public void TestLoadProject()
            {
                WhidbeyProjectInfo project = new WhidbeyProjectInfo(testProjectFile, _buildConfiguration);
                project.ActiveBuildConfiguration = this._buildConfiguration;

                AssertEqualIgnoreCase("ClassLibrary1", project.ProjectName);
                AssertEqualIgnoreCase("Debug", project.ActiveBuildConfiguration.Config);
                AssertEqualIgnoreCase("Any CPU", project.ActiveBuildConfiguration.Platform);
                AssertEqualIgnoreCase("ClassLibrary1.dll", project.AssemblyFileName);
                AssertEqualIgnoreCase("ClassLibrary1", project.AssemblyName);
                AssertEqualIgnoreCase(".csproj", project.ProjectFileExtension);
                AssertEqualIgnoreCase(".cs", project.CodeFileExtension);
                AssertEqualIgnoreCase(projectRoot.FullName, project.ProjectDir.FullName);
                AssertEqualIgnoreCase(@"bin\debug\", project.IProjectBuildProperties.OutputPath);
                AssertEqualIgnoreCase(Path.Combine(projectRoot.FullName, @"bin\debug\"), project.IProjectBuildProperties.OutputPathFull);
                AssertEqualIgnoreCase(Path.Combine(projectRoot.FullName, @"bin\debug\ClassLibrary1.dll"), project.IProjectBuildProperties.OutputFilePath);
                NUnit.Framework.Assert.AreEqual(ProjectOutputType.Library, project.ProjectOutputType);
            }

            private void AssertEqualIgnoreCase(string expected, string actual)
            {
                NUnit.Framework.Assert.AreEqual(expected.ToLower(), actual.ToLower());
            }

            [NUnit.Framework.Test]
            public void TestGetProjectFiles()
            {
                Dictionary<ProjectFileType, String> expectedFiles = new Dictionary<ProjectFileType, String>();
                expectedFiles.Add(ProjectFileType.Compile, "Properties\\AssemblyInfo.cs");
                expectedFiles.Add(ProjectFileType.Content, "ContentItem.txt");
                expectedFiles.Add(ProjectFileType.EmbeddedResource, "EmbeddedItem.bmp");
                expectedFiles.Add(ProjectFileType.None, "NoneItem.txt");

                WhidbeyProjectInfo whidbeyProjectInfo = new WhidbeyProjectInfo(new FileInfo(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\Misc\TestCSharpWinformsProject2005.csproj"), _buildConfiguration);
                whidbeyProjectInfo.ActiveBuildConfiguration = this._buildConfiguration;

                foreach (KeyValuePair<ProjectFileType, String> expectedFile in expectedFiles)
                {
                    ProjectFileType type = (ProjectFileType)expectedFile.Key;
                    List<String> compileItems = whidbeyProjectInfo.GetProjectFiles(type);

                    switch (type)
                    {
                        case ProjectFileType.EmbeddedResource:
                            NUnit.Framework.Assert.AreEqual(2, compileItems.Count, "Length of items for '{0}'", type);
                            NUnit.Framework.Assert.AreEqual(expectedFile.Value, compileItems[1], "Value of item for '{0}'", type);
                            break;
                        case ProjectFileType.Compile:
                            NUnit.Framework.Assert.AreEqual(3, compileItems.Count, "Length of items for '{0}'", type);
                            NUnit.Framework.Assert.AreEqual(expectedFile.Value, compileItems[0], "Value of item for '{0}'", type);
                            break;
                        case ProjectFileType.None:
                            NUnit.Framework.Assert.AreEqual(3, compileItems.Count, "Length of items for '{0}'", type);
                            NUnit.Framework.Assert.AreEqual(expectedFile.Value, compileItems[2], "Value of item for '{0}'", type);
                            break;
                        default:
                            NUnit.Framework.Assert.AreEqual(1, compileItems.Count, "Length of items for '{0}'", type);
                            NUnit.Framework.Assert.AreEqual(expectedFile.Value, compileItems[0], "Value of item for '{0}'", type);
                            break;
                    }
                }
            }

            [NUnit.Framework.Test]
            [NUnit.Framework.ExpectedException(typeof(ArgumentException))]
            public void TestGetBadProject()
            {
                new WhidbeyProjectInfo(new FileInfo(this.GetType().Assembly.Location), new BuildConfiguration("Debug", "AnyCpu"));
            }
        }

#endif
    }
}
