using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Snak.Core;
using System.Text.RegularExpressions;
using NUnit.Framework;
using NAnt.VSNet.Types;

namespace Snak.Core.Whidbey
{
    public class WhidbeySolutionInfo : AbstractSolution
    {
        private const string PROJECT_PATTERN = @"Project\(""(?<projectTypeGuid>[^""]+)""\)\s*=\s*""(?<name>[^""]+)"",\s*""(?<location>[^""]+)"",\s""(?<projectGuid>[^""]+)""((.|\r\n|\s))*?\bEndProject\b"; //(.*?ProjectSection.*EndProjectSection|.*?)EndProject";
        // private const string PROJECT_BUILD_PATTERN = @"^*?(?<projectGuid>\{[^\}]*?\})\.\bSOLUTIONCONFIG\b\.Build\.0\s=\s(?<projectConfig>[^\r\n]*)"; 

        private const string BUILD_CONFIGURATION_SECTION_PATTERN = @"GlobalSection\(SolutionConfigurationPlatforms\)[^\n]*\n(?<configValues>(.|\n)*?)EndGlobalSection";
        private const string BUILD_CONFIGURATION_PATTERN = @"^(?<entireConfigString>(?<configValue>.*?)\|(?<platformValue>.*?))=.*?$";
        private const string WEB_PROJECT_SECTION_PATTERN = @"ProjectSection\(WebsiteProperties\)\s=\spreProject(?<projectSection>(.|\r\n|\s)*?)EndProjectSection";

        internal WhidbeySolutionInfo(FileInfo solutionFile) : base(solutionFile) { }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="solutionFile"></param>
        /// <param name="activeBuildConfiguration"></param>
        internal WhidbeySolutionInfo(FileInfo solutionFile, BuildConfiguration activeBuildConfiguration) : base(solutionFile, activeBuildConfiguration) { }

        #region abstract implementations

        /// <summary>
        /// 
        /// </summary>
        /// <param name="solutionFileContents"></param>
        /// <returns></returns>
        protected override Dictionary<Guid, SolutionsProjectInfo> GetSolutionsProjectInfos(string solutionFileContents, WebMapCollection webMaps)
        {
            SolutionsProjectInfo solutionsProjectInfo;
            Dictionary<Guid, SolutionsProjectInfo> solutionsProjectInfos = new Dictionary<Guid, SolutionsProjectInfo>();

            MatchCollection matches = Regex.Matches(solutionFileContents, PROJECT_PATTERN);

            for (int projectIndex = 0; projectIndex < matches.Count; projectIndex++)
            {
                Match match = matches[projectIndex];
                Guid projectGuid = new Guid(match.Groups["projectGuid"].Value);
                Guid projectTypeGuid = new Guid(match.Groups["projectTypeGuid"].Value);
                string name = match.Groups["name"].Value;
                string location = match.Groups["location"].Value;
                string projectSection = String.Empty;
                bool hasProjectFile = true;

                if (ProjectTypeFactory.GetProjectType(projectTypeGuid) == ProjectType.WebSite)
                {
                    // file based web projects information lives in the solution file (as these type of projects dont have project files)
                    // give this we will grab the info here and assign it to projectSection, this will ultimately end up in solutionsProjectInfo
                    // and later will be used when constructing an IProjectInfo for this web project

                    Match projectSectionMatch = Regex.Match(match.Value, WEB_PROJECT_SECTION_PATTERN);

                    if (projectSectionMatch.Success)
                    {
                        projectSection = projectSectionMatch.Value;
                        hasProjectFile = false;
                    }
                }

                DirectoryInfo projectDir = null;
                FileInfo projectFile = null;

                string fullPath = Path.GetFullPath(Path.Combine(this.SolutionFile.DirectoryName, location));
                string projectFilesString = fullPath; // at this point fullPath and pojectFile may be are the same

                if (Path.HasExtension(fullPath))
                    projectDir = new DirectoryInfo(Path.GetDirectoryName(fullPath));
                else
                    projectDir = new DirectoryInfo(fullPath);

                if (hasProjectFile)
                    projectFile = this.GetProjectFileInfo(location);

                solutionsProjectInfo = new SolutionsProjectInfo(projectGuid, projectTypeGuid, name, projectFile, projectDir, projectSection, hasProjectFile);

                solutionsProjectInfos.Add(solutionsProjectInfo.ProjectGuid, solutionsProjectInfo);
            }

            return solutionsProjectInfos;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="solutionFileContents"></param>
        /// <returns></returns>
        /// <remarks>
        /// There are a lot of assumptions in relation to the VS .sln file structure within this method, 
        /// basically they were made after looking at several .sln example. Hopefully the test also prove 
        /// these assumptions are valid.
        /// </remarks>
        protected override Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> GetSolutionConfigMappingToProjectConfig(string solutionFileContents)
        {
            if (this.SolutionsProjectInfos == null)
                throw new SnakException("SolutionsProjectInfos must first be set before calling this method.");

            Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> projectConfigBySolutionConfig = new Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>>();
            Dictionary<Guid, SolutionsProjectConfig> solutionsProjectConfigs;
            SolutionsProjectConfig solutionsProjectConfig;
            BuildConfiguration buildConfiguration;

            string pattern = @"^*?(?<projectGuid>\{[^\}]*?\})\.(?<solutionConfig>[^\|]*)?\|(?<solutionPlatform>[^\.]*)?\.(?<configMetaData>[^\s]*?)\s=\s(?<projectConfig>[^\|]*)?\|(?<projectPlatform>[^\r\n]*)"; //";

            MatchCollection matchCollection = Regex.Matches(solutionFileContents, pattern);

            // Here is an example section thats (hopefully) representative of what a typical.sln contains
            // in this method we want to extract this info with a regexp
            //
            //    | Project Guid                         | Solutions config         | Projects config for 
            //    |                                      |                          | the solution config
            //GlobalSection(ProjectConfigurationPlatforms) = postSolution
            //    {691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {995B142B-FAD0-4722-89EC-E40D2F138F90}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {995B142B-FAD0-4722-89EC-E40D2F138F90}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {995B142B-FAD0-4722-89EC-E40D2F138F90}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {995B142B-FAD0-4722-89EC-E40D2F138F90}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {A9C997AE-DF4A-48B0-9A87-F063755DED1F}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {A9C997AE-DF4A-48B0-9A87-F063755DED1F}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {A9C997AE-DF4A-48B0-9A87-F063755DED1F}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {A9C997AE-DF4A-48B0-9A87-F063755DED1F}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {D5720673-3207-4F4C-810A-C8E6F0838CF4}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {D5720673-3207-4F4C-810A-C8E6F0838CF4}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {D5720673-3207-4F4C-810A-C8E6F0838CF4}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {D5720673-3207-4F4C-810A-C8E6F0838CF4}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {950577A5-DE2C-4CF4-BEE4-53911AE8ED59}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {950577A5-DE2C-4CF4-BEE4-53911AE8ED59}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {950577A5-DE2C-4CF4-BEE4-53911AE8ED59}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {950577A5-DE2C-4CF4-BEE4-53911AE8ED59}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
            //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Debug|Any CPU.Build.0 = Debug|Any CPU
            //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.ActiveCfg = Release|Any CPU
            //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.Build.0 = Release|Any CPU
            //    {6A718792-82DF-4AD5-9FB8-B1EC47366CB1}.Debug|Any CPU.ActiveCfg = Debug|.NET
            //    {6A718792-82DF-4AD5-9FB8-B1EC47366CB1}.Release|Any CPU.ActiveCfg = Debug|.NET
            //    {4DB77116-FDDC-4E25-A12E-F6A6E41EBC54}.Debug|Any CPU.ActiveCfg = Debug|.NET
            //    {4DB77116-FDDC-4E25-A12E-F6A6E41EBC54}.Release|Any CPU.ActiveCfg = Debug|.NET
            //EndGlobalSection


            // the match should bring back a row at a time e.g. "{4DB77116-FDDC-4E25-A12E-F6A6E41EBC54}.Release|Any CPU.ActiveCfg = Debug|.NET"
            for (int matchIndex = 0; matchIndex < matchCollection.Count; matchIndex++)
            {
                Match projectConfigMatch = matchCollection[matchIndex];

                // get reference to the solution configuration we are currently dealing with
                string solutionConfig = projectConfigMatch.Groups["solutionConfig"].Value;
                string solutionPlatform = projectConfigMatch.Groups["solutionPlatform"].Value;

                buildConfiguration = new BuildConfiguration(solutionConfig, solutionPlatform);

                // check if we have already added an item for this solution config, if not we set it if so we get a reference to it 
                if (!projectConfigBySolutionConfig.ContainsKey(buildConfiguration))
                {
                    solutionsProjectConfigs = new Dictionary<Guid, SolutionsProjectConfig>();
                    projectConfigBySolutionConfig.Add(buildConfiguration, solutionsProjectConfigs);
                }
                else
                    solutionsProjectConfigs = projectConfigBySolutionConfig[buildConfiguration];

                Guid projectGuid = new Guid(projectConfigMatch.Groups["projectGuid"].Value);
                string projectConfig = projectConfigMatch.Groups["projectConfig"].Value;
                string projectPlatform = projectConfigMatch.Groups["projectPlatform"].Value;
                // In the following example the configMetaData will grab ActiveCfg, and Build for the first and second rows retrospectively
                //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.ActiveCfg = Release|Any CPU
                //    {0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.Build.0 = Release|Any CPU    
                string configMetaData = projectConfigMatch.Groups["configMetaData"].Value;
                // the row with ActiveCfg always appears, if visual studio is meant to build the given project for the given config then 
                // where will also be a row with the .Build.0 part in it 
                bool build = configMetaData.Contains("Build");

                // as shown in the example above there may be more than one row for each project so we check if we've added it already
                if (!solutionsProjectConfigs.ContainsKey(projectGuid))
                {
                    solutionsProjectConfig = new SolutionsProjectConfig(this.SolutionsProjectInfos[projectGuid], new BuildConfiguration(projectConfig, projectPlatform), build);
                    solutionsProjectConfigs.Add(projectGuid, solutionsProjectConfig);
                }
                else
                    // if we've already added it we update it's build setting if necessary
                    if (build)
                        solutionsProjectConfigs[projectGuid].Build = true;
            }

            return projectConfigBySolutionConfig;
        }

        protected override IProjectInfo CreateProject(Guid id, string projectName, FileInfo projectFile, DirectoryInfo projectDir, string projectSectionFromSolutionFile, BuildConfiguration buildConfigurationToLoad)
        {
            IProjectInfo iProjectInfo;

            if (projectSectionFromSolutionFile != null && projectSectionFromSolutionFile != String.Empty)
            {
                // if we are dealing with a project that does not have a project file (e.g. a VS 2005 web site) 
                // then we should have something in projectSectionFromSolutionFile
                iProjectInfo = new WhidbeyWebProjectInfo(id, projectName, projectSectionFromSolutionFile, projectDir, buildConfigurationToLoad);
            }
            else
                iProjectInfo = new WhidbeyProjectInfo(id, projectName, projectFile, buildConfigurationToLoad);

            return iProjectInfo;
        }

        #endregion

        private FileInfo GetProjectFileInfo(string expectedProjectFileString)
        {
            FileInfo fileInfo = null;

            if (Path.HasExtension(expectedProjectFileString))
            {
                string fullPath = Path.GetFullPath(Path.Combine(this.SolutionFile.DirectoryName, expectedProjectFileString));
                fileInfo = new FileInfo(fullPath);
            }
            else
            {
                throw new SnakException("Could not locate a project file given the expected address of '" + fileInfo.FullName + "'");
            }

            return fileInfo;        
        }

#if( UNITTEST )

        [TestFixture()]
        public class WhidbeySolutionInfoTester
        {
            // this assumes that the tests are always going to be run from the \bin\debug directory
            private const string TEST_SOLUTION_FILE = @"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\SolutionWithMultipleDifferingProjects.sln";
            private FileInfo testSolutionFile = new FileInfo(TEST_SOLUTION_FILE);
            private string _testSolutionFileContents = String.Empty;
            private Guid _webApplication1_ID = new Guid("{691EF7BC-5F22-486D-8E7B-C062F61A9EEE}");
            private Guid _webApplication1_BusinessLayer_ID = new Guid("{995B142B-FAD0-4722-89EC-E40D2F138F90}");
            private Guid _webControlLibrary1_ID = new Guid("{A9C997AE-DF4A-48B0-9A87-F063755DED1F}");
            private Guid _classLibrary1_ID = new Guid("{A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}");
            private Guid _consoleApplication1_ID = new Guid("{D5720673-3207-4F4C-810A-C8E6F0838CF4}");
            private Guid _windowsService1_ID = new Guid("{950577A5-DE2C-4CF4-BEE4-53911AE8ED59}");
            private Guid _webService1_ID = new Guid("{0E075513-8FC8-4B05-9374-E74A374F05E1}");
            private Guid _fileSystemBasedWebSite_ID = new Guid("{6A718792-82DF-4AD5-9FB8-B1EC47366CB1}");
            private Guid _fileBasedWebService_ID = new Guid("{AEF7339B-58AB-4FB9-932A-12B0E94E29F7}");

            // below we create a wrapper for the class we want to test, as the test class is within the class on test we have access to the internals we want to test.
            // we could just have our test fixture inherit from the class we want to test but then  if something goes wrong in the constructor we cant really get 
            // meaningful feedback to the console about what went wrong. At least with the dummy wrapper we can wrap its instantiation in a try catch and dump any 
            // errors to the console before throwing the initial exception
            public class DummyWhidbeySolutionInfo : WhidbeySolutionInfo
            {
                public DummyWhidbeySolutionInfo() : base(new FileInfo(TEST_SOLUTION_FILE)) { }

                // we dont need to wrap DummyWhidbeySolutionInfo's protected base methods because our test fixture is actually within the WhidbeySolutionInfo, therefore we can still
                // see the protected methods on DummyWhidbeySolutionInfo from within WhidbeySolutionInfoTester
                //public string SolutionFileContents_Exposed { get; }
                //public Dictionary<Guid, SolutionsProjectInfo> GetSolutionsProjectInfos_Exposed(string solutionFileContents, WebMapCollection webMaps) { }
                //public Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> GetSolutionConfigMappingToProjectConfig_Exposed(string solutionFileContents) { }
                //public IProjectInfo CreateProject_Exposed(Guid id, string projectName, FileInfo projectFile, DirectoryInfo projectDir, string projectSectionFromSolutionFile, BuildConfiguration buildConfigurationToLoad) { }
            }

            [TestFixtureSetUp]
            public void TestFixtureSetUp()
            {
                try
                {
                    this._testSolutionFileContents = File.ReadAllText(testSolutionFile.FullName);
                }
                catch (Exception ex)
                {
                    throw new AssertionException("Error during a TestFixtureSetUp. See the inner exception.", ex);
                }
            }


            [Test()]
            public void GetSolutionsProjectInfosTest()
            {
                try
                {
                    DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                    string errorMessage = "Project from solution was not found in collection.";

                    Dictionary<Guid, SolutionsProjectInfo> solutionsProjectInfos = dummyWhidbeySolutionInfo.GetSolutionsProjectInfos(this._testSolutionFileContents, null);

                    Assert.AreEqual(9, solutionsProjectInfos.Count);


                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webApplication1_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webApplication1_BusinessLayer_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webControlLibrary1_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_classLibrary1_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_consoleApplication1_ID), errorMessage);
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_windowsService1_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_webService1_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_fileSystemBasedWebSite_ID), errorMessage); 
                    Assert.IsTrue(solutionsProjectInfos.ContainsKey(_fileBasedWebService_ID), errorMessage);

                    foreach (KeyValuePair<Guid, SolutionsProjectInfo> keyValuePair in solutionsProjectInfos)
                    {
                        SolutionsProjectInfo solutionsProjectInfo = keyValuePair.Value;

                        if (solutionsProjectInfo.ProjectName == null || solutionsProjectInfo.ProjectName == String.Empty)
                            Assert.Fail("ProjectName not set for project guid '" + keyValuePair.Key + "'");

                        if (solutionsProjectInfo.ProjectFile == null && solutionsProjectInfo.ProjectDir == null)
                            Assert.Fail("ProjectFile and or ProjectDir not set for project '" + solutionsProjectInfo.ProjectName + "'. For fileless project ProjectDir needs to be set for all others both need to be set.");

                        // if its a fileless project (e.g. a web site) solutionsProjectInfo.ProjectSection should not be empty but ProjectFile would be
                        // here we assume that the project file must exist for projects that have a file (i.e. ProjectSection is not empty). bit
                        // of a nasty check but better than nothing.
                        if (solutionsProjectInfo.ProjectSection == String.Empty && solutionsProjectInfo.ProjectFile == null)
                            Assert.Fail("ProjectFile not set for project '" + solutionsProjectInfo.ProjectName + "'.");

                        if (solutionsProjectInfo.ProjectDir == null)
                            Assert.Fail("ProjectDir not set for project '" + solutionsProjectInfo.ProjectName + "'.");

                        if (solutionsProjectInfo.ProjectGuid == Guid.Empty)
                            Assert.Fail("ProjectGuid not set for project '" + solutionsProjectInfo.ProjectName + "'");

                        if (solutionsProjectInfo.ProjectTypeGuid == Guid.Empty)
                            Assert.Fail("ProjectTypeGuid not set for project '" + solutionsProjectInfo.ProjectName + "'");

                        if (ProjectTypeFactory.GetProjectType(solutionsProjectInfo.ProjectTypeGuid) == ProjectType.WebSite)
                            if (solutionsProjectInfo.ProjectSection == null || solutionsProjectInfo.ProjectSection == String.Empty)
                                Assert.Fail("ProjectSection not set for web project '" + solutionsProjectInfo.ProjectName + "'");
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            [Test()]
            public void GetSolutionConfigMappingToProjectConfigTest()
            {
                try
                {
                    DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                    Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> projectConfigBySolutionsConfigs = dummyWhidbeySolutionInfo.GetSolutionConfigMappingToProjectConfig(this._testSolutionFileContents);

                    //
                    // Of all the projects in the sample solution we will only check some of them below,
                    // the line from the solution file has been copied in for easy reference
                    //

                    BuildConfiguration debugAnyCpuConfig = new BuildConfiguration("Debug", "Any CPU");
                    BuildConfiguration releaseConfig = new BuildConfiguration("Release", "Any CPU");
                    BuildConfiguration releaseDotNETConfig = new BuildConfiguration("Debug", ".NET");

                    //
                    // 1st project to check: WebApplication1
                    //
                    //{691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugAnyCpuConfig][_webApplication1_ID].BuildConfiguration == debugAnyCpuConfig), "For the given solution configuration, the projects configuration is not correct.");
                    //{691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Release|Any CPU.ActiveCfg = Release|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_webApplication1_ID].BuildConfiguration == releaseConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built for both configurations so check that to..
                    //{691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Debug|Any CPU.Build.0 = Debug|Any CPU
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugAnyCpuConfig][_webApplication1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig not been set.");
                    //{691EF7BC-5F22-486D-8E7B-C062F61A9EEE}.Release|Any CPU.Build.0 = Release|Any CPU
                    Assert.IsTrue(projectConfigBySolutionsConfigs[releaseConfig][_webApplication1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig not been set.");

                    //
                    // 2nd project to check: ClassLibrary1
                    //
                    //{A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugAnyCpuConfig][_classLibrary1_ID].BuildConfiguration == debugAnyCpuConfig), "For the given solution configuration, the projects configuration is not correct.");
                    //{A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Release|Any CPU.ActiveCfg = Release|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_classLibrary1_ID].BuildConfiguration == releaseConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built in its debug config so check that to..
                    //{A49EF1A3-5F78-44F5-A66D-AD51B84A5B28}.Debug|Any CPU.Build.0 = Debug|Any CPU
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugAnyCpuConfig][_classLibrary1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig not been set.");
                    // this project is NOT set to be built for its release configurations so check that..
                    Assert.IsFalse(projectConfigBySolutionsConfigs[releaseConfig][_classLibrary1_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig not been set.");

                    //
                    // 3rd  project to check: WebService1
                    //
                    //{0E075513-8FC8-4B05-9374-E74A374F05E1}.Debug|Any CPU.ActiveCfg = Debug|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugAnyCpuConfig][_webService1_ID].BuildConfiguration == debugAnyCpuConfig), "For the given solution configuration, the projects configuration is not correct.");
                    //{0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.ActiveCfg = Release|Any CPU
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_webService1_ID].BuildConfiguration == releaseConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is set to be built for both configurations so check that to..
                    //{0E075513-8FC8-4B05-9374-E74A374F05E1}.Debug|Any CPU.Build.0 = Debug|Any CPU
                    Assert.IsTrue(projectConfigBySolutionsConfigs[debugAnyCpuConfig][_webService1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig not been set.");
                    //{0E075513-8FC8-4B05-9374-E74A374F05E1}.Release|Any CPU.Build.0 = Release|Any CPU
                    Assert.IsTrue(projectConfigBySolutionsConfigs[releaseConfig][_webService1_ID].Build, "The solution should have stated to build this project however the build property on SolutionsProjectConfig not been set.");

                    //
                    // 4th project to check: FileSystemBasedWebSite
                    //
                    // notice that in this next group the Release configuration for the solution has the individual project set to its "debug|.Net" configuration, this "debug|.Net" configuration is not used by the solution, 
                    // it just knows that for its Release|Any Cpu configuration it has to build this project in its "debug|.Net" configuration
                    //{6A718792-82DF-4AD5-9FB8-B1EC47366CB1}.Debug|Any CPU.ActiveCfg = Debug|.NET
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugAnyCpuConfig][_fileSystemBasedWebSite_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    //{6A718792-82DF-4AD5-9FB8-B1EC47366CB1}.Release|Any CPU.ActiveCfg = Debug|.NET
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_fileSystemBasedWebSite_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is NOT set to be built for both configurations so check that to..
                    Assert.IsFalse(projectConfigBySolutionsConfigs[debugAnyCpuConfig][_fileSystemBasedWebSite_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig not been set.");
                    Assert.IsFalse(projectConfigBySolutionsConfigs[releaseConfig][_fileSystemBasedWebSite_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig not been set.");

                    //
                    // 5th to check: FileBasedWebService
                    //
                    // notice that in this next group the Release configuration for the solution has the individual project set to its "debug|.Net" configuration, this "debug|.Net" configuration is not used by the solution, 
                    // it just knows that for its Release|Any Cpu configuration it has to build this project in its "debug|.Net" configuration
                    //{4DB77116-FDDC-4E25-A12E-F6A6E41EBC54}.Debug|Any CPU.ActiveCfg = Debug|.NET
                    Assert.IsTrue((projectConfigBySolutionsConfigs[debugAnyCpuConfig][_fileBasedWebService_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    //{4DB77116-FDDC-4E25-A12E-F6A6E41EBC54}.Release|Any CPU.ActiveCfg = Debug|.NET
                    Assert.IsTrue((projectConfigBySolutionsConfigs[releaseConfig][_fileBasedWebService_ID].BuildConfiguration == releaseDotNETConfig), "For the given solution configuration, the projects configuration is not correct.");
                    // this project is NOT set to be built for both configurations so check that to..
                    Assert.IsFalse(projectConfigBySolutionsConfigs[debugAnyCpuConfig][_fileBasedWebService_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig not been set.");
                    Assert.IsFalse(projectConfigBySolutionsConfigs[releaseConfig][_fileBasedWebService_ID].Build, "The solution should have stated NOT to build this project however the build property on SolutionsProjectConfig not been set.");
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            [Test]
            public void EnumeratorTest()
            {
                try
                {
                    DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                    //
                    // test "Debug|Any CPU"
                    //
                    dummyWhidbeySolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug", "Any CPU"));

                    int projectToBuildCount = 0;
                    int totalProjects = 0;
                    foreach (IProjectInfo project in dummyWhidbeySolutionInfo)
                    {
                        if (project.Build)
                            projectToBuildCount++;

                        totalProjects++;
                    }

                    Assert.AreEqual(7, projectToBuildCount);
                    Assert.AreEqual(9, totalProjects);
                    //
                    // test "Release|Any CPU"
                    //
                    dummyWhidbeySolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Release", "Any CPU"));

                    projectToBuildCount = 0;
                    totalProjects = 0;

                    foreach (IProjectInfo project in dummyWhidbeySolutionInfo)
                    {
                        if (project.Build)
                            projectToBuildCount++;

                        totalProjects++;
                    }

                    Assert.AreEqual(6, projectToBuildCount);
                    Assert.AreEqual(9, totalProjects);
                }
                catch (Exception ex)
                {
                    Console.Write(ex);
                    throw;
                }
            }

            [Test]
            public void TestAbstractSolutionProjectCollectionsTest()
            {
                try
                {
                    DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                    //
                    // test "Debug|Any CPU"
                    //
                    dummyWhidbeySolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug", "Any CPU"));

                    Assert.AreEqual(9, dummyWhidbeySolutionInfo.ProjectsByName.Count);

                    //
                    // test "Release|Any CPU"
                    //
                    dummyWhidbeySolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Release", "Any CPU"));

                    Assert.AreEqual(9, dummyWhidbeySolutionInfo.ProjectsByName.Count);
                }
                catch (Exception ex)
                {
                    Console.Write(ex);
                    throw;
                }
            }

            [Test(), Ignore()]
            public void CreateProjectsTest_Isolated()
            {
                //IProjectInfo project;
                //Dictionary<String, IProjectInfo> projectsByName = new Dictionary<string,IProjectInfo>();
                //DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                //// snippet from the solution file were using for the test
                ////Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "ConsoleApplication1", "ConsoleApplication\ConsoleApplication1\ConsoleApplication1.csproj", "{D5720673-3207-4F4C-810A-C8E6F0838CF4}"
                ////EndProject
                ////Project("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}") = "WebService1", "AspNetWebApplication\WebService1\WebService1.csproj", "{0E075513-8FC8-4B05-9374-E74A374F05E1}"
                ////EndProject
                ////Project("{E24C65DC-7377-472B-9ABA-BC803B73C61A}") = "C:\...\FileSystemBasedWebSite\", "AspNetWebApplication\FileSystemBasedWebSite\", "{6A718792-82DF-4AD5-9FB8-B1EC47366CB1}"
                ////    ProjectSection(WebsiteProperties) = preProject
                ////        Debug.AspNetCompiler.VirtualPath = "/FileSystemBasedWebSite"
                ////        Debug.AspNetCompiler.PhysicalPath = "AspNetWebApplication\FileSystemBasedWebSite\"
                ////        Debug.AspNetCompiler.TargetPath = "PrecompiledWeb\FileSystemBasedWebSite\"
                ////        Debug.AspNetCompiler.Updateable = "true"
                ////        Debug.AspNetCompiler.ForceOverwrite = "true"
                ////        Debug.AspNetCompiler.FixedNames = "false"
                ////        Debug.AspNetCompiler.Debug = "True"
                ////        Release.AspNetCompiler.VirtualPath = "/FileSystemBasedWebSite"
                ////        Release.AspNetCompiler.PhysicalPath = "AspNetWebApplication\FileSystemBasedWebSite\"
                ////        Release.AspNetCompiler.TargetPath = "PrecompiledWeb\FileSystemBasedWebSite\"
                ////        Release.AspNetCompiler.Updateable = "true"
                ////        Release.AspNetCompiler.ForceOverwrite = "true"
                ////        Release.AspNetCompiler.FixedNames = "false"
                ////        Release.AspNetCompiler.Debug = "False"
                ////        VWDPort = "1097"
                ////        DefaultWebSiteLanguage = "Visual C#"
                ////    EndProjectSection
                ////EndProject
                

                //project = dummyWhidbeySolutionInfo.CreateProject(
                //    new Guid("{FAE04EC0-301F-11D3-BF4B-00C04F79EFBC}"), "ConsoleApplication1", new FileInfo(;
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyWhidbeySolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyWhidbeySolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //project = dummyWhidbeySolutionInfo.CreateProject();
                //projectsByName.Add(project.ProjectName, project);

                //CheckProjectsCreatedOk(projectsByName);
            }

            [Test()]
            public void CreateProjectsTest_IntergrationWithOtherClassInternals()
            {
                try
                {
                    DummyWhidbeySolutionInfo dummyWhidbeySolutionInfo = new DummyWhidbeySolutionInfo();

                    // setting the active configuration will trigger a load of the projects for the given configuration.
                    dummyWhidbeySolutionInfo.ConfigurationManager.SetActiveBuildConfigurationAndLoad(new BuildConfiguration("Debug", "Any CPU"));

                    CheckProjectsCreatedOk(dummyWhidbeySolutionInfo.ProjectsByName);
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex);
                    throw;
                }
            }

            private void CheckProjectsCreatedOk(Dictionary<String, IProjectInfo> projectsByName)
            {
                foreach (IProjectInfo project in projectsByName.Values)
                {
                    //
                    // check some of the IProjectProperties props
                    // We check IsNotNull for most and additionally 
                    // IsNotEmpty for strings.
                    //
                    Assert.IsNotNull(project.ProjectName);
                    Assert.IsNotEmpty(project.ProjectName);

                    Assert.IsNotNull(project.ID);

                    if (project.ProjectType != ProjectType.WebSite)
                    {
                        Assert.IsNotNull(project.ProjectFile);

                        Assert.IsNotNull(project.ProjectFileExtension);
                        Assert.IsNotEmpty(project.ProjectFileExtension);

                        Assert.IsNotNull(project.AssemblyName);
                        Assert.IsNotEmpty(project.AssemblyName);

                        Assert.IsNotNull(project.AssemblyFileName);
                        Assert.IsNotEmpty(project.AssemblyFileName);
                    }

                    Assert.IsNotNull(project.CodeFileExtension);
                    Assert.IsNotEmpty(project.CodeFileExtension);

                    Assert.IsNotNull(project.ProjectDir);

                    Assert.IsNotNull(project.ProjectLanguage);

                    Assert.IsNotNull(project.ProjectOutputType);

                    Assert.IsNotNull(project.ProjectType);

                    Assert.IsNotNull(project.ConfigFile);
                    Assert.IsNotEmpty(project.ConfigFile);

                    Assert.IsNotNull(project.DotNetFrameworkVersion);

                    Assert.IsNotNull(project.IProjectBuildProperties);

                    //
                    // check some of the IProjectBuildProperties props
                    // We check IsNotNull for most and additionally 
                    // IsNotEmpty for strings.
                    //

                    if (project.ProjectType != ProjectType.WebSite)
                    {
                        Assert.IsNotNull(project.IProjectBuildProperties.OutputPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.OutputPathFull);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputPathFull);

                        Assert.IsNotNull(project.IProjectBuildProperties.OutputFilePath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.OutputFilePath);
                    }
                    else
                    {
                        Assert.IsNotNull(project.IProjectBuildProperties.VirtualPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.VirtualPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.PhysicalPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.PhysicalPath);

                        Assert.IsNotNull(project.IProjectBuildProperties.TargetPath);
                        Assert.IsNotEmpty(project.IProjectBuildProperties.TargetPath);
                    }
                }            
            }
        }


#endif
    }
}
