using System;
using System.Collections.Generic;
using System.Text;

using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Tasks;
using NAnt.Core.Types;
using NAnt.Contrib.Tasks.Web;
using Microsoft.TeamFoundation.VersionControl.Client;
using System.Net;
using Microsoft.TeamFoundation.Client;

namespace Snak.Tasks
{
    /// <summary>
    /// 
    /// </summary>
    /// <remarks>Code bases on code from the VSTS plugins available at: http://vstsplugins.sourceforge.net/index.php/downloads/</remarks>
    [TaskName("tfsLabel")]
    public class TFSLabelTask : Task
    {
        private string _server;

        /// <summary>
        ///   The name or URL of the team foundation server.  For example http://vstsb2:8080 or vstsb2 if it
        ///   has already been registered on the machine.
        /// </summary>
        [TaskAttribute("server", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        private string _projectPath;

        /// <summary>
        ///   The path to the project in source control, for example $\VSTSPlugins
        /// </summary>
        [TaskAttribute("projectPath", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string ProjectPath
        {
            get { return _projectPath; }
            set { _projectPath = value; }
        }

        private string _username;

        /// <summary>
        ///   Username that should be used.  Domain cannot be placed here, rather in domain property.
        /// </summary>
        [TaskAttribute("username", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string Username
        {
            get { return _username; }
            set { _username = value; }
        }

        private string _password;

        /// <summary>
        ///   The password in clear test of the domain user to be used.
        /// </summary>
        [TaskAttribute("password", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string Password
        {
            get { return _password; }
            set { _password = value; }
        }

        private string _domain;

        /// <summary>
        ///  The domain of the user to be used.
        /// </summary>
        [TaskAttribute("domain", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string Domain
        {
            get { return _domain; }
            set { _domain = value; }
        }

        private string _labelText;

        /// <summary>
        ///  The label to apply
        /// </summary>
        /// <remarks>must be less than 64 characters, cannot end with a space or period, and cannot contain any of the following characters: "/:<>\|*?@ ---></remarks>
        [TaskAttribute("labelText", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string LabelText
        {
            get { return _labelText; }
            set { _labelText = value; }
        }

        private string _comment;

        /// <summary>
        /// The comment to apply with the label
        /// </summary>
        [TaskAttribute("comment", Required = true)]
        [StringValidator(AllowEmpty = false)]
        public string Comment
        {
            get { return _comment; }
            set { _comment = value; }
        }

        private TeamFoundationServer _teamFoundationServer = null;

        /// <summary>
        ///   locally cached instance of TeamFoundationServer.
        /// </summary>
        internal TeamFoundationServer TFS
        {
            get
            {
                if (null == _teamFoundationServer)
                {
                    _teamFoundationServer = new TeamFoundationServer(this.Server, this.Credentials);
                }
                return _teamFoundationServer;
            }
            set
            {
                _teamFoundationServer = value;
            }
        }

        private NetworkCredential _networkCredential;

        internal NetworkCredential Credentials
        {
            get
            {
                if (null == _networkCredential)
                {
                    if (Username != null && Password != null)
                    {
                        if (Domain != null)
                            _networkCredential = new NetworkCredential(Username, Password, Domain);
                        else
                            _networkCredential = new NetworkCredential(Username, Password);
                    }
                    else
                    {
                        _networkCredential = CredentialCache.DefaultNetworkCredentials;
                    }
                }
                return _networkCredential;
            }
            set
            {
                _networkCredential = value;
            }
        }

        private VersionControlServer _sourceControl;

        internal VersionControlServer SourceControl
        {
            get
            {
                if (null == _sourceControl)
                {
                    _sourceControl = (VersionControlServer)this.TFS.GetService(typeof(VersionControlServer));
                }
                return _sourceControl;
            }
            set
            {
                _sourceControl = value;
            }
        }

        protected override void ExecuteTask()
        {
            VersionControlLabel label = new VersionControlLabel(this.SourceControl, _labelText, this.SourceControl.AuthenticatedUser, this.ProjectPath, _comment);

            // Create Label Item Spec.
            ItemSpec itemSpec = new ItemSpec(this.ProjectPath, RecursionType.Full);
            LabelItemSpec[] labelItemSpec = new LabelItemSpec[] {  
                new LabelItemSpec(itemSpec, new DateVersionSpec(DateTime.Now), false)
            };

            this.SourceControl.CreateLabel(label, labelItemSpec, LabelChildOption.Replace);
        }
    }
}
